PROGRAM_NAME='Big checksums vs. the 16000 byte limit'
(***********************************************************)
(*  FILE CREATED ON: 08/05/2008  AT: 18:07:10              *)
(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 08/22/2012  AT: 13:22:55        *)
(***********************************************************)

#include 'DEV_TO_STRING'
#include 'file'
#include 'checksum'

(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE

vdvControl = 33000:1:0

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

VOLATILE CHAR BUFFER[3700000]
VOLATILE CHAR DIRECTORY[255]

VOLATILE CHAR Control_BUFFER[3700000]

SLONG slError
SLONG slHandle

LONG chksm

(***********************************************************)
(*           SUBROUTINE DEFINITIONS GO BELOW               *)
(***********************************************************)
(* EXAMPLE: DEFINE_FUNCTION <RETURN_TYPE> <NAME> (<PARAMETERS>) *)
(* EXAMPLE: DEFINE_CALL '<NAME>' (<PARAMETERS>) *)

DEFINE_FUNCTION LONG processBigFile_Version1 (DEV dvD, CHAR str[], LONG len)
{
  STACK_VAR LONG l

  l = LENGTH_ARRAY(str)

  IF (l <> len)
  {
	SEND_STRING dvD,"'ERROR: processed length ',ITOA(l),' <> original length ',ITOA(len)"
	RETURN $FFFFFFFF;
  }
  ELSE
  {
	SEND_STRING dvD,"ITOA(l),' bytes, calculating crc32...'"
	RETURN crc32(str)
  }
}

DEFINE_FUNCTION LONG processBigFile_Version2 (DEV dvD, CHAR str[16001], LONG len)
{
  STACK_VAR LONG l

  l = LENGTH_ARRAY(str)

  IF (l <> len)
  {
	SEND_STRING dvD,"'ERROR: processed length ',ITOA(l),' <> original length ',ITOA(len)"
	RETURN $FFFFFFFF;
  }
  ELSE
  {
	SEND_STRING dvD,"ITOA(l),' bytes, calculating crc32...'"
	RETURN crc32(str)
  }
}


(***********************************************************)
(*                 STARTUP CODE GOES BELOW                 *)
(***********************************************************)
DEFINE_START

CREATE_BUFFER vdvControl,Control_BUFFER
pwd(0:1:0,directory)				// set directory to the current working directory

(***********************************************************)
(*                  THE EVENTS GO BELOW                    *)
(***********************************************************)
DEFINE_EVENT

DATA_EVENT[vdvControl]
{
  ONLINE:
  {
	WAIT 51
	  SEND_COMMAND vdvControl,'HELP'
  }
  COMMAND:
  {
	STACK_VAR CHAR CMD[32]
	SEND_STRING 0,"'CMD: ',DATA.TEXT"

	IF (FIND_STRING(DATA.TEXT,'-',1)) { CMD = UPPER_STRING(REMOVE_STRING(DATA.TEXT,'-',1)) }
	ELSE                              { CMD = UPPER_STRING(LEFT_STRING(DATA.TEXT,MAX_LENGTH_ARRAY(CMD))) }

	SWITCH (CMD)
	{
	  CASE 'OPEN-':
	  {
		SEND_STRING 0,"'Opening ',DATA.TEXT"
		slError = FILE_OPEN("directory,'/',DATA.TEXT",FILE_READ_ONLY)
		printFileResult(0:1:0,"ITOA(__LINE__),' file open: '",slError,FALSE)

		IF (slError > 0)
		{
		  slHandle = slError
		  SEND_STRING 0,"'BYTES READ: ',ITOA(FILE_READ(slHandle,BUFFER,MAX_LENGTH_ARRAY(BUFFER)))"
		  FILE_CLOSE(slHandle)
		  SEND_STRING 0,"$09"

		  //#WARN 'Unbounded parameter: passed as "BUFFER" - buffer is truncated to 2048 bytes'
		  SEND_STRING 0,"'line ',ITOA(__line__),' Passing "BUFFER" ',ITOA(LENGTH_ARRAY(BUFFER)),' bytes via unbounded parameter'"
		  chksm = processBigFile_Version1(0:1:0,"BUFFER",LENGTH_ARRAY(BUFFER))
		  SEND_STRING 0,"'Test 1a crc32 = 0x',ITOHEX(chksm)"
		  SEND_STRING 0,"$09"

		  //#WARN 'Unbounded parameter: no "" around BUFFER - no truncation'
		  SEND_STRING 0,"'line ',ITOA(__line__),' Passing BUFFER ',ITOA(LENGTH_ARRAY(BUFFER)),' bytes via unbounded parameter'"
		  chksm = processBigFile_Version1(0:1:0,BUFFER,LENGTH_ARRAY(BUFFER))
		  SEND_STRING 0,"'Test 1b crc32 = 0x',ITOHEX(chksm)"
		  SEND_STRING 0,"$09"

		  //#WARN '15999+ byte parameter: passed as "BUFFER" - buffer is truncated to 15999 bytes'
		  SEND_STRING 0,"'line ',ITOA(__line__),' Passing "BUFFER" ',ITOA(LENGTH_ARRAY(BUFFER)),' bytes via 16001 byte parameter'"
		  chksm = processBigFile_Version2(0:1:0,"BUFFER",LENGTH_ARRAY(BUFFER))
		  SEND_STRING 0,"'Test 2a crc32 = 0x',ITOHEX(chksm)"
		  SEND_STRING 0,"$09"

		  //#WARN '15999+ byte parameter: no "" around BUFFER - no truncation'
  		  SEND_STRING 0,"'line ',ITOA(__line__),' Passing BUFFER ',ITOA(LENGTH_ARRAY(BUFFER)),' bytes via 16001 byte parameter'"
		  chksm = processBigFile_Version2(0,BUFFER,LENGTH_ARRAY(BUFFER))
		  SEND_STRING 0,"'Test 2b crc32 = 0x',ITOHEX(chksm)"
		  SEND_STRING 0,"$09"

		  //#WARN '15999+ byte-by-byte copy/concatenation - no truncation'
		  SEND_STRING 0,"'line ',ITOA(__line__),' Making byte-by-byte copy of BUFFER ',ITOA(LENGTH_ARRAY(BUFFER)),' bytes...'"
		  {// local scope for variables
			STACK_VAR LONG i
			STACK_VAR CHAR BUFFER_COPY[3700000]

			FOR (i = LENGTH_ARRAY(BUFFER); i; i--)
			{
			  BUFFER_COPY[i] = BUFFER[i];
			}
			SET_LENGTH_ARRAY(BUFFER_COPY,LENGTH_ARRAY(BUFFER))
			chksm = processBigFile_Version2(0:1:0,BUFFER_COPY,LENGTH_ARRAY(BUFFER_COPY))
			SEND_STRING 0,"'Test 3 crc32 = 0x',ITOHEX(chksm)"
		  }// end local scope for variables
		  SEND_STRING 0,"$09"

		  SEND_STRING 0,"'line ',ITOA(__line__),' Concatenation example using ????_STRING for the bulk of ',ITOA(LENGTH_ARRAY(BUFFER)),' then byte-by-byte for the last 2008 bytes...'"
		  {// local scope for variables
			STACK_VAR LONG i
			STACK_VAR LONG j
			STACK_VAR CHAR BUFFER_COPY[3700000]
			STACK_VAR CHAR new_data[2008]

									// In real world use, MID_STRING or RIGHT_STRING would be used to get the right side of the buffer
			BUFFER_COPY = LEFT_STRING(BUFFER,LENGTH_ARRAY(BUFFER)-2008)
			new_data = RIGHT_STRING(BUFFER,2008)
			FOR (i = 2008, j = LENGTH_ARRAY(BUFFER); i; i--, j--)
			{
			  BUFFER_COPY[j] = new_data[i];
			}
			SET_LENGTH_ARRAY(BUFFER_COPY,LENGTH_ARRAY(BUFFER))
			chksm = processBigFile_Version2(0:1:0,BUFFER_COPY,LENGTH_ARRAY(BUFFER_COPY))
			SEND_STRING 0,"'Test 3 crc32 = 0x',ITOHEX(chksm)"
		  }// end local scope for variables
		  SEND_STRING 0,"$09"

		  // SEND_STRING to vdv 2008 bytes at a time
		  SEND_STRING 0,"'line ',ITOA(__line__),' Reopening ',DATA.TEXT,' to read 2008 bytes at a time to buffered ',
		                  DEV_TO_STRING(vdvControl),'...'"
		  CLEAR_BUFFER Control_BUFFER
		  								    // open file and print the result
		  slError = printFileResult(0:1:0,"ITOA(__line__),' FILE_OPEN '",FILE_OPEN("directory,'/',DATA.TEXT",FILE_READ_ONLY),TRUE)
		  IF (slError > 0)
		  {
		                            // local_var can be changed in NetLinx Studio debug, for testing
									// the FILE_READ limit appears to be 31312 - set it higher, and the test will fail.
									// Values higher than 2008 get split into multiple packets, e.g. 2008 bytes, 4bytes, etc.
			LOCAL_VAR VOLATILE LONG bytes_to_read
			STACK_VAR CHAR sBuf[32000]
			IF (bytes_to_read = 0)                           { bytes_to_read = 2008 }
			ELSE IF (bytes_to_read > MAX_LENGTH_ARRAY(sBuf)) { bytes_to_read = MAX_LENGTH_ARRAY(sBuf) }

			slHandle = slError
			WHILE (slError > 0)
			{
			  slError = FILE_READ(slHandle,sBuf,bytes_to_read)
			  SEND_STRING vdvControl,sBuf
			}
									  // print the final file_read result
			printFileResult(0:1:0,"ITOA(__line__),' FILE_READ '",slError,TRUE)
									  // close file and print the result
			slError = printFileResult(0:1:0,"ITOA(__line__),' FILE_CLOSE '",FILE_CLOSE(slHandle),TRUE)
		  }
		}
	  }
									// change current directory, thenb
									// print the new directory
	  CASE 'PWD-':
	  {
		cd(0:1:0,directory,DATA.TEXT)
		send_string 0,"'directory = ',directory"
	  }
									// print working directory
	  CASE 'PWD':
	  {
		pwd(0:1:0,directory)
	  }
	  								// change current directory, then
									// List contents of new directory
	  CASE 'LS-':
	  {
		cd(0:1:0,directory,DATA.TEXT)
		ls(0:1:0,directory)
	  }
									// List contents of current directory
	  CASE 'LS':
	  {
		ls(0:1:0,directory)
	  }
									// search for file in current directory
	  CASE 'DIR-':
	  {
		SEND_STRING 0,"'dir result = ',ITOA(dir(0:1:0,directory,DATA.TEXT))"
	  }
									// change current directory
	  CASE 'CD-':
	  {
		cd(0:1:0,directory,DATA.TEXT)
		send_string 0,"'directory = ',directory"
	  }

	  CASE 'HELP':
	  {
	    SEND_STRING 0,"DEV_TO_STRING(DATA.DEVICE),' -- Commands Available:'"
		SEND_STRING 0,"'Display Help screen:'"
		SEND_STRING 0,"'  HELP'"
		SEND_STRING 0,"$09"
		SEND_STRING 0,"'Print Working Directory:'"
		SEND_STRING 0,"'  PWD'"
		SEND_STRING 0,"'Change Working Directory, then PWD new directory:'"
		SEND_STRING 0,"'  PWD-<new directory>'"
		SEND_STRING 0,"'or'"
		SEND_STRING 0,"'  CD-<new directory>'"
		SEND_STRING 0,"$09"
		SEND_STRING 0,"'List contents of current directory'"
		SEND_STRING 0,"'  LS'"
		SEND_STRING 0,"'Change Working Directory, then LS new directory'"
		SEND_STRING 0,"'  LS-<new directory>'"
		SEND_STRING 0,"$09"
		SEND_STRING 0,"'Open a file to checksum/crc test'"
		SEND_STRING 0,"'  OPEN-<filepath>'"
	  }
	}
  }
  STRING:
  {
	CANCEL_WAIT 'CREATE_BUFFER test'
	IF (LENGTH_ARRAY(Control_BUFFER) == LENGTH_ARRAY(BUFFER))
	{
	  SEND_STRING 0,"'OK len = ',itoa(LENGTH_ARRAY(Control_BUFFER)),' expected = ',itoa(LENGTH_ARRAY(BUFFER))"
	  chksm = processBigFile_Version2(0:1:0,Control_BUFFER,LENGTH_ARRAY(Control_BUFFER))
	  SEND_STRING 0,"'Test 4 crc32 = 0x',ITOHEX(chksm)"
	  SEND_STRING 0,"$09"

	  CLEAR_BUFFER Control_BUFFER
	}
	ELSE
	{
	  send_string 0,"'len = ',itoa(LENGTH_ARRAY(Control_BUFFER)),' expected = ',itoa(LENGTH_ARRAY(BUFFER)),'...'"
	  WAIT 300 'CREATE_BUFFER test'
	  {
		SEND_STRING 0,"'ERROR: processed length ',ITOA(LENGTH_ARRAY(Control_BUFFER)),' <> original length ',ITOA(LENGTH_ARRAY(BUFFER))"
		SEND_STRING 0,"'Test 4 crc32 = 0xFFFFFAIL'"
		SEND_STRING 0,"$09"
		CLEAR_BUFFER Control_BUFFER
	  }
	}
  }
}

(***********************************************************)
(*                     END OF PROGRAM                      *)
(*        DO NOT PUT ANY CODE BELOW THIS COMMENT           *)
(***********************************************************)
