PROGRAM_NAME='DEV_TO_STRING'
(***********************************************************)
(*  FILE CREATED ON: 09/08/2008  AT: 17:06:51              *)
(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 12/27/2022  AT: 11:45:16        *)
(***********************************************************)
/*
 * tab-width: 2
 * indent-level: 1
 *
 */
DEFINE_PROGRAM                      // rev notes
(***********************************************************)
(*  FILE REVISION: 4.01                                    *)
(*  REVISION DATE: 12/27/2022  AT: 11:45:16                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Clean up master version check for AMX_LOG().         *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 4.00                                    *)
(*  REVISION DATE: 08/18/2021  AT: 14:19:15                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Use AMX_LOG() when master supports it.               *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 3.09                                    *)
(*  REVISION DATE: 05/17/2017  AT: 15:14:34                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - printDevice_Info() should now print '' for blank     *)
(*    NetLinx serial numbers, instead of 0000000000000000. *)
(*  - printDevice_Info() now prints single-quotes around   *)
(*    strings, e.g. ...10003:1:8 Device 'MXT-1001i'.       *)
(*  - Other libs used:                                     *)
(*           statistics.axi v0.17                          *)
(*           strings.axi v2.15                             *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 3.08                                    *)
(*  REVISION DATE: 07/22/2016  AT: 17:57:24                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Return DEVICE_ID_STRING along with the version from  *)
(*    getMasterVersion().                                  *)
(*  - Moved getMasterVersion() here from debug.axi v3.17.  *)
(*  - Other libs used:                                     *)
(*           statistics.axi v0.17                          *)
(*           strings.axi v2.14                             *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 3.07/3.06                               *)
(*  REVISION DATE: 02/15/2016  AT: 17:03:15                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Changed getXREF_INDEX(), getXREF_DEV(), and          *)
(*    getXREF_DEV_INDEX() to return their array length,    *)
(*     and update the array as an affected parameter.      *)
(*  - For commands that accept mixed lists of D:P:S and    *)
(*    dv indexes as a parameter: Added support for ranges  *)
(*    denoted by <n1>*<n2> to getXREF_INDEX(),             *)
(*    getXREF_DEV(), and getXREF_DEV_INDEX().              *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 3.05                                    *)
(*  REVISION DATE: 09/30/2015  AT: 12:38:01                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added BMP_DEVICE_INFO_FORCE flag to print device     *)
(*    info even when DEVICE_ID(<d:p:s>) returns 0.         *)
(*    This feature was added to confirm that all           *)
(*    device_info is blank if DEVICE_ID() is blank.        *)
(*  - Added DEFINE_PROGRAM to top of Rev Notes to enable   *)
(*    code folding the notes without hiding the timestamp. *)
(*  - Added 'Protocol-File' to _sDUET_STUB_PROPERTIES,     *)
(*    needed for XDD aka DeviceDriver aka Duet2 modules.   *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 3.04/3.03                               *)
(*  REVISION DATE: 03/13/2015  AT: 06:44:31                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Moved functions getDEVICE_INFO_BITMAP() and          *)
(*    printDEVICE_INFO() here from debug.axi.              *)
(*  - printDEVICE_INFO() now accepts parameters that will  *)
(*    filter the printed info.                             *)
(*  - Added functions getXREF_DEV(), getXREF_INDEX(), and  *)
(*    getXREF_DEV_INDEX().                                 *)
(*  - Added _sBMP_DUET_STUB_PROPERTIES structure to store  *)
(*    stub data, DUET_RESET_STUB_PROPERTIES() to clear it. *)
(*  - Added a return value to DUET_PARSE_STUB_PROPERTY().  *)
(*  - DUET_QUERY_STUB_PROPERTIES() now accepts a flags     *)
(*    parameter that will filter the queries sent.         *)
(*  - Added DUET_GET_QUERY_BITMAP() to create those flags. *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 3.02                                    *)
(*  REVISION DATE: 03/03/2014  AT: 11:09:33                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added functions DUET_PARSE_STUB_PROPERTY() and       *)
(*    DUET_QUERY_STUB_PROPERTIES().                        *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 3.01                                    *)
(*  REVISION DATE: 08/09/2012  AT: 11:39:11                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added function STRING_TO_DEV_ARRAY().                *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 3.00                                    *)
(*  REVISION DATE: 05/14/2012  AT: 17:04:01                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added function STRING_TO_DEV().                      *)
(*  - DPS_TO_STRING() and 'STRING TO DEV' are now just     *)
(*    wrappers.                                            *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.55                                    *)
(*  REVISION DATE: 11/08/2008  AT: 13:02:55                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Speed Tweaks:                                        *)
(*    * Use ITOA() instead of FORMAT() if 0 padding isn't  *)
(*      necessary.                                         *)
(*    * In control expressions use IF (var) instead of     *)
(*      IF (var > 0) or IF (var < value).                  *)
(*      For example, reverse FOR loops.                    *)
(*  - Moved function descriptions below DEFINE_FUNCTION    *)
(*    so they will be visible when code is folded above    *)
(*    the function.                                        *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.54                                    *)
(*  REVISION DATE: 10/06/2008  AT: 07:17:07                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added __display_lib_version__ variable.              *)
(*                                                         *)
(***********************************************************)
(*!!FILE REVISION: 2.53 GLM                                *)
(*  REVISION DATE: 09/08/2008                              *)
(*  COMMENTS:                                              *)
(*  - DEV_TO_STRING subroutines now in seperate lib from   *)
(*    debug subroutines. Debug.axi includes this lib -     *)
(*    but now the DEV_TO_STRING subroutines can be used    *)
(*    seperately, if desired.                              *)
(*                                                         *)
(* DEV_TO_STRING FUNCTIONS AND CALLS                       *)
/*  1   FUNCTION CHAR[17] DEV_TO_STRING (DEV dvDEV)                           */
/*  2   FUNCTION CHAR[2303] DEV_ARRAY_TO_STRING (DEV dvDA[])                  */
/*  3   FUNCTION CHAR[25] DEVCHAN_TO_STRING (DEVCHAN dcDC)                    */
/*  4   FUNCTION CHAR[25] DEV_CHAN_TO_STRING (DEV dvDEV, INTEGER nCH)         */
/*  5   FUNCTION CHAR[25] DEVLEV_TO_STRING (DEVLEV dlDL)                      */
/*  6   FUNCTION CHAR[25] DEV_LEV_TO_STRING (DEV dvDEV, INTEGER nLV)          */
/*  7   FUNCTION DUET_PARSE_STUB_PROPERTY (CHAR txt[], CHAR duet_props)       */
/*  8   FUNCTION DUET_QUERY_STUB_PROPERTIES (DEV dvDev, INTEGER flags)        */
/*  9   FUNCTION DUET_RESET_STUB_PROPERTIES (_sBMP_DUET_STUB_PROPERTIES uStub, INTEGER flags) */
/* 10   FUNCTION INTEGER getDEVICE_INFO_BITMAP (CHAR args[])                  */
/* 11   FUNCTION INTEGER getDUET_QUERY_BITMAP (CHAR args[])                   */
/* 12   FUNCTION CHAR[32] getMasterVersion(DEV dvMaster)                      */
/* 13   FUNCTION INTEGER getXREF_DEV (CHAR DATA[], DEV dv[], INTEGER ret[])   */
/* 14   FUNCTION INTEGER getXREF_DEV_INDEX (CHAR DATA[], DEV dv[], INTEGER ndx[], INTEGER ret[] */
/* 15   FUNCTION INTEGER getXREF_INDEX (CHAR DATA[], INTEGER ndx[], INTEGER ret[] */
/* 16   FUNCTION printDEVICE_INFO(CHAR hdr[], DEV dvDEV, INTEGER flags)       */
/* 17   FUNCTION          STRING_TO_DEV (sDEV[],dvDEV)                        */
/* 18   FUNCTION          STRING_TO_DEV_ARRAY (sDEV[],dvDA[])                 */
/* 19   FUNCTION CHAR[17] DPS_TO_STRING (DEV dvDEV)                           */
/* 20   CALL 'STRING TO DEV' (sDEV[],dvDEV)                                   */
(**********************************************************)
#IF_NOT_DEFINED __DEV_TO_STRING_LIB__
#DEFINE __DEV_TO_STRING_LIB__

DEFINE_VARIABLE                     // #INCLUDE
#INCLUDE 'statistics'
#INCLUDE 'strings'
(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE

(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

// VERSION FOR CODE

CHAR __DEV_TO_STRING_LIB_NAME__[20]  = '   DEV_TO_STRING.axi';
CHAR __DEV_TO_STRING_LIB_VERSION__[] = '4.01';

// DEVICE_INFO() bitmap values
BMP_DEVICE_INFO_MANUFACTURER_STRING = $0001 // i.e. make
BMP_DEVICE_INFO_MANUFACTURER_ID     = $0002
BMP_DEVICE_INFO_DEVICE_ID_STRING    = $0004 // i.e. model name
BMP_DEVICE_INFO_DEVICE_ID           = $0008
BMP_DEVICE_INFO_VERSION             = $0010
BMP_DEVICE_INFO_FIRMWARE_ID         = $0020
BMP_DEVICE_INFO_SERIAL_NUMBER       = $0040
BMP_DEVICE_INFO_SOURCE_TYPE         = $0080
BMP_DEVICE_INFO_SOURCE_STRING       = $0100
BMP_DEVICE_INFO_FORCE								= $8000 // print data even when DEVICE_ID(<d:p:s>) == 0

// Duet Stub Properties bitmap values
BMP_DUET_STUB_PHYSICAL_DEVICE = $0001
BMP_DUET_STUB_DUET_DEVICE     = $0002
BMP_DUET_STUB_DUET_MODULE     = $0004
BMP_DUET_STUB_BUNDLE_VERSION  = $0008
BMP_DUET_STUB_DEVICE_CATEGORY = $0010
BMP_DUET_STUB_DEVICE_MAKE     = $0020 // i.e. Manufacturer String
BMP_DUET_STUB_DEVICE_MODEL    = $0040 // i.e. DEVICE_ID_STRING
BMP_DUET_STUB_DEVICE_SDKCLASS = $0080
BMP_DUET_STUB_DEVICE_REVISION = $0100
BMP_DUET_STUB_PROTOCOL_FILE   = $0200

DEFINE_TYPE
(***********************************************************)
(*              STRUCTURE DEFINITIONS GO BELOW             *)
(***********************************************************)

STRUCT _sDUET_STUB_PROPERTIES {
 CHAR _1_PhysicalDevice[36]          // 1 'Physical-Device=[ 5 ].[ 5 ].[ 5 ]'
 CHAR _2_DuetDevice[36]              // 2 'Duet-Device=[ 5 ].[ 5 ].[ 5 ]'
 CHAR _3_DuetModule[84]              // 3 'Duet-Module=<name>'
 CHAR _4_BundleVersion[32]           // 4 'Bundle-Version=[3].[3].[3]'
 CHAR _5_DeviceCategory[64]          // 5 'Device-Category=ip,serial,rs-232,???'
 CHAR _6_DeviceMake[64]              // 6 'Device-Make=<name>'
 CHAR _7_DeviceModel[300]            // 7 'Device-Model=<name-1[,name-2...name-n]>'
 CHAR _8_DeviceSdkClass[72]          // 8 'Device-SDKClass=<java class name>'
 CHAR _9_DeviceRevision[32]          // 9 'Device-Revision=[3].[3].[3]'
 CHAR _10_ProtocolFile[128]          // 10 'Protocol-File=<name>'
}

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

#IF_NOT_DEFINED __display_lib_version__
VOLATILE __display_lib_version__ = TRUE
#END_IF

#IF_NOT_DEFINED __V4_or_NX_master__
VOLATILE INTEGER __V4_or_NX_master__ = 0
#END_IF

#IF_NOT_DEFINED __V4_and_NX_check__
VOLATILE INTEGER __V4_and_NX_check__ = 0
#END_IF

(***********************************************************)
(*           SUBROUTINE DEFINITIONS GO BELOW               *)
(***********************************************************)
(* EXAMPLE: DEFINE_FUNCTION <RETURN_TYPE> <NAME> (<PARAMETERS>) *)
(* EXAMPLE: DEFINE_CALL '<NAME>' (<PARAMETERS>) *)
(***********************************************************)
(*           SUBROUTINE DEFINITIONS GO BELOW               *)
(***********************************************************)
(* EXAMPLE: DEFINE_FUNCTION <RETURN_TYPE> <NAME> (<PARAMETERS>) *)
(* EXAMPLE: DEFINE_CALL '<NAME>' (<PARAMETERS>) *)

DEFINE_FUNCTION CHAR[17] DEV_TO_STRING (DEV dvDEV)
// DEFINE_FUNCTION DEV_TO_STRING
// PARAMETER:
// dvDEV - DEVICE USED
// RETURNS:
// dvDEV AS A STRING
{
  IF (dvDEV.SYSTEM)
	RETURN "ITOA(dvDEV.NUMBER),':',ITOA(dvDEV.PORT),':',ITOA(dvDEV.SYSTEM)";
  ELSE                    // if device is defined as local, i.e. system is 0
    RETURN "ITOA(dvDEV.NUMBER),':',ITOA(dvDEV.PORT),':',ITOA(SYSTEM_NUMBER)";
}

DEFINE_FUNCTION CHAR[2303] DEV_ARRAY_TO_STRING (DEV dvDA[])
// DEFINE_FUNCTION DEV_ARRAY_TO_STRING
// PARAMETER:
// dvDA - DEVICE ARRAY USED
// RETURNS:
// dvDA AS A STRING, comma seperated
{
  STACK_VAR INTEGER i;
  STACK_VAR INTEGER len;
  STACK_VAR CHAR    ASCII[2303];     // 2303 = ((17 + 1) * 128) - 1

  len = LENGTH_ARRAY(dvDA);

  IF (len > 0)
  {
	FOR (i = len; i; i--)
	{
	  IF (i = len)
	  {
		ASCII = "DEV_TO_STRING(dvDA[i])";
	  }
	  ELSE
	  {
		ASCII = "DEV_TO_STRING(dvDA[i]),',',ASCII";
	  }
	}
  }

  RETURN ASCII;
}

DEFINE_FUNCTION CHAR[25] DEVCHAN_TO_STRING (DEVCHAN dcDC)
// DEFINE_FUNCTION DEVCHAN_TO_STRING
// PARAMETER:
// dcDC - DEVCHAN USED
// RETURNS:
// DEVCHAN AS A STRING, IN BRACKETS
{
  IF (dcDC.DEVICE.SYSTEM = 0)       // if device is defined as local, i.e. system is 0
  {
	RETURN "'[',
			ITOA(dcDC.DEVICE.NUMBER),':',
			ITOA(dcDC.DEVICE.PORT),':',
			ITOA(SYSTEM_NUMBER),
			',',
			ITOA(dcDC.CHANNEL),
			']'";
  }
  ELSE
  {
	RETURN "'[',
			ITOA(dcDC.DEVICE.NUMBER),':',
			ITOA(dcDC.DEVICE.PORT),':',
			ITOA(dcDC.DEVICE.SYSTEM),
			',',
			ITOA(dcDC.CHANNEL),
			']'";
  }
}
DEFINE_FUNCTION CHAR[25] DEV_CHAN_TO_STRING (DEV dvDEV, INTEGER nCH)
// DEFINE_FUNCTION DEV_CHAN_TO_STRING
// PARAMETER:
// dvDEV - DEV USED
// nCH   - CHAN USED
// RETURNS:
// DEVCHAN AS A STRING, IN BRACKETS
{
  IF (dvDEV.SYSTEM = 0)             // if device is defined as local, i.e. system is 0
  {
	RETURN "'[',
			ITOA(dvDEV.NUMBER),':',
			ITOA(dvDEV.PORT),':',
			ITOA(SYSTEM_NUMBER),
			',',
			ITOA(nCH),
			']'";
  }
  ELSE
  {
  	RETURN "'[',
			ITOA(dvDEV.NUMBER),':',
			ITOA(dvDEV.PORT),':',
			ITOA(dvDEV.SYSTEM),
			',',
			ITOA(nCH),
			']'";
  }
}

DEFINE_FUNCTION CHAR[25] DEVLEV_TO_STRING (DEVLEV dlDL)
// DEFINE_FUNCTION DEVLEV_TO_STRING
// PARAMETER:
// dlDL - DEVLEV USED
// RETURNS:
// DEVLEV AS A STRING
{
  IF (dlDL.DEVICE.SYSTEM = 0)       // if device is defined as local, i.e. system is 0
  {
	RETURN "ITOA(dlDL.DEVICE.NUMBER),':',
			ITOA(dlDL.DEVICE.PORT),':',
			ITOA(SYSTEM_NUMBER),
			' L',
			FORMAT('%03d',dlDL.LEVEL)";
  }
  ELSE
  {
	RETURN "ITOA(dlDL.DEVICE.NUMBER),':',
			ITOA(dlDL.DEVICE.PORT),':',
			ITOA(dlDL.DEVICE.SYSTEM),
			' L',
			FORMAT('%03d',dlDL.LEVEL)";
  }
}
DEFINE_FUNCTION CHAR[25] DEV_LEV_TO_STRING (DEV dvDEV, INTEGER nLV)
// DEFINE_FUNCTION DEV_LEV_TO_STRING
// PARAMETER:
// dvDEV - DEV USED
// nLV   - LEV USED
// RETURNS:
// DEVLEV AS A STRING
{
  IF (dvDEV.SYSTEM = 0)             // if device is defined as local, i.e. system is 0
  {
	RETURN "ITOA(dvDEV.NUMBER),':',
			ITOA(dvDEV.PORT),':',
			ITOA(SYSTEM_NUMBER),
			' L',
			FORMAT('%03d',nLV)";
  }
  ELSE
  {
	RETURN "ITOA(dvDEV.NUMBER),':',
			ITOA(dvDEV.PORT),':',
			ITOA(dvDEV.SYSTEM),
			' L',
			FORMAT('%03d',nLV)";
  }
}

DEFINE_FUNCTION INTEGER DUET_PARSE_STUB_PROPERTY (CHAR txt[], CHAR duet_props[][])
/*
 * DEFINE_FUNCTION DUET_PARSE_STUB_PROPERTY()
 * Purpose: Format response from a ?PROPERTY query of an essential DUET property
 *          to the correct format for the DUET_PROPERTIES array used by LOAD_DUET_MODULE()
 *          and UNLOAD_DUET_MODULE(), and load the property into the array
 *          E.g. 'PROPERTY-Device-Category,"serial,rs-232"' or
 *                        'Device-Category,"serial,rs-232"' to
 *                        'Device-Category=serial,rs-232' in duet_props[5]
 * Parameter:
 *   txt - response from '?PROPERTY-' query, raw or with 'PROPERTY-' removed i.e.:
 *         'PROPERTY-<prop name>,<data>' or
 *         '<prop name>,<data>'
 *   duet_props - 2dim array of duet properties
 * Return:
 *   index into BMP_DUET_STUB_PROPERTIES array
 */
{
 STACK_VAR CHAR sProp[32]

 REMOVE_STRING(txt,'PROPERTY-',1)   // remove 'PROPERTY-' from txt, if it's there
 sProp = REMOVE_STRING(txt,',',1)   // get property name
																		// remove trailing comma from property name
 SET_LENGTH_ARRAY(sProp,LENGTH_ARRAY(sProp)-1)
 IF (LENGTH_ARRAY(txt)) {           // remove enclosing "" from property data, i.e. "<dat>" becomes <dat>
	IF (txt[1] == '"') { GET_BUFFER_CHAR(txt) }
	IF (txt[LENGTH_ARRAY(txt)] == '"' ) {
	 SET_LENGTH_ARRAY(txt,LENGTH_ARRAY(txt)-1)
	}
 }

 SWITCH (LEFT_STRING(sProp,11))
 {
	// 'Physical-Device'
	CASE 'Physical-De': { duet_props[1] = "sProp,'=',txt"; RETURN 1; }
	CASE 'Duet-Device': { duet_props[2] = "sProp,'=',txt"; RETURN 2; }
	CASE 'Duet-Module': { duet_props[3] = "sProp,'=',txt"; RETURN 3; }
	// 'Bundle-Version'
	CASE 'Bundle-Vers': { duet_props[4] = "sProp,'=',txt"; RETURN 4; }
	// 'Device-Category'
	CASE 'Device-Cate': { duet_props[5] = "sProp,'=',txt"; RETURN 5; }
	CASE 'Device-Make': { duet_props[6] = "sProp,'=',txt"; RETURN 6; }
	// 'Device-Model'
	CASE 'Device-Mode': { duet_props[7] = "sProp,'=',txt"; RETURN 7; }
	// 'Device-SDKClass'
	CASE 'Device-SDKC': { duet_props[8] = "sProp,'=',txt"; RETURN 8; }
	// 'Device-Revision'
	CASE 'Device-Revi': { duet_props[9] = "sProp,'=',txt"; RETURN 9; }
	// 'Protocol-File'
	CASE 'Protocol-Fi': { duet_props[10] = "sProp,'=',txt"; RETURN 10; }
	DEFAULT:            { RETURN 0; }
 }
}

DEFINE_FUNCTION DUET_QUERY_STUB_PROPERTIES (DEV dvDev, INTEGER flags)
/*
 * DEFINE_FUNCTION DUET_QUERY_STUB_PROPERTIES()
 * Purpose: Poll a DUET d:p:s for its essential DUET properties
 * These are needed by LOAD_DUET_MODULE() and UNLOAD_DUET_MODULE()
 *
 * Parameter:
 *   dvDev - D:P:S to poll
 *   flags - bitmap of properties to poll
 */
{
 IF (flags & BMP_DUET_STUB_PHYSICAL_DEVICE) { SEND_COMMAND dvDev,"'?PROPERTY-Physical-Device'" }
 IF (flags & BMP_DUET_STUB_DUET_DEVICE)     { SEND_COMMAND dvDev,"'?PROPERTY-Duet-Device'" }
 IF (flags & BMP_DUET_STUB_DUET_MODULE)     { SEND_COMMAND dvDev,"'?PROPERTY-Duet-Module'" }
 IF (flags & BMP_DUET_STUB_BUNDLE_VERSION)  { SEND_COMMAND dvDev,"'?PROPERTY-Bundle-Version'" }
 IF (flags & BMP_DUET_STUB_DEVICE_CATEGORY) { SEND_COMMAND dvDev,"'?PROPERTY-Device-Category'" }
 IF (flags & BMP_DUET_STUB_DEVICE_MAKE)     { SEND_COMMAND dvDev,"'?PROPERTY-Device-Make'"  }
 IF (flags & BMP_DUET_STUB_DEVICE_MODEL)    { SEND_COMMAND dvDev,"'?PROPERTY-Device-Model'" }
 IF (flags & BMP_DUET_STUB_DEVICE_SDKCLASS) { SEND_COMMAND dvDev,"'?PROPERTY-Device-SDKClass'" }
 IF (flags & BMP_DUET_STUB_DEVICE_REVISION) { SEND_COMMAND dvDev,"'?PROPERTY-Device-Revision'" }
 IF (flags & BMP_DUET_STUB_PROTOCOL_FILE)   { SEND_COMMAND dvDev,"'?PROPERTY-Protocol-File'" }
}

DEFINE_FUNCTION DUET_RESET_STUB_PROPERTIES (_sDUET_STUB_PROPERTIES uStub, INTEGER flags)
/*
 * DEFINE_FUNCTION DUET_PARSE_STUB_PROPERTY()
 * Purpose: clear one or more saved properties in a _sBMP_DUET_STUB_PROPERTIES structure
 *          before polling for new values
 *
 * Parameter:
 *   uStub - struct with saved data
 *   flags - bitmap of properties to clear
 */
{
 IF (flags & BMP_DUET_STUB_PHYSICAL_DEVICE) { uStub._1_PhysicalDevice = '' }
 IF (flags & BMP_DUET_STUB_DUET_DEVICE)     { uStub._2_DuetDevice = '' }
 IF (flags & BMP_DUET_STUB_DUET_MODULE)     { uStub._3_DuetModule = '' }
 IF (flags & BMP_DUET_STUB_BUNDLE_VERSION)  { uStub._4_BundleVersion = '' }
 IF (flags & BMP_DUET_STUB_DEVICE_CATEGORY) { uStub._5_DeviceCategory = '' }
 IF (flags & BMP_DUET_STUB_DEVICE_MAKE)     { uStub._6_DeviceMake = '' }
 IF (flags & BMP_DUET_STUB_DEVICE_MODEL)    { uStub._7_DeviceModel = '' }
 IF (flags & BMP_DUET_STUB_DEVICE_SDKCLASS) { uStub._8_DeviceSdkClass = '' }
 IF (flags & BMP_DUET_STUB_DEVICE_REVISION) { uStub._9_DeviceRevision = '' }
 IF (flags & BMP_DUET_STUB_PROTOCOL_FILE)   { uStub._10_ProtocolFile = '' }
}

DEFINE_FUNCTION INTEGER getDEVICE_INFO_BITMAP (CHAR args[])
/*
 * DEFINE_FUNCTION DUET_GET_QUERY_BITMAP()
 * Purpose: Create the flags for printDEVICE_INFO() from a comma-delimited
 *          string. This is done in a seperate function so if printDEVICE_INFO
 *          is run in a loop, there is no nested loop to create the flags.
 * Parameter:
 *   args - comma delimited array of properties to poll
 *          If args is empty, all DEVICE_INFO will be printed
 */
{
 STACK_VAR INTEGER flags

 IF (LENGTH_ARRAY(args)) {
	STACK_VAR CHAR sArgs[16][32]
	STACK_VAR INTEGER i
	flags = $0000
	i = stringTokenizer(args,',',sArgs)
	IF (i) {
	 FOR (; i; i--) {
		SWITCH (UPPER_STRING(sArgs[i]))
		{
		 CASE 'MANUFACTURER_STRING':
		 CASE 'MAKE':
		 CASE 'MAN':
		 CASE 'MAK':
		 CASE 'MK':
		 CASE 'DK': // Device maKe

			flags = flags ^ BMP_DEVICE_INFO_MANUFACTURER_STRING
		 CASE 'MANUFACTURER':
		 CASE 'MI':
			flags = flags ^ BMP_DEVICE_INFO_MANUFACTURER_ID
		 CASE 'DEVICE_ID_STRING':
		 CASE 'MODEL':
		 CASE 'MOD':
		 CASE 'MD':
		 CASE 'DN': // Device Name
			flags = flags ^ BMP_DEVICE_INFO_DEVICE_ID_STRING
		 CASE 'DEVICE_ID':
		 CASE 'DI':
			flags = flags ^ BMP_DEVICE_INFO_DEVICE_ID
		 CASE 'VERSION':
		 CASE 'VER':
			flags = flags ^ BMP_DEVICE_INFO_VERSION
		 CASE 'FIRMWARE_ID':
		 CASE 'FI':
			flags = flags ^ BMP_DEVICE_INFO_FIRMWARE_ID
		 CASE 'SERIAL_NUMBER':
		 CASE 'SN':
			flags = flags ^ BMP_DEVICE_INFO_SERIAL_NUMBER
		 CASE 'SOURCE_TYPE':
		 CASE 'ST':
			flags = flags ^ BMP_DEVICE_INFO_SOURCE_TYPE
		 CASE 'SOURCE_STRING':
		 CASE 'SS':
			flags = flags ^ BMP_DEVICE_INFO_SOURCE_STRING
		 CASE 'FORCE':
			flags = flags ^ BMP_DEVICE_INFO_FORCE
		 CASE 'M':                      // single chars must be at the end of a switch/case
			flags = flags ^ BMP_DEVICE_INFO_MANUFACTURER_STRING
		 CASE 'D':                      // single chars must be at the end of a switch/case
			flags = flags ^ BMP_DEVICE_INFO_DEVICE_ID_STRING
		 CASE 'V':                      // single chars must be at the end of a switch/case
			flags = flags ^ BMP_DEVICE_INFO_VERSION
		 CASE 'F':                      // single chars must be at the end of a switch/case
			flags = flags ^ BMP_DEVICE_INFO_FORCE
		}
	 }
	}
 }
                                    // set data printed to all if none are flagged
																		// don't change the FORCE flag
 IF (!flags & $01FF) {flags = flags ^ $01FF }
 RETURN flags
}

DEFINE_FUNCTION INTEGER getDUET_QUERY_BITMAP (CHAR args[])
/*
 * DEFINE_FUNCTION DUET_GET_QUERY_BITMAP()
 * Purpose: Create the flags for DUET_QUERY_STUB_PROPERTIES() from a
 *          comma-delimited string. This is done in a seperate function so if the
 *          query is run in a loop, there is no nested loop to create the flags.
 * Parameter:
 *   args - comma delimited array of properties to poll
 *          If args is empty, all Duet stub properties will be polled
 */
{
 STACK_VAR INTEGER flags

 IF (LENGTH_ARRAY(args)) {
	STACK_VAR CHAR sArgs[16][16]
	STACK_VAR INTEGER i
	flags = $0000
	i = stringTokenizer(args,',',sArgs)
	FOR (; i; i--) {
	 SWITCH (UPPER_STRING(sArgs[i]))
	 {
		CASE 'PHYSICAL-DEVICE':
		CASE 'PDV':
		CASE 'DV':
		CASE 'PD': // Physical Device
		 flags = flags ^ BMP_DUET_STUB_PHYSICAL_DEVICE
		CASE 'DUET-DEVICE':
		CASE 'VDV':
		CASE 'VD':
		CASE 'DD': // Duet Device
		 flags = flags ^ BMP_DUET_STUB_DUET_DEVICE
		CASE 'DUET-MODULE':
		CASE 'MODULE':
		CASE 'DM': // Duet Module
		 flags = flags ^ BMP_DUET_STUB_DUET_MODULE
		CASE 'BUNDLE-VERSION':
		CASE 'VER':
		CASE 'BV': // Bundle Version
		 flags = flags ^ BMP_DUET_STUB_BUNDLE_VERSION
		CASE 'DEVICE-CATEGORY':
		CASE 'DVCAT':
		CASE 'CAT':
		CASE 'DC': // Device Category
		 flags = flags ^ BMP_DUET_STUB_DEVICE_CATEGORY
		CASE 'DEVICE-MAKE':
		CASE 'MAKE':
		CASE 'MAK':
		CASE 'MK':
		CASE 'DK': // Device maKe
		 flags = flags ^ BMP_DUET_STUB_DEVICE_MAKE
		CASE 'DEVICE-MODEL':
		CASE 'MODEL':
		CASE 'MOD':
		CASE 'MD':
		CASE 'DN': // Device Name
		 flags = flags ^ BMP_DUET_STUB_DEVICE_MODEL
		CASE 'DEVICE-SDKCLASS':
		CASE 'DVS':
		CASE 'SDK':
		CASE 'SC': // Sdk Class
		 flags = flags ^ BMP_DUET_STUB_DEVICE_SDKCLASS
		CASE 'DEVICE-REVISION':
		CASE 'DVR':
		CASE 'REV':
		CASE 'DR': // Device Revision
		 flags = flags ^ BMP_DUET_STUB_DEVICE_REVISION
		CASE 'PROTOCOL-FILE':
		CASE 'XDD':// protocol files will have the extension .XDD
		CASE 'PF': // Protocol File
		 flags = flags ^ BMP_DUET_STUB_PROTOCOL_FILE
		CASE 'V':                      // single chars must be at the end of a switch/case
		 flags = flags ^ BMP_DUET_STUB_BUNDLE_VERSION
		CASE 'M':                      // single chars must be at the end of a switch/case
		 flags = flags ^ BMP_DUET_STUB_DEVICE_MAKE
		CASE 'D':                      // single chars must be at the end of a switch/case
		 flags = flags ^ BMP_DUET_STUB_DEVICE_MODEL
	 }
	}
 }
                                    // set data printed to all if none are flagged
 IF (!flags & $03FF) { flags = flags ^ $03FF }
 RETURN flags
}
DEFINE_FUNCTION CHAR[32] getMasterVersion(DEV dvMaster)
/*
 * function: getMasterVersion()
 * purpose:
 *    Provide a simple way to check version continuity in NetLinx masters.
 *    Starting with NX-x200 masters, the major version has been reset to 1.
 *    The versions for these newer masters will be prefixed with 'NX-'.
 *    The DEVICE_ID_STRING identifying the master is also returned.
 * parameter:
 *    dvMaster - D:P:S provides the option of checking masters with other system numbers.
 *
 */
{
 IF (DEVICE_ID(dvMaster))
 {
	STACK_VAR DEV_INFO_STRUCT uDevInfo
	STACK_VAR INTEGER nMajor
	DEVICE_INFO(dvMaster,uDevInfo)
	nMajor = ATOI(uDevInfo.VERSION)
	IF (uDevInfo.DEVICE_ID >= $18C)   // NX-x200 master or newer:
	{
	 RETURN "'NX-',uDevInfo.VERSION,' ''',uDevInfo.DEVICE_ID_STRING,''''"
	}
																		// If the DEVICE_ID_STRING is 'NI Master' and
																		// the NI-device is online at device number 5001,
																		// replace the generic string with the NI Device string.
	IF ((uDevInfo.DEVICE_ID_STRING == 'NI Master') && (DEVICE_ID(5001:1:dvMaster.SYSTEM)))
	{
	 RETURN "uDevInfo.VERSION,' ''',DEVICE_ID_STRING(5001:1:dvMaster.SYSTEM),''''"
	}
																	  // If the DEVICE_ID_STRING is 'NI Master' and
																		// the DGX switcher device is online at device number 5002,
																		// replace the generic string with the switcher device string.
	IF ((uDevInfo.DEVICE_ID_STRING == 'NI Master') && (DEVICE_ID(5002:1:dvMaster.SYSTEM)))
	{
	 RETURN "uDevInfo.VERSION,' ''',DEVICE_ID_STRING(5002:1:dvMaster.SYSTEM),''''"
	}

																		// other NetLinx masters
	RETURN "uDevInfo.VERSION,' ''',uDevInfo.DEVICE_ID_STRING,''''"
 }
 RETURN 'not online'                // device offline
}
DEFINE_FUNCTION INTEGER getXREF_DEV (CHAR DATA[], DEV dv[], INTEGER ret[])
{
/* Function: getXREF_DEV()
 * Purpose: Take a comma-delimited list of DPS or index values, check if their
 *          items are in a dev array, i.e. find the cross-references between
 *          the list and the dev array.
 * Parameter:
 *    DATA - string array containing the comma-delimited list of D:P:S or index values.
 *           If the value contains ':' it is treated as a dev, otherwise as numeric.
 *    DV   - dev array to check
 *    RET  - Array of indexes to the dev array locations that match the D:P:S values.
 * Affects:
 *    RET  - Array of indexes to the dev array locations that match the D:P:S values.
 * Return:
 *    Length of RET
 *
 */
 STACK_VAR DEV dvTmp
 STACK_VAR INTEGER nTokens          // number of tokens from data
 STACK_VAR INTEGER nDVs             // number of values from dv
 STACK_VAR INTEGER d
 STACK_VAR INTEGER i
 STACK_VAR CHAR sToken[128][17]
 STACK_VAR INTEGER t
 //STACK_VAR INTEGER ret[128]

 STACK_VAR CHAR sList[1024]         // enough for 256 3-digit numbers, comma delimited
 STACK_VAR CHAR sT_Tmp[256][3]
 STACK_VAR INTEGER nTT              // length sT_Tmp
																		// get the tokens and how many, from the input data
 nTokens = StringTokenizer(DATA,',',sToken)
 nDVs = LENGTH_ARRAY(dv)            // get how many devs from the input array
 FOR (; nTokens; nTokens--)
 {
																		// if sToken[nTokens] has a D:P:S value
	IF (FIND_STRING(sToken[nTokens],':',1))
	{
	 STRING_TO_DEV(sToken[nTokens],dvTmp)
	 FOR (d = nDVs; d; d-- )
	 {
		IF (dv[d] == dvTmp)             // if a match is found, save dev index and break the inner loop
		{
		 i++
		 IF (i > MAX_LENGTH_ARRAY(ret)) { break } // range check
		 ret[i] = d
		 BREAK
		}
	 }
	}
																		// if sToken[nTokens] is a numeric range '<n1>*<n2>'
	ELSE IF (FIND_STRING(sToken[nTokens],'*',1))
	{
	 rangeToList(sToken[nTokens],'*',1,',',sList)
	 nTT = StringTokenizer(sList,',',sT_Tmp)
	 FOR (; nTT; nTT--)
	 {
		t = ATOI(sT_Tmp[nTT])
		IF (t && t <= nDVs)
		{
		 i++
		 IF (i > MAX_LENGTH_ARRAY(ret)) { break } // range check
		 ret[i] = t
		}
	 }
	}
	ELSE                              // if sToken[nTokens] is a numeric value
	{
	 t = ATOI(sToken[nTokens])
	 IF (t && t <= nDVs)
	 {
		i++
		IF (i > MAX_LENGTH_ARRAY(ret)) { break } // range check
		ret[i] = t
	 }
	}
 }
 IF (i)                             // if value(s) were found...
 {
	IF (i > MAX_LENGTH_ARRAY(ret))
	{
	 i = MAX_LENGTH_ARRAY(ret)
	 IF (__V4_or_NX_master__)         // if master supports AMX_LOG()
	 {
		AMX_LOG(AMX_ERROR,"'DEV_TO_STRING.axi ',ITOA(__LINE__),' getXREF_DEV() return array overflow'")
	 }
	 ELSE
	 {
		SEND_STRING 0,"'DEV_TO_STRING.axi ',ITOA(__LINE__),' getXREF_DEV() return array overflow'"
	 }
	}
	SET_LENGTH_ARRAY(ret,i)           // set how many are returned...
	quickSort(ret,1,i)                // sort them
	i = removeDuplicates(ret,1,i)
 }
 ELSE { SET_LENGTH_ARRAY(ret,0) }   // set how many are returned...
 
 return i
}

DEFINE_FUNCTION INTEGER getXREF_DEV_INDEX (CHAR DATA[], DEV dv[], INTEGER ndx[], INTEGER ret[])
{
/* Function: getXREF_DEV_INDEX()
 * Purpose: Take a comma-delimited list of DPS or index values, check if their
 *          items are in a dev array AND in an index array, i.e.
 *          find the cross-references between the list, dev, and index arrays.
 * Parameter:
 *    DATA - string array containing the comma-delimited list of D:P:S or index values.
 *           If the value contains ':' it is treated as a dev, otherwise as numeric.
 *    DV   - dev array to check
 *    NDX  - index array to check
 *    RET  - Array of indexes into the dev array, that are also valid for the integer array
 * Affects:
 *    RET  - Array of indexes into the dev array, that are also valid for the integer array
 * Return:
 *    Length of RET
 *
 */
 STACK_VAR DEV dvTmp
 STACK_VAR INTEGER nTokens          // number of tokens from data
 STACK_VAR INTEGER nDVs             // number of values from dv
 STACK_VAR INTEGER nVals            // number of values from ndx
 STACK_VAR INTEGER nVV              // number of valid values found, i.e. value to return
 STACK_VAR INTEGER i
 STACK_VAR INTEGER j
 STACK_VAR CHAR sToken[256][17]
 STACK_VAR INTEGER t
 //STACK_VAR INTEGER ret[128]
 STACK_VAR CHAR sList[1024]         // enough for 256 3-digit numbers, comma delimited
 STACK_VAR CHAR sT_Tmp[256][5]
 STACK_VAR INTEGER nTT              // length sT_Tmp
																		// get the tokens and how many, from the input data
 nTokens = StringTokenizer(DATA,',',sToken)
 i = nTokens
 FOR (; i; i--)                     // expand any ranges found
 {
	IF (FIND_STRING(sToken[i],'*',1))
	{
	 rangeToList(sToken[i],'*',1,',',sList)
	 nTT = StringTokenizer(sList,',',sT_Tmp)
	 
	 sToken[i] = sT_Tmp[nTT]          // Set the current sToken to the last value in sT_Tmp
	 nTT--                            // Subtract 1 from nTT to get the number of values to be added to sToken
	 j = nTokens + 1                  // set j to the index of the next value we will add
	 nTokens = nTokens + nTT          // set nTokens to its new maximum
	 IF (nTokens > MAX_LENGTH_ARRAY(sToken)) // range check
	 {
		nTokens = MAX_LENGTH_ARRAY(sToken)
		IF (__V4_or_NX_master__)        // if master supports AMX_LOG()
		{
		 AMX_LOG(AMX_ERROR,"'DEV_TO_STRING.axi ',ITOA(__LINE__),' getXREF_DEV_INDEX() input array overflow'")
		}
		ELSE
		{
		 SEND_STRING 0,"'DEV_TO_STRING.axi ',ITOA(__LINE__),' getXREF_DEV_INDEX() input array overflow'"
		}
	 }
	 FOR (; nTT; nTT--)
	 {
		sToken[j] = sT_Tmp[nTT]
		IF (j = nTokens) { BREAK }
		ELSE { j++ }
	 }
	}
 }

 nDVs = LENGTH_ARRAY(dv)
 FOR (; nTokens; nTokens--)
 {
																	  // get how many values from the index array
	FOR (nVals = LENGTH_ARRAY(ndx); nVals; nVals--)
	{
																		// if sToken[nTokens] has a D:P:S value
	 IF (FIND_STRING(sToken[nTokens],':',1))
	 {
		STRING_TO_DEV(sToken[nTokens],dvTmp)
		IF (dv[ndx[nVals]] == dvTmp)
		{
		 nVV++
		 IF (nVV > MAX_LENGTH_ARRAY(ret)) { BREAK } // range check
		 ret[nVV] = ndx[nVals]
		 BREAK                          // break the inner loop
		}
	 }
	 ELSE                             // if sToken[nTokens] has an index value
	 {
		t = ATOI(sToken[nTokens])
		IF (t && t <= nDVs)
		{
		 IF (ndx[nVals] == t)
		 {
			nVV++
			IF (nVV > MAX_LENGTH_ARRAY(ret)) { BREAK } // range check
			ret[nVV] = ndx[nVals]
			BREAK                         // break the inner loop
		 }
		}
	 }
	}
 }
 IF (nVV)                           // if value(s) were found...
 {
	IF (nVV > MAX_LENGTH_ARRAY(ret))
	{
	 nVV = MAX_LENGTH_ARRAY(ret)
	 IF (__V4_or_NX_master__)         // if master supports AMX_LOG()
	 {
		AMX_LOG(AMX_ERROR,"'DEV_TO_STRING.axi ',ITOA(__LINE__),' getXREF_DEV_INDEX() return array overflow'")
	 }
	 ELSE
	 {
		SEND_STRING 0,"'DEV_TO_STRING.axi ',ITOA(__LINE__),' getXREF_DEV_INDEX() return array overflow'"
	 }
	}
	SET_LENGTH_ARRAY(ret,nVV)         // set how many are returned...
	quickSort(ret,1,nVV)              // sort them
	nVV = removeDuplicates(ret,1,nVV)
 }
 ELSE { SET_LENGTH_ARRAY(ret,0) }   // set how many are returned...
 return nVV
}

DEFINE_FUNCTION INTEGER getXREF_INDEX (CHAR DATA[], INTEGER ndx[], INTEGER ret[])
{
/* Function: getXREF_INDEX()
 * Purpose: check what items from a list of indexes are in an integer array,
 *          i.e. find the cross-references between the list and the index array.
 * Parameter:
 *    DATA - string array containing the comma-delimited list of indexes or ranges. Range = <n1>*<n2>
 *    NDX  - index array to check
 *    RET  - Array of indexes from data that the integer array contains
 * Affects:
 *    RET  - Array of indexes from data that the integer array contains
 * Return:
 *    Length of RET
 *
 */
 STACK_VAR INTEGER nTokens          // number of tokens from data
 STACK_VAR INTEGER nVals            // number of values from ndx
 STACK_VAR INTEGER i
 STACK_VAR INTEGER j
 STACK_VAR INTEGER nToken[256]      // this array large enough to accept some duplicate values
 //STACK_VAR INTEGER ret[128]

																		// get the tokens and how many, from the input data
 nTokens = mixedListToArray(DATA,'*',1,',',nToken,1,65535,TRUE)
 FOR (; nTokens; nTokens--)
 {
																	  // get how many values from the index array
	FOR (nVals = LENGTH_ARRAY(ndx); nVals; nVals-- )
	{
																		// if a value is found amoung the tokens, save it and break the inner loop
	 IF (ndx[nVals] == nToken[nTokens])
	 {
		i++
		IF (i > MAX_LENGTH_ARRAY(ret)) { BREAK } // range check
		ret[i] = ndx[nVals]
		BREAK
	 }
	}
 }
 IF (i)                             // if value(s) were found...
 {
	IF (i > MAX_LENGTH_ARRAY(ret))
	{
	 i = MAX_LENGTH_ARRAY(ret)
	 IF (__V4_or_NX_master__)         // if master supports AMX_LOG()
	 {
		AMX_LOG(AMX_ERROR,"'DEV_TO_STRING.axi ',ITOA(__LINE__),' getXREF_INDEX() return array overflow'")
	 }
	 ELSE
	 {
		SEND_STRING 0,"'DEV_TO_STRING.axi ',ITOA(__LINE__),' getXREF_INDEX() return array overflow'"
	 }
	}
	SET_LENGTH_ARRAY(ret,i)           // set how many are returned...
	quickSort(ret,1,i)                // sort them
	i = removeDuplicates(ret,1,i)
 }
 ELSE { SET_LENGTH_ARRAY(ret,0) }   // set how many are returned...
 return i
}

DEFINE_FUNCTION printDEVICE_INFO (DEV dvDEBUG, CHAR hdr[], DEV dvDEV, INTEGER flags)
// FUNCTION printDEVICE_INFO (DEV dvDEBUG, DEV dvDEV)
// PURPOSE:    Get device info structure for a device,
//             then send the info to a debug device such
//             as the master's debug port.
// Difference from CALL 'SEND DEVICE_INFO TO DEBUG':
//          1. Header parameter for companion debug module v2.xx and higher.
//             Header is prepended to each line.
// PARAMETERS: dvDEBUG   - device to receive device info
//             hdr       - Header
//             dvDEV     - device from which to get info
{
 STACK_VAR CHAR sTerm[2];
 IF (dvDEBUG.NUMBER = 0 && dvDEBUG.PORT > 1) { sTerm = "13,10" }
 ELSE IF (dvDEBUG.NUMBER > 0)                { sTerm = "13,10" }

 IF ((DEVICE_ID(dvDEV)) ||           // if device is online, or
     (flags & BMP_DEVICE_INFO_FORCE))// the force flag is set
 {
	STACK_VAR DEV_INFO_STRUCT devinfo;

	STACK_VAR CHAR sSTR[100];

	DEVICE_INFO(dvDEV,devinfo);

	send_string dvDebug,"hdr,'D:P:S ',DEV_TO_STRING(dvDEV),sTerm";
	IF (flags & BMP_DEVICE_INFO_MANUFACTURER_STRING)
	{
	 if (length_array(devinfo.MANUFACTURER_STRING))
		send_string dvDebug,"hdr,'Manufacturer ''',devinfo.MANUFACTURER_STRING,'''',sTerm";
	 else
		send_string dvDebug,"hdr,'Manufacturer not available',sTerm";
	}
	IF (flags & BMP_DEVICE_INFO_MANUFACTURER_ID)
	{
	 if (devinfo.MANUFACTURER)
		send_string dvDebug,"hdr,'Manufacturer ID ',ITOA(devinfo.MANUFACTURER),sTerm";
	 else
		send_string dvDebug,"hdr,'Manufacturer ID not available',sTerm";
	}
	IF (flags & BMP_DEVICE_INFO_DEVICE_ID_STRING)
	{
	 if (length_array(devinfo.DEVICE_ID_STRING))
		send_string dvDebug,"hdr,'Device ''',devinfo.DEVICE_ID_STRING,'''',sTerm";
	 else
		send_string dvDebug,"hdr,'Device not available',sTerm";
	}
	IF (flags & BMP_DEVICE_INFO_DEVICE_ID)
	{
	 send_string dvDebug,"hdr,'Device ID $',ITOHEX(devinfo.DEVICE_ID),sTerm";
	}
	IF (flags & BMP_DEVICE_INFO_VERSION)
	{
	 if (length_array(devinfo.VERSION))
		send_string dvDebug,"hdr,'Version ''',devinfo.VERSION,'''',sTerm";
	 else
		send_string dvDebug,"hdr,'Version not available',sTerm";
	}
	IF (flags & BMP_DEVICE_INFO_FIRMWARE_ID)
	{
	 if (devinfo.FIRMWARE_ID)
		send_string dvDebug,"hdr,'Firmware ID $',ITOHEX(devinfo.FIRMWARE_ID),sTerm";
	 else
		send_string dvDebug,"hdr,'Firmware ID not available',sTerm";
	}
	IF (flags & BMP_DEVICE_INFO_SERIAL_NUMBER)
	{
	 if (length_array(devinfo.SERIAL_NUMBER))
	 {
									 // if serial_number is blank for Netlinx device,
									 // first byte is $00
		 if (find_string(devinfo.SERIAL_NUMBER,"$00",1) = 1)
									 // 16 0's
			 send_string dvDebug,"hdr,'Serial Number ''''',sTerm";

									 // trim excess bytes if SN is not 16 chars long
		 else if (find_string(devinfo.SERIAL_NUMBER,"$00",1) > 1)
		 send_string dvDebug,"hdr,'Serial Number ''',
											    LEFT_STRING(devinfo.SERIAL_NUMBER,
LENGTH_ARRAY(devinfo.SERIAL_NUMBER) - (17 - FIND_STRING(devinfo.serial_number,"$00",1))
															),'''',sTerm";

		 else                           // if no $00, just send the whole string
			 send_string dvDebug,"hdr,'Serial Number ''',devinfo.SERIAL_NUMBER,'''',sTerm";
	 }
	 else                             // this should never happen
		 send_string dvDebug,"hdr,'Serial Number not available',sTerm";
	}
	IF (flags & BMP_DEVICE_INFO_SOURCE_TYPE)
	{
	 SWITCH (devinfo.SOURCE_TYPE)
	 {
		 CASE SOURCE_TYPE_NO_ADDRESS:
		 send_string dvDebug,FORMAT("hdr,'Source Type $%02X: no address',sTerm",devinfo.SOURCE_TYPE)
		 CASE SOURCE_TYPE_NEURON_ID:
		 send_string dvDebug,FORMAT("hdr,'Source Type $%02X: connected via ICSNet',sTerm",devinfo.SOURCE_TYPE)
		 CASE SOURCE_TYPE_IP_ADDRESS:
		 send_string dvDebug,FORMAT("hdr,'Source Type $%02X: connected via ethernet',sTerm",devinfo.SOURCE_TYPE)
		 CASE SOURCE_TYPE_AXLINK:
		 send_string dvDebug,FORMAT("hdr,'Source Type $%02X: connected via AXLink',sTerm",devinfo.SOURCE_TYPE)
		 CASE SOURCE_TYPE_NEURON_SUBNODE_ICSP:
		 send_string dvDebug,FORMAT("hdr,'Source Type $%02X: connected via ICSNet',sTerm",devinfo.SOURCE_TYPE)
		 CASE SOURCE_TYPE_NEURON_SUBNODE_PL:
		 send_string dvDebug,FORMAT("hdr,'Source Type $%02X: connected via ICSNet',sTerm",devinfo.SOURCE_TYPE)
		 CASE SOURCE_TYPE_IP_SOCKET_ADDRESS:
		 send_string dvDebug,FORMAT("hdr,'Source Type $%02X: connected via ethernet',sTerm",devinfo.SOURCE_TYPE)
		 CASE SOURCE_TYPE_RS232:
		 send_string dvDebug,FORMAT("hdr,'Source Type $%02X: connected via RS232',sTerm",devinfo.SOURCE_TYPE)
		 CASE SOURCE_TYPE_INTERNAL:
		 send_string dvDebug,FORMAT("hdr,'Source Type $%02X: internal to the NetLinx controller',sTerm",devinfo.SOURCE_TYPE)

		 // next 3 source types added NetLinx.axi v1.25
		 CASE SOURCE_TYPE_MAC_ADDRESS:
		 send_string dvDebug,FORMAT("hdr,'Source Type $%02X: connected via ethernet',sTerm",devinfo.SOURCE_TYPE)
		 CASE SOURCE_TYPE_IPv4_PORT:
		 send_string dvDebug,FORMAT("hdr,'Source Type $%02X: connected via ethernet',sTerm",devinfo.SOURCE_TYPE)
		 CASE SOURCE_TYPE_IPv4_PORT_MAC_ADDRESS:
		 send_string dvDebug,FORMAT("hdr,'Source Type $%02X: connected via ethernet',sTerm",devinfo.SOURCE_TYPE)

		 DEFAULT:
		 send_string dvDebug,FORMAT("hdr,'Source Type $%02X: unknown',sTerm",devinfo.SOURCE_TYPE)
		}
	 }
	 IF (flags & BMP_DEVICE_INFO_SOURCE_STRING)
	 {
		if (length_array(devinfo.SOURCE_STRING))
		 send_string dvDebug,"hdr,'Source String ''',devinfo.SOURCE_STRING,'''',sTerm";
		else
			send_string dvDebug,"hdr,'Source String not available',sTerm";
	 }
 }
 else
	send_string dvDebug,"hdr,'D:P:S ',DEV_TO_STRING(dvDEV),' not online',sTerm";
}

DEFINE_FUNCTION STRING_TO_DEV (char Txt[],dev dvDEV)
/* FUNCTION: Takes ASCII string of the form '...<dev>' or  */
/*           '...<dev>:<prt>:<sys>' and puts those values  */
/*           into dvDEV.                                   */
/* PARAMETERS: Txt   - String that contains the dv:prt:sys */
/*                     as ASCII text.                      */
/*             dvDEV - storage for the returned value      */
{
  STACK_VAR INTEGER DV
  STACK_VAR INTEGER PRT
  STACK_VAR INTEGER SYS
  STACK_VAR INTEGER WherePrt
  STACK_VAR INTEGER WhereSys

									/*********************/
                                    /* get device number */
									/*********************/
  DV = ATOI(Txt)                    // if Txt does not contain a number,
		                            // DV will be 0 (the master)

									/*******************/
                                    /* get port number */
									/*******************/
  IF (FIND_STRING(Txt,':',2))       // if Txt is of form '...<dv>:<prt>...'
  {
    WherePrt = FIND_STRING(Txt,':',1)
    IF (WherePrt)                   // if there might be a port number
    {
									// if port 0 was entered and
									// Txt if of form '...<dv>:<prt>:<sys>'
	  IF (FIND_STRING(Txt,':0:',1) = WherePrt)
	    PRT = 0
      ELSE
      {
	    PRT = ATOI(RIGHT_STRING(Txt,LENGTH_ARRAY(Txt)-WherePrt))
	    IF (PRT = 0)                // if Txt does not contain a number past the
		  PRT = 1                   // first ':', PRT will be 1
	  }
    }
  }
  ELSE								// if Txt does not contain a number past the
    PRT = 1                         // first ':', PRT will be 1

									/*********************/
                                    /* get system number */
									/*********************/
									// if Txt is of form '...<dv>:<prt>:<sys>'
									// note: SYS 0 means local system
  IF (FIND_STRING(Txt,':',WherePrt + 1))
  {
	WhereSys = FIND_STRING(Txt,':',WherePrt + 1)
	SYS = ATOI(RIGHT_STRING(Txt,LENGTH_ARRAY(Txt)-WhereSys))
  }
  dvDEV = DV:PRT:SYS
}
DEFINE_FUNCTION INTEGER STRING_TO_DEV_ARRAY (char Txt[],dev dvDA[])
/* FUNCTION: Takes ASCII string with comma-delimited list  */
/*           of '<dev1>,<dev2>,...etc'  and puts those     */
/*           values into dvDA[].                           */
/* PARAMETERS: Txt   - String that contains the dv:prt:sys */
/*                     as ASCII text.                      */
/*             dvDA  - storage for the returned values     */
/* RETURN: Length of dvDA                                  */
{
  STACK_VAR INTEGER i
									// if there is a comma,
	                                // assume a comma-delimited list of devs
  WHILE (FIND_STRING(Txt,',',1))
  {
	i++
	STRING_TO_DEV (REMOVE_STRING(Txt,',',1),dvDA[i])
  }
  IF (LENGTH_ARRAY(Txt))            // if there is anything left,
  {                                 // or there is no comma
	i++
	STRING_TO_DEV (Txt,dvDA[i])
  }
  SET_LENGTH_ARRAY(dvDA,i)
  RETURN i
}

DEFINE_PROGRAM /* The subroutines below are just wrappers for other functions above */
DEFINE_FUNCTION CHAR[17] DPS_TO_STRING (DEV dvDEV) { RETURN DEV_TO_STRING(dvDEV) }
DEFINE_CALL 'STRING TO DEV' (char Txt[],dev dvDEV) { STRING_TO_DEV (Txt,dvDEV) }

(***********************************************************)
(*                 STARTUP CODE GOES BELOW                 *)
(***********************************************************)
DEFINE_START

IF (!__V4_and_NX_check__)
{
  // Declare
  STACK_VAR DEV_INFO_STRUCT uDevInfo
  STACK_VAR CHAR    cVer[50]
  STACK_VAR CHAR    cTemp[50]
  STACK_VAR INTEGER nMajor
  STACK_VAR INTEGER nMinor
  STACK_VAR INTEGER nBuild

  // Check master version
  DEVICE_INFO(0:0:0,uDevInfo)
  cVer = uDevInfo.VERSION
  cTemp = REMOVE_STRING(cVer,'.',1)
  nMajor = ATOI(cTemp)
  cTemp = REMOVE_STRING(cVer,'.',1)
  nMinor = ATOI(cTemp)
  nBuild = ATOI(cVer)

  IF (uDevInfo.DEVICE_ID >= $18C)   // NI-x200 master or newer
	{
	 __V4_or_NX_master__ = $500+nMajor// 1280 + nMajor
	}
	ELSE IF (nMajor >= 4)             // Older master v4.xx.xx
  {
	 __V4_or_NX_master__ = nMajor
  }
  ELSE                              // Older master v3.xx.xx, v2.xx.xx
  {
	 __V4_or_NX_master__ = FALSE
  }
	__V4_and_NX_check__ = TRUE
}

(***********************************************************)
(*                THE EVENTS GOES BELOW                    *)
(***********************************************************)
DEFINE_EVENT

DATA_EVENT[0:1:0]
{
 ONLINE:
 {
	IF (__display_lib_version__ == TRUE)
	{
	 wait 50                          // WAIT FOR DEVICE HOLDOFF
	 {
		// WHAT VERSION?
		IF (__V4_or_NX_master__)        // if master supports AMX_LOG()
		{
		 AMX_LOG(AMX_ERROR,"'Using lib ',__DEV_TO_STRING_LIB_NAME__,
								               ', v',__DEV_TO_STRING_LIB_VERSION__")
		}
		ELSE
		{
		 SEND_STRING 0:0:0,"'Using lib ',__DEV_TO_STRING_LIB_NAME__,
								               ', v',__DEV_TO_STRING_LIB_VERSION__"
		}
	 }
	}
 }
}

#END_IF
(***********************************************************)
(*                     END OF PROGRAM                      *)
(*        DO NOT PUT ANY CODE BELOW THIS COMMENT           *)
(***********************************************************)	