PROGRAM_NAME='checksum'
(***********************************************************)
(*  FILE CREATED ON: 11/09/2004  AT: 19:46:52              *)
(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 03/03/2023  AT: 09:38:32        *)
(***********************************************************)
(* System Type : NETLINX                                   *)
(***********************************************************)
(*  FILE REVISION: 4.02                                    *)
(*  REVISION DATE: 03/03/2023  AT: 09:38:32                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Removed 15999 character limit from all function      *)
(*    parameters.                                          *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 4.01                                    *)
(*  REVISION DATE: 12/27/2022  AT: 15:44:50                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Use AMX_LOG() when master supports it.               *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 3.13                                    *)
(*  REVISION DATE: 07/17/2016  AT: 10:06:10                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - __CHECKSUM_LIB_NAME__ now lines up with other libs.  *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 3.12                                    *)
(*  REVISION DATE: 10/31/2008  AT: 18:44:12                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Speed tweaks                                         *)
(*    * Reduce FOR and WHILE loop control expressions to   *)
(*      only the variable name where possible.             *)
(*    * Avoid use of LENGTH_ARRAY or LENGTH_STRING() in    *)
(*      FOR or WHILE loop control expressions.             *)
(*    For example:                                         *)
(*    Bad  -   FOR (i = 1; i <= LENGTH_STRING(str); i++)   *)
(*    Good -   len = LENGTH_STRING(str)                    *)
(*             FOR (i = 1; i <= len; i++)                  *)
(*    Better - FOR (i = LENGTH_STRING(str); i > 0; i--)    *)
(*    Best -   FOR (i = LENGTH_STRING(str); i; i--)        *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 3.11                                    *)
(*  REVISION DATE: 10/06/2008  AT: 08:04:43                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added __display_lib_version__ variable.              *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 3.10                                    *)
(*  REVISION DATE: 08/25/2008  AT: 06:58:32                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - checksumAdd32() and checksumAdler32() were rewritten *)
(*    as they do not need to be called recursively when    *)
(*    checking a file - the output of FILE_READ() will not *)
(*    be truncated if you don't put it inside "".          *)
(*  - crc32() now outputs a long, not a string.            *)
(*    The old version has been renamed crc32s().           *)
(*  - Iterator and string length variables in crc16(),     *)
(*    crc32(), crc32s(), checksumAdd32(), and              *)
(*    checksumAdler32() were promoted to LONG so they can  *)
(*    handle string arrays longer than 65535 bytes.        *)
(*  - String-to-be-processed parameters of subroutines     *)
(*    were set to 15999 to workaround a feature that       *)
(*    truncates string expressions to 2048 bytes if passed *)
(*    in via an unbounded array.                           *)
(*    With the parameter size set to 15999 they are not    *)
(*    truncated below this "normal" NetLinx interpreter    *)
(*    limit.                                               *)
(*    String arrays, for example the output of FILE_READ() *)
(*    or GET_BUFFER_STRING(), will still pass through      *)
(*    unaffected at 16000 bytes or greater.                *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: Rev 3.00                                *)
(*  REVISION DATE: 04/17/2008  AT: 13:51:39                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - added checksumAdd32() and checksumAdler32().         *)
(*  - removed version 1 functions.                         *)
(*  - removed version 2 misnamed functions.                *)
(***********************************************************)
(*  FILE REVISION: Rev 2.05                                *)
(*  REVISION DATE: 08/01/2006  AT: 08:34:49                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - complement, not compliment. Kept older function      *)
(*    names as wrappers.                                   *)
(*                                                         *)
(***********************************************************)
(*!!FILE REVISION: 2.04 GLM                                *)
(*  REVISION DATE: 02/27/2006                              *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - renamed crc32IEEE to crc32, added comments and       *)
(*    tweaked variable names for consistency.              *)
(***********************************************************)
(* System Type : NETLINX                                   *)
(*!!FILE REVISION: 2.03 GLM                                *)
(*  REVISION DATE: 02/25/2006                              *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - added CRC8 subroutine                                *)
(*  - added "big table" crc16 (IBM) subroutine - gives     *)
(*    same result as older crc16_calc but is ~2x faster.   *)
(*  - added bitwise crc16 (IBM) subroutine - gives same    *)
(*    result as older crc16_calc but is ~4x slower.        *)
(*  - added bitwise crc16C (CCITT) subroutine.             *)
(*  - added table-driven crc32IEEE (802.3) subrroutine.    *)
(*  - reordered subroutines alphabetically.                *)
(***********************************************************)
(*!!FILE REVISION: 2.02 GLM                                *)
(*  REVISION DATE: 02/11/2004                              *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - swapped names of hi byte, lo byte in CRC16           *)
(*    subroutines                                          *)
(***********************************************************)
(*!!FILE REVISION: 2.01 GLM                                *)
(*  REVISION DATE: 01/06/2004                              *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - added 1's compliment checksums                       *)
(***********************************************************)
(*!!FILE REVISION: 2.00 GLM                                *)
(*  REVISION DATE: 11/16/2004                              *)
(*                                                         *)
(*  COMMENTS:                                              *)
(* checksum functions and calls                            *)
/* 1    function char checksumAdd (sBytes[])									*/
/* 2    function char checksumBxor (sBytes[])									*/
/* 3    function char checksum1sComplement (sBytes[])							*/
/* 4    function char checksum1sComplement2Byte (sBytes)						*/
/* 5    function char checksum2sComplement (sBytes[])							*/
/* 6    function char checksum2sComplement7bit (sBytes[])						*/
/* 7    function char crc8 (sBytes[])                                           */
/* 8    function char[2] crc16 (sBytes[])         **these 3 give same result**  */
/* 9    function char[2] crc16_bitwise (sBytes[]) **these 3 give same result**  */
/* 10   function char[2] crc16_calc (sBytes[])    **these 3 give same result**  */
/* 11   function char[2] crc16C (sbytes[])        **CRC16-CCITT**               */
/* 12   function long crc32 (sBytes[])                                          */
/* 13   function char[4] crc32s (sBytes[])                                      */
/* 14   function long checksumAdd32 (char sBytes[])                             */
/* 15   function long checksumAdler32 (char sBytes[])                           */
(***********************************************************)
(*!!FILE REVISION: 1.00 GLM                                *)
(*  REVISION DATE: 03/22/2002                              *)
/* 16    CALL 'CRC 16 CALC' (CHAR sSTRING[255], INTEGER  CRCLO, INTEGER  CRCHI) */
(***********************************************************)
#IF_NOT_DEFINED __CHECKSUM_LIB__
#DEFINE __CHECKSUM_LIB__
(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE

(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

// VERSION FOR CODE

CHAR __CHECKSUM_LIB_NAME__[20]  = '        checksum.axi';
CHAR __CHECKSUM_LIB_VERSION__[] = '4.02';

#IF_NOT_DEFINED __CRC8TABLE__
CHAR __CRC8TABLE__[256] =
{
  $00, $5E, $BC, $E2, $61, $3F, $DD, $83, $C2, $9C, $7E, $20, $A3, $FD, $1F, $41,
  $9D, $C3, $21, $7F, $FC, $A2, $40, $1E, $5F, $01, $E3, $BD, $3E, $60, $82, $DC,
  $23, $7D, $9F, $C1, $42, $1C, $FE, $A0, $E1, $BF, $5D, $03, $80, $DE, $3C, $62,
  $BE, $E0, $02, $5C, $DF, $81, $63, $3D, $7C, $22, $C0, $9E, $1D, $43, $A1, $FF,
  $46, $18, $FA, $A4, $27, $79, $9B, $C5, $84, $DA, $38, $66, $E5, $BB, $59, $07,
  $DB, $85, $67, $39, $BA, $E4, $06, $58, $19, $47, $A5, $FB, $78, $26, $C4, $9A,
  $65, $3B, $D9, $87, $04, $5A, $B8, $E6, $A7, $F9, $1B, $45, $C6, $98, $7A, $24,
  $F8, $A6, $44, $1A, $99, $C7, $25, $7B, $3A, $64, $86, $D8, $5B, $05, $E7, $B9,
  $8C, $D2, $30, $6E, $ED, $B3, $51, $0F, $4E, $10, $F2, $AC, $2F, $71, $93, $CD,
  $11, $4F, $AD, $F3, $70, $2E, $CC, $92, $D3, $8D, $6F, $31, $B2, $EC, $0E, $50,
  $AF, $F1, $13, $4D, $CE, $90, $72, $2C, $6D, $33, $D1, $8F, $0C, $52, $B0, $EE,
  $32, $6C, $8E, $D0, $53, $0D, $EF, $B1, $F0, $AE, $4C, $12, $91, $CF, $2D, $73,
  $CA, $94, $76, $28, $AB, $F5, $17, $49, $08, $56, $B4, $EA, $69, $37, $D5, $8B,
  $57, $09, $EB, $B5, $36, $68, $8A, $D4, $95, $CB, $29, $77, $F4, $AA, $48, $16,
  $E9, $B7, $55, $0B, $88, $D6, $34, $6A, $2B, $75, $97, $C9, $4A, $14, $F6, $A8,
  $74, $2A, $C8, $96, $15, $4B, $A9, $F7, $B6, $E8, $0A, $54, $D7, $89, $6B, $35
}
#END_IF

#IF_NOT_DEFINED __CRC16TABLE__
INTEGER __CRC16TABLE__[256] =
{
  $0000, $C0C1, $C181, $0140, $C301, $03C0, $0280, $C241,
  $C601, $06C0, $0780, $C741, $0500, $C5C1, $C481, $0440,
  $CC01, $0CC0, $0D80, $CD41, $0F00, $CFC1, $CE81, $0E40,
  $0A00, $CAC1, $CB81, $0B40, $C901, $09C0, $0880, $C841,
  $D801, $18C0, $1980, $D941, $1B00, $DBC1, $DA81, $1A40,
  $1E00, $DEC1, $DF81, $1F40, $DD01, $1DC0, $1C80, $DC41,
  $1400, $D4C1, $D581, $1540, $D701, $17C0, $1680, $D641,
  $D201, $12C0, $1380, $D341, $1100, $D1C1, $D081, $1040,
  $F001, $30C0, $3180, $F141, $3300, $F3C1, $F281, $3240,
  $3600, $F6C1, $F781, $3740, $F501, $35C0, $3480, $F441,
  $3C00, $FCC1, $FD81, $3D40, $FF01, $3FC0, $3E80, $FE41,
  $FA01, $3AC0, $3B80, $FB41, $3900, $F9C1, $F881, $3840,
  $2800, $E8C1, $E981, $2940, $EB01, $2BC0, $2A80, $EA41,
  $EE01, $2EC0, $2F80, $EF41, $2D00, $EDC1, $EC81, $2C40,
  $E401, $24C0, $2580, $E541, $2700, $E7C1, $E681, $2640,
  $2200, $E2C1, $E381, $2340, $E101, $21C0, $2080, $E041,
  $A001, $60C0, $6180, $A141, $6300, $A3C1, $A281, $6240,
  $6600, $A6C1, $A781, $6740, $A501, $65C0, $6480, $A441,
  $6C00, $ACC1, $AD81, $6D40, $AF01, $6FC0, $6E80, $AE41,
  $AA01, $6AC0, $6B80, $AB41, $6900, $A9C1, $A881, $6840,
  $7800, $B8C1, $B981, $7940, $BB01, $7BC0, $7A80, $BA41,
  $BE01, $7EC0, $7F80, $BF41, $7D00, $BDC1, $BC81, $7C40,
  $B401, $74C0, $7580, $B541, $7700, $B7C1, $B681, $7640,
  $7200, $B2C1, $B381, $7340, $B101, $71C0, $7080, $B041,
  $5000, $90C1, $9181, $5140, $9301, $53C0, $5280, $9241,
  $9601, $56C0, $5780, $9741, $5500, $95C1, $9481, $5440,
  $9C01, $5CC0, $5D80, $9D41, $5F00, $9FC1, $9E81, $5E40,
  $5A00, $9AC1, $9B81, $5B40, $9901, $59C0, $5880, $9841,
  $8801, $48C0, $4980, $8941, $4B00, $8BC1, $8A81, $4A40,
  $4E00, $8EC1, $8F81, $4F40, $8D01, $4DC0, $4C80, $8C41,
  $4400, $84C1, $8581, $4540, $8701, $47C0, $4680, $8641,
  $8201, $42C0, $4380, $8341, $4100, $81C1, $8081, $4040
};
#END_IF

#IF_NOT_DEFINED __CRC32TABLE__
LONG  __CRC32TABLE__[256] =
{
 $00000000, $77073096, $EE0E612C, $990951BA,
 $076DC419, $706AF48F, $E963A535, $9E6495A3,
 $0EDB8832, $79DCB8A4, $E0D5E91E, $97D2D988,
 $09B64C2B, $7EB17CBD, $E7B82D07, $90BF1D91,
 $1DB71064, $6AB020F2, $F3B97148, $84BE41DE,
 $1ADAD47D, $6DDDE4EB, $F4D4B551, $83D385C7,
 $136C9856, $646BA8C0, $FD62F97A, $8A65C9EC,
 $14015C4F, $63066CD9, $FA0F3D63, $8D080DF5,
 $3B6E20C8, $4C69105E, $D56041E4, $A2677172,
 $3C03E4D1, $4B04D447, $D20D85FD, $A50AB56B,
 $35B5A8FA, $42B2986C, $DBBBC9D6, $ACBCF940,
 $32D86CE3, $45DF5C75, $DCD60DCF, $ABD13D59,
 $26D930AC, $51DE003A, $C8D75180, $BFD06116,
 $21B4F4B5, $56B3C423, $CFBA9599, $B8BDA50F,
 $2802B89E, $5F058808, $C60CD9B2, $B10BE924,
 $2F6F7C87, $58684C11, $C1611DAB, $B6662D3D,
 $76DC4190, $01DB7106, $98D220BC, $EFD5102A,
 $71B18589, $06B6B51F, $9FBFE4A5, $E8B8D433,
 $7807C9A2, $0F00F934, $9609A88E, $E10E9818,
 $7F6A0DBB, $086D3D2D, $91646C97, $E6635C01,
 $6B6B51F4, $1C6C6162, $856530D8, $F262004E,
 $6C0695ED, $1B01A57B, $8208F4C1, $F50FC457,
 $65B0D9C6, $12B7E950, $8BBEB8EA, $FCB9887C,
 $62DD1DDF, $15DA2D49, $8CD37CF3, $FBD44C65,
 $4DB26158, $3AB551CE, $A3BC0074, $D4BB30E2,
 $4ADFA541, $3DD895D7, $A4D1C46D, $D3D6F4FB,
 $4369E96A, $346ED9FC, $AD678846, $DA60B8D0,
 $44042D73, $33031DE5, $AA0A4C5F, $DD0D7CC9,
 $5005713C, $270241AA, $BE0B1010, $C90C2086,
 $5768B525, $206F85B3, $B966D409, $CE61E49F,
 $5EDEF90E, $29D9C998, $B0D09822, $C7D7A8B4,
 $59B33D17, $2EB40D81, $B7BD5C3B, $C0BA6CAD,
 $EDB88320, $9ABFB3B6, $03B6E20C, $74B1D29A,
 $EAD54739, $9DD277AF, $04DB2615, $73DC1683,
 $E3630B12, $94643B84, $0D6D6A3E, $7A6A5AA8,
 $E40ECF0B, $9309FF9D, $0A00AE27, $7D079EB1,
 $F00F9344, $8708A3D2, $1E01F268, $6906C2FE,
 $F762575D, $806567CB, $196C3671, $6E6B06E7,
 $FED41B76, $89D32BE0, $10DA7A5A, $67DD4ACC,
 $F9B9DF6F, $8EBEEFF9, $17B7BE43, $60B08ED5,
 $D6D6A3E8, $A1D1937E, $38D8C2C4, $4FDFF252,
 $D1BB67F1, $A6BC5767, $3FB506DD, $48B2364B,
 $D80D2BDA, $AF0A1B4C, $36034AF6, $41047A60,
 $DF60EFC3, $A867DF55, $316E8EEF, $4669BE79,
 $CB61B38C, $BC66831A, $256FD2A0, $5268E236,
 $CC0C7795, $BB0B4703, $220216B9, $5505262F,
 $C5BA3BBE, $B2BD0B28, $2BB45A92, $5CB36A04,
 $C2D7FFA7, $B5D0CF31, $2CD99E8B, $5BDEAE1D,
 $9B64C2B0, $EC63F226, $756AA39C, $026D930A,
 $9C0906A9, $EB0E363F, $72076785, $05005713,
 $95BF4A82, $E2B87A14, $7BB12BAE, $0CB61B38,
 $92D28E9B, $E5D5BE0D, $7CDCEFB7, $0BDBDF21,
 $86D3D2D4, $F1D4E242, $68DDB3F8, $1FDA836E,
 $81BE16CD, $F6B9265B, $6FB077E1, $18B74777,
 $88085AE6, $FF0F6A70, $66063BCA, $11010B5C,
 $8F659EFF, $F862AE69, $616BFFD3, $166CCF45,
 $A00AE278, $D70DD2EE, $4E048354, $3903B3C2,
 $A7672661, $D06016F7, $4969474D, $3E6E77DB,
 $AED16A4A, $D9D65ADC, $40DF0B66, $37D83BF0,
 $A9BCAE53, $DEBB9EC5, $47B2CF7F, $30B5FFE9,
 $BDBDF21C, $CABAC28A, $53B39330, $24B4A3A6,
 $BAD03605, $CDD70693, $54DE5729, $23D967BF,
 $B3667A2E, $C4614AB8, $5D681B02, $2A6F2B94,
 $B40BBE37, $C30C8EA1, $5A05DF1B, $2D02EF8D
};
#END_IF

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

#IF_NOT_DEFINED __display_lib_version__
VOLATILE __display_lib_version__ = TRUE
#END_IF

#IF_NOT_DEFINED __V4_or_NX_master__
VOLATILE INTEGER __V4_or_NX_master__ = 0
#END_IF

#IF_NOT_DEFINED __V4_and_NX_check__
VOLATILE INTEGER __V4_and_NX_check__ = 0
#END_IF

(***********************************************************)
(*           SUBROUTINE DEFINITIONS GO BELOW               *)
(***********************************************************)
(* EXAMPLE: DEFINE_FUNCTION <RETURN_TYPE> <NAME> (<PARAMETERS>) *)
(* EXAMPLE: DEFINE_CALL '<NAME>' (<PARAMETERS>) *)

                                    //////////////////////////////
                                    // CHECKSUM FUNCTIONS/CALLS //
                                    //////////////////////////////
// the most common checksum - sum of all bytes
define_function char checksumAdd (char sBytes[])
stack_var integer 	i;
stack_var char		checkSum;
{
  for (i = length_string(sBytes); i; i--)
	{checkSum = checkSum + sBytes[i];}
  return checkSum;
}

// bitwise-exclusive-or checksum
define_function char checksumBxor (char sBytes[])
stack_var integer	i;
stack_var char	    checkSum;
{
  for (i = length_string(sBytes); i; i--)
	{checkSum = checkSum ^ sBytes[i];}
  return checkSum;
}

// 1's complement checksum
define_function char checksum1sComplement (char sBytes[])
stack_var integer	i;
stack_var char		checkSum;
{
									// start with the common checksum
  for (i = length_string(sBytes); i; i--)
    {checkSum = checkSum + sBytes[i];}

  checkSum = ~checkSum;				// get the 1's complement
  return checksum;
}

// 1's complement checksum, returns "lo byte,hi byte" for Hitach HVD30
define_function char[2] checksum1sComplement2byte (char sBytes[])
stack_var integer	i;
stack_var integer	checkSum;
{
									// start with the common checksum
  for (i = length_string(sBytes); i; i--)
    {checkSum = checkSum + sBytes[i];}

  checkSum = ~checkSum;				// get the 1's complement

									// return lo byte, hi byte
  return "checksum & $FF,checksum  >> 8";
}

// 2's complement checksum
define_function char checksum2sComplement (char sBytes[])
stack_var integer	i;
stack_var char		checkSum;
{
                                    // start with the common checksum
  for (i = length_string(sBytes); i; i--)
	{checkSum = checkSum + sBytes[i];}
  checkSum = ~checkSum + 1          // get the 2's complement
  return checkSum;
}

// 2's complement checksum, 7 bit for roland/yamaha MIDI sysx
define_function char checksum2sComplement7bit (char sBytes[])
stack_var integer	i;
stack_var char		checkSum;
{
                                    // start with the common checksum
  for (i = length_string(sBytes); i; i--)
	{checkSum = checkSum + sBytes[i];}
  checkSum = ~checkSum + 1;         // get the 2's complement
  checkSum = checkSum & $7F;        // get the least significant 7 bits
  return checkSum;
}

DEFINE_FUNCTION CHAR crc8(CHAR sBytes[])
{
  STACK_VAR CHAR	crc;
  STACK_VAR INTEGER lengthStr;
  STACK_VAR INTEGER i;

  crc = $FF; 						// initialize the checksum to $FF
  lengthStr = LENGTH_ARRAY(sBytes);

  FOR(i = 1; i <= lengthStr; i++)
  {
	crc = __CRC8TABLE__[(crc ^ sBytes[i])+1]
  }
  RETURN crc;
}

//////////////////////////////////////////////////////////
// Subroutine Name  : crc16                             //
// Purpose          : Calculates CRC16 (IBM) Checksum   //
// Parameters       :                                   //
//      sBytes        - Command String To               //
//                      Calculate Checksum.             //
// Returns          : "crcLo,crcHi"                     //
//////////////////////////////////////////////////////////
DEFINE_FUNCTION CHAR[2] CRC16 (CHAR sBytes[])
{
  STACK_VAR INTEGER Temp;
  STACK_VAR INTEGER CRCWord;
  LONG length;
  LONG i;
  CHAR crcLo;
  CHAR crcHi;
  CRCWord = $0000;
  length = LENGTH_ARRAY(sBytes);

  FOR (i = 1; i <= length; i++) {
	CRCWord = TYPE_CAST(__CRC16Table__[((sBytes[i] ^ CRCWord) & $FF) + 1] ^ (CRCWord >> 8));
  }
  // return CRCWord;  // return as an integer

  crcLo = TYPE_CAST(CRCWord & $FF);
  crcHi = TYPE_CAST(CRCWORD >> 8);
  RETURN "crcLo,crcHi";  // return as a string
}

//////////////////////////////////////////////////////
// Subroutine Name  : crc16_bitwise                 //
// Purpose          : Calculates CRC 16 Checksum    //
// Parameters       :                               //
//      sBytes        - Command String To           //
//                      Calculate Checksum.         //
// Returns          : "crcLo,crcHi"                 //
//////////////////////////////////////////////////////
DEFINE_FUNCTION CHAR[2] crc16_bitwise (CHAR sBytes[])
{
	STACK_VAR INTEGER nFlag;
	STACK_VAR INTEGER i;
	STACK_VAR INTEGER j;
	STACK_VAR INTEGER nPOLYNOMIAL;
	STACK_VAR INTEGER nCRC;
	STACK_VAR CHAR crcHi;
	STACK_VAR CHAR crcLo;
	STACK_VAR INTEGER length;

    nPolynomial = $A001;
    nFlag = 0;
    nCRC = $0000;
	length = LENGTH_STRING (sBytes)

    FOR (i = 1; i <= length; i++)
    {
        nCRC = nCRC ^ sBytes[i];
        FOR(j=1; j<=8 ; j++)
        {
            nFlag = (nCRC & $01) != 0;
            nCRC  = TYPE_CAST (nCRC >> 1);
            if(nFlag)
                nCRC = nCRC ^ nPOLYNOMIAL;
        }
    }

	crcHi = type_cast((nCRC >> 8) & $FF);
    crcLo = type_cast(nCRC & $FF);

	RETURN "crcLo,crcHi";
}

//////////////////////////////////////////////////////
// Subroutine Name  : crc16_calc                    //
// Purpose          : Calculates CRC 16 Checksum    //
// Parameters       :                               //
//      sBytes        - Command String To           //
//                      Calculate Checksum.         //
// Returns          : "crcLo,crcHi"                 //
//////////////////////////////////////////////////////
/* This function is based on the call 'CRC 16 CALC' */
/* below, and returns the same crc value.           */
define_function char[2] crc16_calc (char sBytes[])
stack_var integer crc;
stack_var integer i;
stack_var integer crc16_table[16];
stack_var char crcHi;               // high byte of CRC16
stack_var char crcLo;               // low byte of CRC16
stack_var integer length;
{
  crc16_table[1] =$0000;
  crc16_table[2] =$CC01;
  crc16_table[3] =$D801;
  crc16_table[4] =$1400;
  crc16_table[5] =$F001;
  crc16_table[6] =$3C00;
  crc16_table[7] =$2800;
  crc16_table[8] =$E401;
  crc16_table[9] =$A001;
  crc16_table[10]=$6C00;
  crc16_table[11]=$7800;
  crc16_table[12]=$B401;
  crc16_table[13]=$5000;
  crc16_table[14]=$9C01;
  crc16_table[15]=$8801;
  crc16_table[16]=$4400;

  crc = $0000;
  length = LENGTH_STRING(sBytes);

  for (i = 1; i <= length; i++)
  {
	crc  = type_cast(crc >> 4 ^ crc16_table[((sBytes[I] ^ crc) & $0F) + 1]);
	crc  = type_cast(crc >> 4 ^ crc16_table[(((sBytes[I] >> 4) ^ crc) & $0F) + 1]);
  }

  crcHi = type_cast(crc >> 8);
  crcLo = type_cast(crc & $FF);

  return "crcLo,crcHi";
}

////////////////////////////////////////////////////////
// Subroutine Name  : crc16C                          //
// Purpose          : Calculates CRC16 CCITT Checksum //
// Parameters       :                                 //
//      sBytes        - Command String To             //
//                      Calculate Checksum.           //
// Returns          : "crcHi,crcLo"                   //
////////////////////////////////////////////////////////
DEFINE_FUNCTION CHAR[2] crc16C(CHAR sBytes[])
{
  LONG   dwPos;
  LONG   dwChecksum;
  LONG   dwDataSize;


  INTEGER i;
  INTEGER v;
  INTEGER xor_flag;

  CHAR crcHi;
  CHAR crcLo;

  dwPos = 0;
  dwChecksum = $ffff;
  dwDataSize = LENGTH_ARRAY(sBytes);

  i = 0; v = 0; xor_flag = 0;

  for (dwPos = 1; dwPos <= dwDataSize; dwPos++)
  {
	  // Align test bit with leftmost bit of the message byte.
	  v = $80;

	  for (i = 0; i < 8; i++)
	  {
		  if (dwChecksum & $8000)
			  xor_flag = 1;
		  else
			  xor_flag = 0;

		  dwChecksum = dwChecksum << 1;

		  if (sBytes[dwPos] & v)
		  {
			  // Append next bit of message to end of CRC if it is not zero.
			  // The zero bit placed there by the shift above need not be
			  // changed if the next bit of the message is zero.
			  dwChecksum++;
		  }

		  if (xor_flag)
			  dwChecksum = dwChecksum ^ $1021;   // CRC-CCITT mask

		  // Align test bit with next bit of the message byte.
		  v = TYPE_CAST(v >> 1);

	  } // for (i = 0; i < 8; i++)
  } // for (dwPos = 0; dwPos < dwDataSize; dwPos++)

  for (dwPos = 0; dwPos < 16; dwPos++)
  {
	  if (dwChecksum & $8000)
		  xor_flag = 1;
	  else
		  xor_flag = 0;

	  dwChecksum = dwChecksum << 1;

	  if (xor_flag)
		  dwChecksum = dwChecksum ^ $1021;   // CRC-CCITT mask
  }

  dwChecksum       = dwChecksum & $ffff;
  crcHi = TYPE_CAST(dwChecksum >> 8);
  crcLo = TYPE_CAST(dwChecksum & $ff);

  RETURN "crcHi,crcLo";
}

////////////////////////////////////////////////////////////////
// Subroutine Name  : crc32                                   //
// Purpose          : Calculates CRC32 (IEEE 802.3) Checksum  //
// Parameters       :                                         //
//      sBytes        - Command String To calculate checksum. //
// Returns          : crc32 as a long                         //
////////////////////////////////////////////////////////////////
DEFINE_FUNCTION long crc32 ( char sBytes[] )
{
  long lCrc32;
  long lengthStr;
  long i;


  /* initialize value */
  lCrc32 = $FFFFFFFF;
  lengthStr = LENGTH_ARRAY(sBytes);

  /* process each byte */
  for (i = 1; i <= lengthStr; i++) {
    lCrc32 = __Crc32table__[((lCrc32 ^ sBytes[i]) & $FF) + 1] ^ (lCrc32 >> 8);
  }

  lCrc32 = lCrc32 ^ $FFFFFFFF;      // 1's complement of crc

  RETURN lCrc32;
}

////////////////////////////////////////////////////////////////
// Subroutine Name  : crc32s                                  //
// Purpose          : Calculates CRC32 (IEEE 802.3) Checksum  //
// Parameters       :                                         //
//      sBytes        - Command String To                     //
//                      Calculate Checksum.                   //
// Returns          : sCrc32 - string with MSB first          //
////////////////////////////////////////////////////////////////
DEFINE_FUNCTION char[4] crc32s ( char sBytes[] )
{
  long lCrc32;
  char sCrc32[4];
  long lengthStr;
  long i;


  /* initialize value */
  lCrc32 = $FFFFFFFF;
  lengthStr = LENGTH_ARRAY(sBytes);

  /* process each byte */
  for (i = 1; i <= lengthStr; i++) {
    lCrc32 = __Crc32table__[((lCrc32 ^ sBytes[i]) & $FF) + 1] ^ (lCrc32 >> 8);
  }

  lCrc32 = lCrc32 ^ $FFFFFFFF;      // 1's complement of crc

  sCRC32[1] = TYPE_CAST((lcrc32 >> 24) & $FF);
  sCRC32[2] = TYPE_CAST((lcrc32 >> 16) & $FF);
  sCRC32[3] = TYPE_CAST((lcrc32 >> 8) & $FF);
  sCRC32[4] = TYPE_CAST(lcrc32 & $FF);
  SET_LENGTH_ARRAY(sCRC32,4);
  RETURN sCRC32;                     // crc as a string
}

////////////////////////////////////////////////////////////////
// Subroutine Name  : checksumAdd32                           //
// Purpose          : Adds bytes to create a 32-bit checksum. //
// Parameters       :                                         //
//      sBytes        - Command String To calculate checksum. //
// Returns          : checksum as a LONG                      //
////////////////////////////////////////////////////////////////
define_function long checksumAdd32 (char sBytes[])
{
  stack_var long	i;
  stack_var long	cksm;
  stack_var long    len;

  len = length_string(sBytes);

  for (i = 1; i <= len; i++)
	{cksm = cksm + sBytes[i];}
  return cksm;
}

////////////////////////////////////////////////////////////////
// Subroutine Name  : checksumAdler32                         //
// Purpose          : Adds bytes to create a 32-bit Adler     //
//                    checksum.                               //
// Parameters       :                                         //
//      sBytes        - Command String To calculate checksum. //
// Returns          : checksum as a LONG                      //
////////////////////////////////////////////////////////////////
define_function long checksumAdler32 (char sBytes[])
{
  stack_var long	i;
  stack_var long	cksm;
  stack_var long	lo;             // lo word of checkSum
  stack_var long	hi;             // hi word of checkSum
  stack_var long	len;            // #bytes to process in outer loop
  stack_var long	tlen;           // #bytes to process in inner loop


  lo = 1;
  hi = 0;
  len = LENGTH_ARRAY(sBytes);
  
  while (len)
  {
	// 5550 bytes can be processed without overflowing 32-bit sums
	// so mod 65521 only every 5550 bytes.
	if (len > 5550)	tlen = 5550;
	else 			tlen = len;
  
	len = len - tlen;

	while (tlen)
	{
	  i++;
	  
	  lo = lo + sBytes[i];
	  hi = hi + lo;
	  
	  tlen--;
	}
	
	lo = lo % 65521;
	hi = hi % 65521;
  }
  
  cksm = (hi << 16) + lo;
  return cksm;
}

(************************************************)
(* Subroutine Name : CRC 16 CALC                *)
(* Purpose         : Calculates CRC 16 Checksum *)
(* Parameters      :                            *)
(*      STRING        - Command String To       *)
(*                      Calculate Checksum.     *)
(*      CRCLO         - LO Byte Value Of        *)
(*                      Checksum.               *)
(*      CRCHI         - HI Byte Value Of        *)
(*                      Checksum.               *)
(************************************************)
/* This call is left essentially unchanged from */
/* the original Axcess version.                 */
/* The Netlinx changes are:                     */
/*   - STACK_VAR instead of LOCAL_VAR           */
/*   - Param/Var types are declared explicitly  */
DEFINE_CALL 'CRC 16 CALC' (CHAR sSTRING[], INTEGER  CRCLO, INTEGER  CRCHI)
    STACK_VAR

      INTEGER       CRC
      INTEGER       TEMP
      INTEGER       I
      INTEGER       CRC16_TABLE[16]
      INTEGER       INT_VALUE
      INTEGER       LENGTH

{
	  CRC16_TABLE[1] =$0000
	  CRC16_TABLE[2] =$CC01
	  CRC16_TABLE[3] =$D801
	  CRC16_TABLE[4] =$1400
	  CRC16_TABLE[5] =$F001
	  CRC16_TABLE[6] =$3C00
	  CRC16_TABLE[7] =$2800
	  CRC16_TABLE[8] =$E401
	  CRC16_TABLE[9] =$A001
	  CRC16_TABLE[10]=$6C00
	  CRC16_TABLE[11]=$7800
	  CRC16_TABLE[12]=$B401
	  CRC16_TABLE[13]=$5000
	  CRC16_TABLE[14]=$9C01
	  CRC16_TABLE[15]=$8801
	  CRC16_TABLE[16]=$4400

      CRC = 0
      I   = 1

     LENGTH = LENGTH_STRING(sSTRING)

     WHILE (I <= LENGTH)
     {
        TEMP = ((sSTRING[I] ^ CRC) & $0F) + 1
        CRC  = (CRC / 16) ^ CRC16_TABLE[TEMP]
        TEMP = (((sSTRING[I] / 16) ^ CRC) & $0F) + 1
        CRC  = (CRC / 16) ^ CRC16_TABLE[TEMP]
        I = I + 1
     }
     CRCHI = CRC / 256
     CRCLO = CRC & $FF
}

(***********************************************************)
(*                 STARTUP CODE GOES BELOW                 *)
(***********************************************************)
DEFINE_START

IF (!__V4_and_NX_check__)
{
  // Declare
  STACK_VAR DEV_INFO_STRUCT uDevInfo
  STACK_VAR CHAR    cVer[50]
  STACK_VAR CHAR    cTemp[50]
  STACK_VAR INTEGER nMajor
  STACK_VAR INTEGER nMinor
  STACK_VAR INTEGER nBuild

  // Check master version
  DEVICE_INFO(0:0:0,uDevInfo)
  cVer = uDevInfo.VERSION
  cTemp = REMOVE_STRING(cVer,'.',1)
  nMajor = ATOI(cTemp)
  cTemp = REMOVE_STRING(cVer,'.',1)
  nMinor = ATOI(cTemp)
  nBuild = ATOI(cVer)

  IF (uDevInfo.DEVICE_ID >= $18C)   // NI-x200 master or newer
	{
	 __V4_or_NX_master__ = $500+nMajor// 1280 + nMajor
	}
	ELSE IF (nMajor >= 4)             // Older master v4.xx.xx
  {
	 __V4_or_NX_master__ = nMajor
  }
  ELSE                              // Older master v3.xx.xx, v2.xx.xx
  {
	 __V4_or_NX_master__ = FALSE
  }
	__V4_and_NX_check__ = TRUE
}

(***********************************************************)
(*                THE EVENTS GOES BELOW                    *)
(***********************************************************)
DEFINE_EVENT

DATA_EVENT[0:1:0]
{
 ONLINE:
 {
  wait 50                         // WAIT FOR DEVICE HOLDOFF
  {
	 IF (__display_lib_version__ == TRUE)
	 {
		// WHAT VERSION?
		IF (__V4_or_NX_master__)        // if master supports AMX_LOG()
		{
		 AMX_LOG(AMX_ERROR,"'Using lib ',__CHECKSUM_LIB_NAME__,
															 ', v',__CHECKSUM_LIB_VERSION__")
		}
		ELSE
		{
		 SEND_STRING 0:0:0,"'Using lib ',__CHECKSUM_LIB_NAME__,
															 ', v',__CHECKSUM_LIB_VERSION__";
		}
	 }
	}
 }
}

#END_IF
(***********************************************************)
(*                     END OF PROGRAM                      *)
(*        DO NOT PUT ANY CODE BELOW THIS COMMENT           *)
(***********************************************************)
