PROGRAM_NAME='debug'
(***********************************************************)
(*  FILE CREATED ON: 12/28/2001 AT: 00:00:00               *)
(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 03/03/2023  AT: 09:09:39        *)
(***********************************************************)
/*
 * tab-width: 4
 * indent-level: 2
 *
 */
DEFINE_PROGRAM                      // rev notes
(***********************************************************)
(*  FILE REVISION: 4.02                                    *)
(*  REVISION DATE: 03/03/2023  AT: 09:09:39                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - printToDebug() and printMidiDebug():                 *)
(*    * Removed 15999 character limit from data parameter. *)
(*    * Added header limit of 128. The header should just  *)
(*      be a short prefix, and it is copied into a         *)
(*      STACK_VAR with a length of 128.                    *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 4.01                                    *)
(*  REVISION DATE: 12/27/2022  AT: 18:07:38                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Clean up master version check for AMX_LOG().         *)
(*  - Add conditional compile for logString(), as file.axi *)
(*    also includes it.                                    *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 4.00                                    *)
(*  REVISION DATE: 08/18/2021  AT: 14:18:55                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Use AMX_LOG() when master supports it.               *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 3.19, 3.18                              *)
(*  REVISION DATE: 12/21/2020  AT: 19:53:56                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - __formatDecimal__() added formatting to look good    *)
(*    with IP address info. Formatting is based on length, *)
(*    for now...                                           *)
(*  - Fixed bug in __formatHex__ where the ',' delimiter   *)
(*    was trimmed incorrectly.                             *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 3.17                                    *)
(*  REVISION DATE: 07/19/2016  AT: 10:39:03                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Renamed __formatAscii__() to __formatAsciiExt__().   *)
(*  - Fixed extra ' printing when next byte is $7F in      *)
(*    __formatAsciiHex__().                                *)
(*  - printMidiDebug() can now print SysEx data as ASCII,  *)
(*    when flagged via nMode.                              *)
(*  - Simplified __formatDecimal__() and __formatHex__().  *)
(*  - nMode is now bitmapped. The change is unlikely to be *)
(*    noticed in functions other than printMidiDebug().    *)
(*  - __DEBUG_LIB_NAME__ now 20 chars (16 + 4 for '.axi'). *)
(*  - Moved getMasterVersion() to DEV_TO_STRING.axi v3.08. *)
(*  - Improved 'programmed by' info returned by            *)
(*    printUrlList().                                      *)
(*  - Other libs used:                                     *)
(*           DEV_TO_STRING.axi v3.08                       *)
(*           strings.axi v2.14                             *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 3.16                                    *)
(*  REVISION DATE: 01/11/2016  AT: 10:32:35                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added CUSTOM_EVENT type 768 for ModeroX streaming    *)
(*    video status changes.                                *)
(*  - printCUSTOM_EVENT() tweaks:                          *)
(*    * Returns from __CUSTOM_EVENT_???_TO_STRING__() no   *)
(*      longer start with a space.                         *)
(*      start with a space ' '.                            *)
(*    * __CUSTOM_EVENT_TYPE_TO_STRING__() now only returns *)
(*      "name", not "number,name".                         *)
(*    * Merged __CUSTOM_EVENT_JUSTIFY_TO_STRING__() into   *)
(*      __CUSTOM_EVENT_VALUE2_TO_STRING__().               *)
(*  - Changed how getMasterVersion() reports NX master.    *)
(*    Version is prefixed with 'NX-' instead of adding     *)
(*    5000 to the major version number, e.g.:              *)
(*    'NX-v1.3.110' instead of 'v5001.3.110'.              *)
(*  - NX-x200 master in most comments, not NI-x200.        *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 3.15/3.14                               *)
(*  REVISION DATE: 03/11/2015  AT: 17:47:45                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Moved functions getDEVICE_INFO_BITMAP() and          *)
(*    printDEVICE_INFO() to DEV_TO_STRING.axi from here.   *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 3.13                                    *)
(*  REVISION DATE: 03/11/2014  AT: 12:24:16                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - printToDebug() and printMidiDebug() now print the    *)
(*    header and an empty string '' if the data is empty.  *)
(*  - Added getMasterVersion() function. If used with one  *)
(*    the new NI-x200 masters, it adds 5000 to the master  *)
(*    major version.                                       *)
(*    E.g. getMasterVersion(<NI-x200 v1.2.230>) will       *)
(*    return 'v5001.2.230'.                                *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 3.03                                    *)
(*  REVISION DATE: 02/21/2014  AT: 13:59:05                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - The print<> functions are designed to be used in     *)
(*    NetLinx code to format and print debug statements to *)
(*    CLI, e.g. print DATA.TEXT to device 0:1:0.           *)
(*    * Reordered code so print<> functions are on top.    *)
(*    * Renamed helper functions as __<name>__ to make it  *)
(*      clearer that they were designed to be used only in *)
(*      this code block, by the print<> functions.         *)
(*    * Updated comments in the older wrapper calls. They  *)
(*      are deprecated but may be useful as examples of    *)
(*      print<> function usage.                            *)
(*  - Added printUrlList() function.                       *)
(*  - Added DEFINE_PROGRAM to top of Rev Notes to enable   *)
(*    code folding the notes without hiding the timestamp. *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 3.02                                    *)
(*  REVISION DATE: 08/24/2012  AT: 13:02:44                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Fixed MID_STRING() in printToDebug() so it will run  *)
(*    on masters below v3.21.354.                          *)
(*  - Added CUSTOM_EVENT type 1400 for Dynamic Image       *)
(*    resource load (i.e. refresh) notification.           *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 3.01                                    *)
(*  REVISION DATE: 07/27/2012  AT: 14:57:11                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added CUSTOM_EVENT types 700, 710, 1325 for ModeroX  *)
(*    NFC Tag, Smart Card Reader, and Audio Device Used.   *)
(*  - Added CUSTOM_EVENT type 600 for gestures.            *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 3.00                                    *)
(*  REVISION DATE: 06/05/2012  AT: 18:38:23                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Modified how DEV_INFO_STRUCT.SOURCE_STRING is        *)
(*    printed, so it can be compared with DATA.SOURCEIP    *)
(*    when both are printed by the debug module.           *)
(*  - printCUSTOM_EVENT() now uses the nCols and nMode     *)
(*    parameters like printToDebug() et al.                *)
(*  - printCUSTOM_EVENT() now returns values in the same   *)
(*    order that they are listed in the structure TCUSTOM. *)
(*  - Custom event functions now print digits before the   *)
(*    descriptions of the digits.                          *)
(*    Previous versions would sometimes print them before, *)
(*    sometimes after.                                     *)
(*  - Other minor tweaks to how custom events print.       *)
(*  - Added the functions printToDebug() and               *)
(*    printMidiDebug().                                    *)
(*  - The old DEFINE_CALL subroutines are now just         *)
(*    wrappers for newer functions.                        *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.64                                    *)
(*  REVISION DATE: 04/24/2012  AT: 11:13:27                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - CUSTOM_EVENT functions: For clarity print 'ID',      *)
(*   'VALUE1', 'VALUE2', 'VALUE3', and 'FLAG' along their  *)
(*    descriptions. Do not print 'ADDRESS' with ID if ID   *)
(*    is 0.                                                *)
(*  - Added justification numbers to be displayed along    *)
(*    text in __CUSTOM_EVENT_JUSTIFY_TO_STRING__().        *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.63                                    *)
(*  REVISION DATE: 03/26/2012  AT: 10:46:24                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Minor tweaks to printCustomEvent() and               *)
(*    __CUSTOM_EVENT_TYPE_TO_STRING__().                   *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.62                                    *)
(*  REVISION DATE: 03/24/2011  AT: 12:57:12                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Modified code to add <CR><LF> if dvDebug is not a    *)
(*    master debug terminal.                               *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.61                                    *)
(*  REVISION DATE: 06/28/2010  AT: 17:47:07                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added return values and tweaked code as needed for   *)
(*    new CUSTOM_EVENT types.                              *)
(*  - Set line length returned by printCUSTOM_EVENT() for  *)
(*    a terminal with 144 columns.                         *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.60                                    *)
(*  REVISION DATE: 06/09/2010  AT: 18:40:08                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Tweaked RETURN code for new functions.               *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.59                                    *)
(*  REVISION DATE: 05/16/2010  AT: 10:37:53                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*   New functions: __formatAscii__(),                     *)
(*   __formatAsciiHex__(), __formatDecimal__(),            *)
(*   __formatHex__(), __formatMIDI__().                    *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.58                                    *)
(*  REVISION DATE: 05/13/2010  AT: 12:34:39                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Removed 131 character line limit in subroutines with *)
(*    an "nLine" parameter.                                *)
(*    If  nLine > 131, longer messages may be truncated,   *)
(*    e.g. when dvDebug is the master (send_string 0).     *)
(*  - Updated ethernet connection types returned by        *)
(*    printDEVICE_INFO() and                               *)
(*   'SEND DEVICE_INFO TO DEBUG'().                        *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.57                                    *)
(*  REVISION DATE: 11/08/2008  AT: 13:17:15                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Speed Tweak:                                         *)
(*    * Change FORMAT('$%0 2X',data[X]) to                 *)
(*             FORMAT('$%02X',data[X])                     *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.56                                    *)
(*  REVISION DATE: 10/10/2008  AT: 18:52:56                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - added not-in-terminal parameter to                   *)
(*    printCustom_Event().                                 *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.55                                    *)
(*  REVISION DATE: 10/08/2008  AT: 09:04:35                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added functions printCUSTOM_EVENT() and              *)
(*    printDEVICE_INFO().                                  *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.54                                    *)
(*  REVISION DATE: 10/06/2008  AT: 08:01:21                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added __display_lib_version__ variable.              *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.53                                    *)
(*  REVISION DATE: 09/08/2008  AT: 17:24:28                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Moved DEV_TO_STRING and its variations to            *)
(*    DEV_TO_STRING.axi.                                   *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.52                                    *)
(*  REVISION DATE: 08/18/2008  AT: 05:49:49                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added call 'SEND MIDI TO DEBUG'.                     *)
(*  - Made __SPACES__ a volatile variable instead of a     *)
(*    constant.                                            *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.51, 2.50                              *)
(*  REVISION DATE: 08/13/2008  AT: 14:28:35                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - 'SEND...' calls were rewritten to handle very large  *)
(*    string arrays and string expressions that can output *)
(*    more than 15999 bytes of processed data.             *)
(*  - Large string parameters of 'SEND...' calls were set  *)
(*    to 15999 to workaround a feature that truncates      *)
(*    string expressions to 2048 bytes if passed in via an *)
(*    unbounded array.                                     *)
(*    With the parameter size set to 15999 they are not    *)
(*    truncated below this "normal" NetLinx interpreter    *)
(*    limit.                                               *)
(*    String arrays, for example the output of FILE_READ() *)
(*    or GET_BUFFER_STRING(), will still pass through      *)
(*    unaffected at 16000 bytes or greater.                *)
(*  - 'SEND ASCII TO MASTER' and 'SEND ITOHEX TO MASTER'   *)
(*    are now just wrappers that call                      *)
(*   'SEND ASCII TO DEBUG' and 'SEND ITOHEX TO DEBUG'      *)
(*    using the default settings.                          *)
(*  - CALL 'SEND 131 BYTE PACKETS TO MASTER' no longer     *)
(*    parses destructively, that is, it does not modify    *)
(*    the array passed in.                                 *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.14                                    *)
(*  REVISION DATE: 04/27/2008  AT: 11:14:35                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - renamed 'SEND 7BIT ASCII TO DEBUG' to                *)
(*    'SEND ASCII/HEX TO DEBUG'.                           *)
(*    For backwards compatibility, the old name is now a   *)
(*    wrapper that calls the new name.                     *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.13                                    *)
(*  REVISION DATE: 11/10/2007  AT: 17:18:44                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - tweaked 'SEND CUSTOM_EVENT DATA TO DEBUG' messages   *)
(*    to be more organized and more consistent with        *)
(*    'SEND DEVICE_INFO TO DEBUG'.                         *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: Rev 2.12                                *)
(*  REVISION DATE: 10/18/2007  AT: 22:00:11                *)
(*      - Fixed issue with '...ASCII...' calls where a     *)
(*        string  starting with single printable character *)
(*        would not print with a comma following that      *)
(*        character.                                       *)
(***********************************************************)
(*  FILE REVISION: 2.11                                    *)
(*  REVISION DATE: 09/26/2007  AT: 13:24:34                *)
(*      - Increased max size of string returned by         *)
(*        DEV_ARRAY_TO_STRING.                             *)
(***********************************************************)
(*  FILE REVISION: 2.10                                    *)
(*  REVISION DATE: 09/24/2007  AT: 14:59:42                *)
(*      - STR1, passed into 'SEND TO...' subroutines, is   *)
(*        no longer overwritten with spaces if the debug   *)
(*        message takes more than 1 line.                  *)
(*      - Changed SOURCE_TYPE_NO_ADDRESS message from      *)
(*        '...not available' to '...no address'.           *)
(*      - Code cleanup.                                    *)
(***********************************************************)
(*  FILE REVISION: Rev 2.09                                *)
(*  REVISION DATE: 01/09/2007  AT: 10:05:25                *)
(*      - Fixed incorrect use of 0:1:0 in                  *)
(*       'SEND CUSTOM_EVENT DATA TO DEBUG'.                *)
(***********************************************************)
(*  FILE REVISION: Rev 2.08                                *)
(*  REVISION DATE: 10/13/2006  AT: 17:21:44                *)
(*      - Use SOURCE_TYPE_INTERNAL ($14) instead of $20 in *)
(*        the call 'SEND DEVICE_INFO TO DEBUG'. The value  *)
(*        was corrected in NetLinx.axi v1.25.              *)
(***********************************************************)
(*  FILE REVISION: Rev 2.07                                *)
(*  REVISION DATE: 09/26/2006  AT: 18:22:52                *)
(*      - added subroutines for debugging CUSTOM_EVENT's.  *)
(***********************************************************)
(*  FILE REVISION: 2.06                                    *)
(*  REVISION DATE: 09/21/2006  AT: 13:20:35                *)
(*      - added DEV_ARRAY_TO_STRING().                     *)
(***********************************************************)
(*  FILE REVISION: 2.05                                    *)
(*  REVISION DATE: 08/10/2006  AT: 14:59:37                *)
(*      - remove comma, add 'L' prefix to level number     *)
(*        returned by DEVLEV_TO_STRING and                 *)
(*        DEV_LEV_TO_STRING. In addition, the level number *)
(*        is padded with 0's so it's always 3 digits.      *)
(***********************************************************)
(*  FILE REVISION: Rev 2.04 GLM                            *)
(*  REVISION DATE: 06/30/2006  AT: 16:06:33                *)
(*      - 'SEND DEVICE_INFO TO DEBUG' now trims garbage    *)
(*        from SN's less than 16 characters long.          *)
(***********************************************************)
(*  FILE REVISION: Rev 2.03 GLM                            *)
(*  REVISION DATE: 07/08/2005  AT: 15:12:54                *)
(*      - All "DEV..." functions now return the actual     *)
(*        system number, never 0...                        *)
(***********************************************************)
(*!!FILE REVISION: 2.02 GLM                                *)
(*  REVISION DATE: 05/31/2005                              *)
(*      - In the 'SEND...' calls, replace STR1 with an     *)
(*        equal number of spaces if a single reply takes   *)
(*        more than one line, after the first line.        *)
(*        So it's easier to distiguish between multiple    *)
(*        responses. Starting in v2.10 a copy of STR1 is   *)
(*        used so the passed-in STR1 is not changed.       *)
(***********************************************************)
(*!!FILE REVISION: 2.01 GLM                                *)
(*  REVISION DATE: 02/01/2005                              *)
(*      - added DPS_TO_STRING, similar to DEV_TO_STRING    *)
(*        but always returns actual system number.         *)
(*      - added 'STRING TO DEV' call. A call because       *)
(*        functions cannot return complex data types such  *)
(*        as DEVs.                                         *)
(*      - fixed problem where sometimes an extra ' would   *)
(*        sneak into 'SEND 7BIT ASCII TO DEBUG' responses. *)
(*      - 'SEND ASCII TO DEBUG' and 'SEND ASCII TO MASTER' *)
(*        now format DEL as $7F.                           *)
(*      - changed the 'SEND...' calls so they no longer    *)
(*        add a space between the header and data.         *)
(*      - added 'SEND DEVICE_INFO TO DEBUG'.               *)
(*      - rearranged calls alphabetically.                 *)
(***********************************************************)
(*!!FILE REVISION: 2.00 GLM                                *)
(*  REVISION DATE: 11/22/2004                              *)
(*      - use STACK_VAR instead of LOCAL_VAR.              *)
(*      - no longer has virtual device for control, it was *)
(*        never used in practice.                          *)
(***********************************************************)
(*!!FILE REVISION: 1.07 GLM                                *)
(*  REVISION DATE: 04/22/2004                              *)
(*      - added DEV_CHAN_TO_STRING, DEV_LEV_TO_STRING      *)
(*        these functions take a seperate dev,chan or      *)
(*        dev,lev as parameters.                           *)
(***********************************************************)
(*!!FILE REVISION: 1.06 GLM                                *)
(*  REVISION DATE: 09/15/2003                              *)
(*      - modified 'SEND INTEGER TO DEBUG' to only format  *)
(*        line length. It had been a misnamed version of   *)
(*        'SEND DECIMAL TO DEBUG'.                         *)
(***********************************************************)
(*!!FILE REVISION: 1.05 GLM                                *)
(*  REVISION DATE: 05/29/2003                              *)
(*      - added 'SEND 7BIT ASCII TO DEBUG'                 *)
(*        characters  >= $7F are formated as hex           *)
(*      - remove ; from #DEFINE __DEBUG_LIB__              *)
(*      - added explicit parameter types where missing     *)
(*      - added 'SEND DECIMAL TO DEBUG'. This is actually  *)
(*        what 'SEND INTEGER TO DEBUG' should have been    *)
(*        named.                                           *)
(***********************************************************)
(*!!FILE REVISION: 1.04 GLM                                *)
(*  REVISION DATE: 01/21/2003                              *)
(*      - added 'SEND INTEGER TO DEBUG'                    *)
(***********************************************************)
(*!!FILE REVISION: 1.03 GLM                                *)
(*  REVISION DATE: 08/28/2002                              *)
(*      Code cleanup:                                      *)
(*      - added line ends ";" for Studio debug mode.       *)
(*      - changed remaining LENGTH_STRING's to             *)
(*        LENGTH_ARRAY's.                                  *)
(*      - explicitly declare all variable types.           *)
(***********************************************************)
(*!!FILE REVISION: 1.02 GLM                                *)
(*  REVISION DATE: 04/27/2002                              *)
(*      - Added 'SEND ASCII TO DEBUG' and 'SEND ITOHEX TO  *)
(*        DEBUG'.                                          *)
(*        These calls have:                                *)
(*        - Device parameter to allow sending info to      *)
(*        devices other than 0:0:0, for example to virtual *)
(*        device for multi master debugging.               *)
(*        - Line length parameter to allow different line  *)
(*        lengths. If this is 0, line length defaults to   *)
(*        80 for mode  2, 67 for all other modes           *)
(*        - Mode parameter, if set to 2 appends <cr>,<lf>  *)
(*        to each line for master 'MSG ON 2'               *)
(*!!FILE REVISION: 1.01 GLM                                *)
(*  REVISION DATE: 04/27/2002                              *)
(*      - First line returned by the 'SEND ASCII...' and   *)
(*        'SEND ITOHEX...' calls is one less character     *)
(*        than subsequent lines to account for Netlinx '>' *)
(*        prompt. Removed in version 1.02.                 *)
(***********************************************************)
(*!!FILE REVISION: 1.00 GLM                                *)
(*  REVISION DATE: 03/26/2002                              *)
(*                                                         *)
(*  COMMENTS:                                              *)
(* DEBUG FUNCTIONS AND CALLS                               *)
/*  1   FUNCTION logString (DEV dvDebug, LONG lLogLevel, CHAR str[])          */
/*  2   FUNCTION printCUSTOM_EVENT (DEV dvDebug, CHAR hdr[], INTEGER flagNIT) */
/*  3   FUNCTION printToDebug (DEV dvDEBUG,hdr[],data[],nLine,nMode,nFormat)   */
/*  4   FUNCTION printMidiDebug (dvDEBUG,hdr[],data[],nLINE,nMODE,nStatus)     */
/*  5   FUNCTION printUrlList (dvDebug,dvURL,nType)                           */
/* Below are helper functions for the print<> functions.                          */
/*  6   FUNCTION CHAR[16] __CUSTOM_EVENT_JUSTIFY_TO_STRING__ (SLONG slJustify)    */
/*  7   FUNCTION CHAR[32] __CUSTOM_EVENT_TYPE_TO_STRING__ (LONG lCeType)          */
/*  8   FUNCTION CHAR[32] __CUSTOM_EVENT_VALUE2_TO_STRING__ (LONG lCeType)        */
/*  9   FUNCTION CHAR[32] __CUSTOM_EVENT_VALUE3_TO_STRING__ (LONG lCeType)        */
/* 10   FUNCTION LONG __formatAsciiHex__ (STR[], lenSTR, sTemp[], lenPacket, i)   */
/* 11   FUNCTION LONG __formatAsciiExt__ (STR[], lenSTR, sTemp[], lenPacket, i)   */
/* 12   FUNCTION LONG __formatDecimal__ (STR[], lenSTR, sTemp[], lenPacket, i)    */
/* 13   FUNCTION LONG __formatHex__ (STR[], lenSTR, sTemp[], lenPacket, i)        */
/* 14   FUNCTION LONG __formatMIDI__ (STR[], lenSTR, sTemp[], lenPacket, s, i)    */
/* 15   FUNCTION CHAR[64] __sURLFlags__ (Flg)                                     */
/* The old calls are just wrappers now, included as examples:                 */
/* 16   CALL 'SEND 131 BYTE PACKETS TO MASTER' (sSTRING[])                    */
/* 17   CALL 'SEND ASCII/HEX TO DEBUG' (DEV dvDEBUG,STR1[],STR2[],nLine,nMode)*/
/* 18   CALL 'SEND ASCII TO DEBUG' (DEV dvDEBUG,STR1[],STR2[],nLine,nMode)    */
/* 19   CALL 'SEND ASCII TO MASTER' (STR1[],STR2[])                           */
/* 20   CALL 'SEND CUSTOM_EVENT DATA TO DEBUG' (DEV dvDebug)                  */
/* 21   CALL 'SEND DECIMAL TO DEBUG' (DEV dvDEBUG,STR1[],STR2[],nLine,nMode)  */
/* 22   CALL 'SEND DEVICE_INFO TO DEBUG' (DEV dvDEBUG, DEV dvDEV)             */
/* 23   CALL 'SEND INTEGER TO DEBUG' (DEV dvDEBUG,STR1[],STR2[],nLine,nMode)  */
/* 24   CALL 'SEND ITOHEX TO DEBUG' (DEV dvDEBUG,STR1[],STR2[],nLine,nMode)   */
/* 25   CALL 'SEND ITOHEX TO MASTER' (STR1[],STR2[])                          */
/* 26   CALL 'SEND MIDI TO DEBUG' (dvDEBUG,STR1[],STR2[],nLINE,nMODE,nStatus) */
(***********************************************************)
DEFINE_VARIABLE                     // compiler directive #IF... and #INCLUDE
#IF_NOT_DEFINED __DEBUG_LIB__
#DEFINE __DEBUG_LIB__

#INCLUDE 'DEV_TO_STRING'
#INCLUDE 'strings'
(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE

(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

// VERSION FOR CODE

CHAR __DEBUG_LIB_NAME__[20]     = '           debug.axi'
CHAR __DEBUG_LIB_VERSION__[]    = '4.01';

// printToDebug() values for nFormat
PRINT_UNFORMATTED = 0
PRINT_ASCII_HEX   = 1
PRINT_HEX         = 2
PRINT_DECIMAL     = 3
PRINT_ASCII_EXT   = 4

// bitmap values for nMode

BMP_MODE_MIDI_SYSEX_ASCII = $1
BMP_MODE_CLI_2       = $2

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

VOLATILE CHAR __SPACES__[128] =
{
  $20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,
  $20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,
  $20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,
  $20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,
  $20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,
  $20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,
  $20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,
  $20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20
}

#IF_NOT_DEFINED __display_lib_version__
VOLATILE __display_lib_version__ = TRUE
#END_IF

#IF_NOT_DEFINED __V4_or_NX_master__
VOLATILE INTEGER __V4_or_NX_master__ = 0
#END_IF

#IF_NOT_DEFINED __V4_and_NX_check__
VOLATILE INTEGER __V4_and_NX_check__ = 0
#END_IF

(***********************************************************)
(*           SUBROUTINE DEFINITIONS GO BELOW               *)
(***********************************************************)
(* EXAMPLE: DEFINE_FUNCTION <RETURN_TYPE> <NAME> (<PARAMETERS>) *)
(* EXAMPLE: DEFINE_CALL '<NAME>' (<PARAMETERS>) *)

                                    ///////////////////////////
                                    // DEBUG FUNCTIONS/CALLS //
                                    ///////////////////////////
#IF_NOT_DEFINED __logString__
#DEFINE __logString__
DEFINE_FUNCTION logString(DEV dvDebug, LONG lLogLevel, CHAR str[])
{
  IF (dvDebug.NUMBER == 0)          // if dvDebug is a master...
  {
	IF (__V4_or_NX_master__)        // and if it's a newer NetLinx master
	{
	  AMX_LOG(lLogLevel,str)
	}
	ELSE                            // for older masters, send_string 0
	{
	  SEND_STRING dvDebug,str
	}
  }
  ELSE
  {
	SEND_STRING dvDebug,str
  }
}
#END_IF
DEFINE_FUNCTION printCUSTOM_EVENT (DEV dvDebug, CHAR hdr[], INTEGER nCols, INTEGER nMode)
// FUNCTION printCUSTOM_EVENT (DEV dvDebug, CHAR hdr[])
// This subroutine must be called inside of a CUSTOM_EVENT
// Purpose: Print CUSTOM_EVENT data to dvDebug
//          Difference from CALL 'SEND CUSTOM_EVENT DATA TO DEBUG':
//          1. Header parameter for companion debug module v2.xx and higher.
//             Header is prepended to each line.
// PARAMETERS:
//         dvDEBUG IS DESTINATION
//         hdr IS HEADER
//         nLine IS LINE LENGTH. 0 = default = 80 for nMode 2, 67 for all others.
//         nMode is MSG ON mode. 2 = append <cr>,<lf> to lines.
{
  STACK_VAR CHAR sTerm[2];

  IF (nMode & 2) { sTerm = "13,10" }
  ELSE           { sTerm = "" }

									/****************/
									/* DEVCHAN INFO */
									/****************/
  logString(dvDebug,AMX_ERROR,"hdr,'D:P:S ',DEV_TO_STRING(CUSTOM.DEVICE),sTerm")

									/**************************/
									/* ID, TYPE, FLAG, VALUE1 */
									/**************************/
  SWITCH (CUSTOM.TYPE)
  {
	CASE 600:                       // Gestures
	{
	 STACK_VAR CHAR sTxt[32]
	 logString(dvDebug,AMX_ERROR,"hdr,'ID ',ITOA(CUSTOM.ID),sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'TYPE ',ITOA(CUSTOM.TYPE),' ',__CUSTOM_EVENT_TYPE_TO_STRING__(CUSTOM.TYPE),sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'FLAG ',ITOA(CUSTOM.FLAG),sTerm")
	 SWITCH (CUSTOM.VALUE1)
	 {
		CASE 1: { sTxt = 'SWIPE UP' }
		CASE 2: { sTxt = 'SWIPE DOWN' }
		CASE 3: { sTxt = 'SWIPE RIGHT' }
		CASE 4: { sTxt = 'SWIPE LEFT' }
		CASE 5: { sTxt = 'CLOCKWISE CIRCLE' }
		CASE 6: { sTxt = 'COUNTER-CLOCKWISE CIRCLE' }
		CASE 7: { sTxt = 'DOUBLE-TAP' }
		CASE 8: { sTxt = '2-FINGER SWIPE UP' }
		CASE 9: { sTxt = '2-FINGER SWIPE DOWN' }
		CASE 10: { sTxt = '2-FINGER SWIPE RIGHT' }
		CASE 11: { sTxt = '2-FINGER SWIPE LEFT' }
	 }
	 logString(dvDebug,AMX_ERROR,"hdr,'VALUE1 ',ITOA(CUSTOM.VALUE1),' ',sTxt,sTerm")
	}
	CASE 700:                       // NFC (Near Field Communication)
	{
	 STACK_VAR CHAR sTxt[16]
	 logString(dvDebug,AMX_ERROR,"hdr,'ID ',ITOA(CUSTOM.ID),sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'TYPE ',ITOA(CUSTOM.TYPE),__CUSTOM_EVENT_TYPE_TO_STRING__(CUSTOM.TYPE),sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'FLAG ',ITOA(CUSTOM.FLAG),sTerm")
	 SWITCH (CUSTOM.VALUE1)
	 {
		CASE 1: { sTxt = 'ISO 15693' }
		CASE 2: { sTxt = 'ISO 14443A' }
		CASE 3: { sTxt = 'ISO 14443B' }
	 }
	 logString(dvDebug,AMX_ERROR,"hdr,'VALUE1 ',ITOA(CUSTOM.VALUE1),' Tag Type',sTxt,sTerm")
	}
	CASE 710:                       // USB SMART CARD
	{
	 logString(dvDebug,AMX_ERROR,"hdr,'ID ',ITOA(CUSTOM.ID),sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'TYPE ',__CUSTOM_EVENT_TYPE_TO_STRING__(CUSTOM.TYPE),sTerm")
	 IF (CUSTOM.FLAG) { logString(dvDebug,AMX_ERROR,"hdr,'FLAG ',ITOA(CUSTOM.FLAG),' INSERTED',sTerm") }
	 ELSE             { logString(dvDebug,AMX_ERROR,"hdr,'FLAG ',ITOA(CUSTOM.FLAG),' REMOVED',sTerm") }
	 logString(dvDebug,AMX_ERROR,"hdr,'PART ',ITOA(CUSTOM.VALUE1),'(V1) OF ',ITOA(CUSTOM.VALUE2),'(V2)',sTerm")
	}
	CASE 768:                       // Streaming Video Status Changes
	{
	 STACK_VAR CHAR sTxt[16]
	 logString(dvDebug,AMX_ERROR,"hdr,'ID ',ITOA(CUSTOM.ID),sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'TYPE ',__CUSTOM_EVENT_TYPE_TO_STRING__(CUSTOM.TYPE),sTerm")
	 SWITCH (CUSTOM.FLAG)
	 {
		CASE 1: { sTxt = 'START' }
		CASE 2: { sTxt = 'STOP' }
		CASE 8: { sTxt = 'ERROR' }
	 }
	 logString(dvDebug,AMX_ERROR,"hdr,'FLAG ',ITOA(CUSTOM.FLAG),' ',sTxt,sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'VALUE1 ',ITOA(CUSTOM.VALUE1),' ',sTxt,'(S)',sTerm")
	}
	CASE 1001:                      // user button text
	CASE 1016:                      // setup button text
	{
	 logString(dvDebug,AMX_ERROR,"hdr,'ID ',ITOA(CUSTOM.ID),' BUTTON ADDRESS',sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'TYPE ',__CUSTOM_EVENT_TYPE_TO_STRING__(CUSTOM.TYPE),sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'FLAG ',ITOA(CUSTOM.FLAG),' UNICODE',sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'VALUE1 ',ITOA(CUSTOM.VALUE1),' BUTTON STATE',sTerm")
	}
	                                // other button address queries
	CASE 1002: CASE 1003: CASE 1004: CASE 1005: CASE 1006: CASE 1007: CASE 1008:
	CASE 1009: CASE 1010: CASE 1011: CASE 1012: CASE 1013: CASE 1014: CASE 1015:
	{
	 logString(dvDebug,AMX_ERROR,"hdr,'ID ',ITOA(CUSTOM.ID),' BUTTON ADDRESS',sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'TYPE ',__CUSTOM_EVENT_TYPE_TO_STRING__(CUSTOM.TYPE),sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'FLAG ',ITOA(CUSTOM.FLAG)")
	 logString(dvDebug,AMX_ERROR,"hdr,'VALUE1 ',ITOA(CUSTOM.VALUE1),' BUTTON STATE',sTerm")
	}
	CASE 1301:                     // Frame Buffer CRC (G4, G3?; not supported in G5)
	{
	 logString(dvDebug,AMX_ERROR,"hdr,'ID ',ITOA(CUSTOM.ID),sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'TYPE ',__CUSTOM_EVENT_TYPE_TO_STRING__(CUSTOM.TYPE),sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'FLAG ',ITOA(CUSTOM.FLAG),sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'VALUE1 $',ITOHEX(CUSTOM.VALUE1),' (',ITOA(CUSTOM.VALUE1),')',sTerm")
	}
	CASE 1310:                      // XML PANEL STATS
	CASE 1311:                      // XML PANEL LOGS
	{
	 logString(dvDebug,AMX_ERROR,"hdr,'ID ',ITOA(CUSTOM.ID),sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'TYPE ',__CUSTOM_EVENT_TYPE_TO_STRING__(CUSTOM.TYPE),sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'FLAG ',ITOA(CUSTOM.FLAG),sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'PART ',ITOA(CUSTOM.VALUE1),'(V1) OF ',ITOA(CUSTOM.VALUE2),'(V2)',sTerm")
	}
	CASE 1325:                      // Audio Device
	{
	 STACK_VAR CHAR sTxt[24]
	 logString(dvDebug,AMX_ERROR,"hdr,'ID ',ITOA(CUSTOM.ID),sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'TYPE ',__CUSTOM_EVENT_TYPE_TO_STRING__(CUSTOM.TYPE),sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'FLAG ',ITOA(CUSTOM.FLAG),sTerm")
	 SWITCH (CUSTOM.VALUE1)
	 {
		CASE 1: { sTxt = 'Built-in Speaker' }
		CASE 2: { sTxt = 'USB Headset' }
		CASE 3: { sTxt = 'Bluetooth Headset' }
	 }
	 logString(dvDebug,AMX_ERROR,"hdr,'VALUE1 ',ITOA(CUSTOM.VALUE1),' ',sTxt,sTerm")
	}
	CASE 1326:                      // port 0 level (G4 ModeroX)
	{
	 logString(dvDebug,AMX_ERROR,"hdr,'ID ',ITOA(CUSTOM.ID),' BUTTON LEVEL',sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'TYPE ',__CUSTOM_EVENT_TYPE_TO_STRING__(CUSTOM.TYPE),sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'FLAG ',ITOA(CUSTOM.FLAG),sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'VALUE1 ',ITOA(CUSTOM.VALUE1),' LEVEL STATE',sTerm")
	}
	CASE 1327:                      // port 0 channel state (G4 ModeroX)
	{
	 logString(dvDebug,AMX_ERROR,"hdr,'ID ',ITOA(CUSTOM.ID),' BUTTON CHANNEL',sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'TYPE ',__CUSTOM_EVENT_TYPE_TO_STRING__(CUSTOM.TYPE),sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'FLAG ',ITOA(CUSTOM.FLAG),sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'VALUE1 ',ITOA(CUSTOM.VALUE1),' CHANNEL STATE',sTerm")
	}
	DEFAULT:
	{
	 logString(dvDebug,AMX_ERROR,"hdr,'ID ',ITOA(CUSTOM.ID),sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'TYPE ',__CUSTOM_EVENT_TYPE_TO_STRING__(CUSTOM.TYPE),sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'FLAG ',ITOA(CUSTOM.FLAG),sTerm")
	 logString(dvDebug,AMX_ERROR,"hdr,'VALUE1 ',ITOA(CUSTOM.VALUE1),sTerm")
	}
  }
									/**********/
									/* VALUE2 */
									/**********/
									// if this is not a multipart response (exception: types >= 32001)
  IF ((CUSTOM.TYPE <> 710) && (CUSTOM.TYPE <> 1310) && (CUSTOM.TYPE <> 1311))
  {
	logString(dvDebug,AMX_ERROR,"hdr,'VALUE2 ',ITOA(CUSTOM.VALUE2),' ',__CUSTOM_EVENT_VALUE2_TO_STRING__(CUSTOM.TYPE),sTerm")
  }
  ELSE
  {
	// multipart responses print value 2 above: 'PART <value1> OF <value2>'
  }
									/**********/
									/* VALUE3 */
									/**********/
  logString(dvDebug,AMX_ERROR,"hdr,'VALUE3 ',ITOA(CUSTOM.VALUE3),' ',__CUSTOM_EVENT_VALUE3_TO_STRING__(CUSTOM.TYPE),sTerm")

									/********/
									/* TEXT */
									/********/
  logString(dvDebug,AMX_ERROR,"hdr,'LENGTH_ARRAY(TEXT) ',ITOA(LENGTH_ARRAY(CUSTOM.TEXT)),sTerm")

  IF (nMode & BMP_MODE_CLI_2)       // if result may be parsed by code
  {
									// Put header and description in STR2 so printToDebug()
									// won't indent if there are multiple lines
	printToDebug (dvDebug,"","hdr,'TEXT ',CUSTOM.TEXT",nCols,nMode,PRINT_UNFORMATTED);
	printToDebug (dvDebug,"","hdr,'ENCODE ',CUSTOM.ENCODE",nCols,nMode,PRINT_UNFORMATTED);
  }
  ELSE                              // if result is to be read in via master debug terminal
  {
	printToDebug (dvDebug,"hdr,'TEXT '",CUSTOM.TEXT,nCols,nMode,PRINT_ASCII_HEX);
	printToDebug (dvDebug,"hdr,'ENCODE '",CUSTOM.ENCODE,nCols,nMode,PRINT_ASCII_HEX);
  }
}
DEFINE_FUNCTION printToDebug(DEV dvDEBUG,CHAR hdr[128],CHAR data[],INTEGER nLINE,INTEGER nMODE,INTEGER nFormat)
/* FUNCTION printToDebug()
 * PURPOSE: prints data to debug device, formatted in a convenient way
 * PARAMETERS:
 *     dvDEBUG IS DESTINATION
 *     hdr IS HEADER
 *     data IS DATA TO BE FORMATTED
 *     nLine IS LINE LENGTH. 0 = default = 80 for nMode 2, 67 for all others.
 *     nMode is MSG ON mode. 2 = append <cr>,<lf> to lines.
 *     nFormat 0: Does not format, sets line length only
 *             1: Printable ASCII unformated, inside single-quotes
 *                Chars >= $7F (del) as 2 digit hex, seperated by commas
 *                Chars <  $20  as 2 digit hex, seperated by commas
 *             2: All chars as 2 digit hex, seperated by commas
 *             3: All chars as 3 digit decimal, seperated by commas
 *             4: Printable ASCII and extended ASCII (>= $80) unformatted, inside single quotes
 *                Prints control characters as 2 digit hex, seperated by commas
 */
{
  STACK_VAR	integer header_len;
  STACK_VAR LONG data_len;
  STACK_VAR CHAR header[128];
  STACK_VAR CHAR sTerm[2];

  header_len = LENGTH_ARRAY(hdr);
  data_len = LENGTH_ARRAY(data);
  header = hdr;

  IF (nMode & BMP_MODE_CLI_2) { sTerm = "13,10" }
  ELSE                              { sTerm = "" }

  IF (data_len)
  {
	STACK_VAR LONG X;
	STACK_VAR char ASCII[15999];
	STACK_VAR integer ASCII_PACKET_LEN;
	STACK_VAR INTEGER FIRST_STRING_SENT;
	STACK_VAR LONG len_to_get

	SELECT                          // SET THE ASCII PACKET LENGTH
	{
//    If  nLine > 131, longer messages may be truncated
//    e.g. when dvDebug is the master (send_string 0).
	  ACTIVE (nLINE > MAX_LENGTH_ARRAY(ASCII)):
	  {
		ASCII_PACKET_LEN = MAX_LENGTH_ARRAY(ASCII) - header_len;
	  }
	  ACTIVE (nLINE > header_len):
	  {
		ASCII_PACKET_LEN = nLINE - header_len;
	  }
									// MODE 2 DEFAULT TO 80 CHARACTER LINE
	  ACTIVE (nMODE & BMP_MODE_CLI_2):
	  {
		ASCII_PACKET_LEN = 80 - header_len;
	  }
	  ACTIVE (1):                   // DEFAULT TO 67 CHARACTER LINE
	  {
		ASCII_PACKET_LEN = 67 - header_len;
	  }
	}
									// BUILD UP ASCII STRING FROM data
	FOR (X = 1; X <= data_len;      /* x is incremented via format subroutines */ )
	{
	  SWITCH (nFormat)
	  {
		CASE PRINT_UNFORMATTED:
		{
									// need to limit len_to_get for masters below v3.21.354
		  IF ((data_len - X - 1) > ASCII_PACKET_LEN) { len_to_get = ASCII_PACKET_LEN }
		  ELSE                                       { len_to_get = data_len }

		  ASCII = MID_STRING(data,X,len_to_get)
		  X = X + LENGTH_ARRAY(ASCII)
		}
		CASE PRINT_ASCII_HEX: { X = __formatAsciiHex__ (data,data_len,ASCII,ASCII_PACKET_LEN,X) }
		CASE PRINT_HEX:       { X = __formatHex__ (data,data_len,ASCII,ASCII_PACKET_LEN,X) }
		CASE PRINT_DECIMAL:   { X = __formatDecimal__ (data,data_len,ASCII,ASCII_PACKET_LEN,X) }
		CASE PRINT_ASCII_EXT: { X = __formatAsciiExt__ (data,data_len,ASCII,ASCII_PACKET_LEN,X) }
	  }
									// if the line is ready, send it
	  IF (LENGTH_ARRAY(ASCII) >= ASCII_PACKET_LEN)
	  {
		logString(dvDebug,AMX_ERROR,"header,GET_BUFFER_STRING(ASCII,ASCII_PACKET_LEN),sTerm")

		IF (!FIRST_STRING_SENT)		// replace header with spaces after first pass
		{
		  ON[FIRST_STRING_SENT]
		  header = LEFT_STRING(__SPACES__,LENGTH_STRING(header))
		}
	  }
	}

	IF (LENGTH_ARRAY(ASCII))        // IF THERE'S ANYTHING LEFT SEND IT
	{
	  logString(dvDebug,AMX_ERROR,"header,ASCII,sTerm")
	}
  }
  ELSE {                            // if data was empty, print "<header>''"
	logString(dvDebug,AMX_ERROR,"header,'''''',sTerm")
  }
}
DEFINE_FUNCTION printMidiDebug (DEV dvDEBUG,CHAR hdr[128],CHAR data[],INTEGER nLINE,INTEGER nMODE, INTEGER nSTATUS)
// FUNCTION printMidiDebug()
// PURPOSE - Format MIDI channel voice messages with:
//              MIDI status bytes as 2 digit hex, seperated by commas.
//              MIDI data bytes as 3 digit decimal, seperated by commas.
//         - Formats MIDI system messages as 2 digit hex, seperated by commas.
// PARAMETERS:
// dvDEBUG IS DESTINATION
// hdr IS header
// data IS DATA TO BE FORMATTED
// nLine IS LINE LENGTH. 0 = default = 80 for nMode 2, 67 for all others.
// nMode is bitmapped flags:
//          - $1 Print SYSEX data as ASCII
//          - $2 MSG ON mode 2, append <cr>,<lf> to lines.
// nStatus is MIDI status byte:
//         - $EF and lower = default = format status bytes as hex, data bytes as decimal.
//         - $F0 and higher = format status and data as hex.
{
  STACK_VAR	integer header_len;
  STACK_VAR LONG data_len;
  STACK_VAR CHAR header[128];
  STACK_VAR CHAR sTerm[2];

  header_len = LENGTH_ARRAY(hdr);
  data_len = LENGTH_ARRAY(data);
  header = hdr;

  IF (nMode & BMP_MODE_CLI_2) { sTerm = "13,10" }
  ELSE                              { sTerm = "" }

  IF (data_len)
  {
	STACK_VAR LONG X;
	STACK_VAR char ASCII[15999];
	STACK_VAR integer ASCII_PACKET_LEN;
	STACK_VAR INTEGER FIRST_STRING_SENT;
	STACK_VAR LONG len_to_get

	SELECT                            // SET THE ASCII PACKET LENGTH
	{
  //    If  nLine > 131, longer messages may be truncated
  //    e.g. when dvDebug is the master (send_string 0).
	  ACTIVE (nLINE > MAX_LENGTH_ARRAY(ASCII)):
	  {
		ASCII_PACKET_LEN = MAX_LENGTH_ARRAY(ASCII) - header_len;
	  }
	  ACTIVE (nLINE > header_len):
	  {
		ASCII_PACKET_LEN = nLINE - header_len;
	  }
									// MODE 2 DEFAULT TO 80 CHARACTER LINE
	  ACTIVE (nMODE & BMP_MODE_CLI_2):
	  {
		ASCII_PACKET_LEN = 80 - header_len;
	  }
	  ACTIVE (1):                     // DEFAULT TO 67 CHARACTER LINE
	  {
		ASCII_PACKET_LEN = 67 - header_len;
	  }
	}

									  // BUILD UP ASCII STRING FROM data
	FOR (X = 1; X <= data_len;/* x is incremented via __formatMIDI__() */ )
	{
	  X = __formatMIDI__(data,data_len,ASCII,ASCII_PACKET_LEN,nMODE,nSTATUS,X)

									  // if the line is ready, send it
	  IF (LENGTH_ARRAY(ASCII) >= ASCII_PACKET_LEN)
	  {
		logString(dvDebug,AMX_ERROR,"header,GET_BUFFER_STRING(ASCII,ASCII_PACKET_LEN),sTerm")

		IF (!FIRST_STRING_SENT)		// replace header with spaces after first pass
		{
		  ON[FIRST_STRING_SENT]
		  header = LEFT_STRING(__SPACES__,LENGTH_STRING(header))
		}
	  }
	}

	IF (LENGTH_ARRAY(ASCII))          // IF THERE'S ANYTHING LEFT SEND IT
	{
	  logString(dvDebug,AMX_ERROR,"header,ASCII,sTerm")
	}
  }
  ELSE {                            // if data was empty, print "<header>''"
	logString(dvDebug,AMX_ERROR,"header,'''''',sTerm")
  }
}
DEFINE_FUNCTION printUrlList (DEV dvDebug, DEV dvURL, INTEGER nType)
/*
 * Function printUrlList()
 * Parameters:
 *   dvDebug:  destination for function output
 *   dvURL:    device to check for URL list
 *   nType:    0 = All URLs
 *             1 = NetLinx programmed URLs
 *             2 = IDE programmed URLs
 */
{
  STACK_VAR URL_STRUCT uURL_ENTRY[200]
  STACK_VAR SLONG sl
  sl = GET_URL_LIST(dvURL,uURL_ENTRY,nType)
  IF (sl > 0)
  {
	STACK_VAR INTEGER i
	FOR (i = ABS_VALUE(sl); i; i--)
	{
	  logString(dvDebug,AMX_ERROR,"DEV_TO_STRING(dvURL),' URL[',ITOA(i),'].Flags $',FORMAT('%02X',uURL_ENTRY[i].Flags),':',__sURLFlags__(uURL_ENTRY[i].Flags)")
	  logString(dvDebug,AMX_ERROR,"DEV_TO_STRING(dvURL),' URL[',ITOA(i),'].Port ',FORMAT('%lu',uURL_ENTRY[i].Port)")
	  logString(dvDebug,AMX_ERROR,"DEV_TO_STRING(dvURL),' URL[',ITOA(i),'].URL ''',uURL_ENTRY[i].URL,''''")
	  // User info was added in Master v3.0.316, NETLINX_AXI_VERSION 1.21, NS v2.3.97
	  IF (uURL_ENTRY[i].Flags & URL_Flg_AcctInfoPresent)
	  {
		logString(dvDebug,AMX_ERROR,"DEV_TO_STRING(dvURL),' URL[',ITOA(i),'].User ''',uURL_ENTRY[i].User,''''")
		logString(dvDebug,AMX_ERROR,"DEV_TO_STRING(dvURL),' URL[',ITOA(i),'].Password ''',uURL_ENTRY[i].Password,''''")
	  }
	}
  }
  ELSE IF (sl = 0) {
	SWITCH (nType)
	{
	  CASE 2:  { logString(dvDebug,AMX_ERROR,"DEV_TO_STRING(dvURL),' No IDE URLs in URL list'") }
	  CASE 1:  { logString(dvDebug,AMX_ERROR,"DEV_TO_STRING(dvURL),' No NetLinx URLs in URL list'") }
	  DEFAULT: { logString(dvDebug,AMX_ERROR,"DEV_TO_STRING(dvURL),' No URLs in URL list'") }
	}
  }
  ELSE IF (sl = -1) {
	logString(dvDebug,AMX_ERROR,"DEV_TO_STRING(dvURL),' URL list device invalid or is not online'")
  }
  ELSE IF (sl = -2) {
	logString(dvDebug,AMX_ERROR,"DEV_TO_STRING(dvURL),' URL list request timed out'")
  }
  ELSE {
	logString(dvDebug,AMX_ERROR,"DEV_TO_STRING(dvURL),' URL list error ',FORMAT('%d',sl)")
  }
}

DEFINE_FUNCTION CHAR[32] __CUSTOM_EVENT_TYPE_TO_STRING__ (LONG lCeType)
// FUNCTION __CUSTOM_EVENT_TYPE_TO_STRING__ (LONG lCeType)
// PARAMETER:
// lCeType - CUSTOM.TYPE from CUSTOM.EVENT
// RETURNS:
// CUSTOM.TYPE name string
{
 SWITCH (lCeType)
 {
	CASE  600: RETURN "'GESTURE'";
	CASE  700: RETURN "'NFC TAG'";
	CASE  710: RETURN "'SMART CARD'";
	CASE  768: RETURN "'STREAMING VIDEO'";

	CASE 1001: RETURN "'TEXT'";
	CASE 1002: RETURN "'BITMAP'";
	CASE 1003: RETURN "'ICON'";
	CASE 1004: RETURN "'TEXT JUSTIFICATION'";
	CASE 1005: RETURN "'BITMAP JUSTIFICATION'";
	CASE 1006: RETURN "'ICON JUSTIFICATION'";
	CASE 1007: RETURN "'FONT'";
	CASE 1008: RETURN "'TEXT EFFECT NAME'";
	CASE 1009: RETURN "'TEXT EFFECT COLOR'";
	CASE 1010: RETURN "'WORD WRAP'";
	CASE 1011: RETURN "'BORDER COLOR'";
	CASE 1012: RETURN "'FILL COLOR'";
	CASE 1013: RETURN "'TEXT COLOR'";
	CASE 1014: RETURN "'BORDER NAME'";
	CASE 1015: RETURN "'OPACITY'";
	CASE 1016: RETURN "'PORT 0 TEXT'";

	CASE 1301: RETURN "'FRAME BUFFER CRC'";
	CASE 1302: RETURN "'DISPLAY TIMEOUT'";
	CASE 1303: RETURN "'PANEL BRIGHTNESS'";
	CASE 1304: RETURN "'PANEL INFO'";
	CASE 1305: RETURN "'MUTE'";
	CASE 1306: RETURN "'MASTER VOLUME'";
	CASE 1307: RETURN "'SHUTDOWN TIMEOUT'";
	CASE 1308: RETURN "'CHARGING STATUS'";
	CASE 1309: RETURN "'WIRELESS INFO'";
	CASE 1310: RETURN "'XML PANEL STATS'";
	CASE 1311: RETURN "'XML PANEL LOGS'";
	CASE 1312: RETURN "'LAMP HOURS'";
	CASE 1313: RETURN "'BATTERY BASE VERSION'";
	CASE 1314: RETURN "'ANALOG MIC OUT'";
	CASE 1315: RETURN "'MAC'";

	CASE 1322: RETURN "'USB'";
	CASE 1325: RETURN "'AUDIO DEVICE USED'";
	CASE 1326: RETURN "'PORT 0 LEVEL STATE'";
	CASE 1327: RETURN "'PORT 0 CHANNEL STATE'";
	CASE 1400: RETURN "'DYNAMIC IMAGE LOADED'";
	DEFAULT:   RETURN "''";
 }
}

DEFINE_FUNCTION CHAR[32] __CUSTOM_EVENT_VALUE2_TO_STRING__ (LONG lCeType)
// FUNCTION __CUSTOM_EVENT_VALUE2_TO_STRING__ (LONG lCeType)
// PARAMETER:
// lCeType - CUSTOM.TYPE from CUSTOM.EVENT
// RETURNS:
// VALUE2 name string
{
 SWITCH (lCeType)
 {
	CASE 600:               // Gesture simplified velocity
	{
	 SWITCH (CUSTOM.VALUE2)
	 {
		CASE 1: { RETURN ' FAST'; }
		CASE 2: { RETURN ' NORMAL'; }
		CASE 3: { RETURN ' SLOW'; }
	 }
	}
	CASE 700:               // NFC Tag - data type 0 is ASCII, data type 1 is actual values
	{
	 SWITCH (CUSTOM.VALUE2)
	 {
	 	CASE 0: { RETURN 'TEXT ASCII'; }
	  CASE 1: { RETURN 'TEXT RAW DATA'; }
	 }
	}
	CASE 710: RETURN 'TOTAL PARTS';// Smart Card
	CASE 768: RETURN 'BUTTON ADDRESS';// Streaming video
	CASE 1001: RETURN 'TEXT LENGTH';
	CASE 1002: RETURN 'TEXT LENGTH';
	CASE 1003: RETURN 'ICON INDEX';
	CASE 1004:                        // text justification
	CASE 1005:                        // bitmap justification
	CASE 1006:                        // icon justification
	{
	 SWITCH (CUSTOM.VALUE2)
	 {
		CASE 1: RETURN 'TOP LEFT';
		CASE 2: RETURN 'TOP CENTER';
		CASE 3: RETURN 'TOP RIGHT';
		CASE 4: RETURN 'MIDDLE LEFT';
		CASE 5: RETURN 'MIDDLE CENTER';
		CASE 6: RETURN 'MIDDLE RIGHT';
		CASE 7: RETURN 'BOTTOM LEFT';
		CASE 8: RETURN 'BOTTOM CENTER';
		CASE 9: RETURN 'BOTTOM RIGHT';
	 }
	}
	CASE 1007: RETURN 'FONT INDEX';
	CASE 1008: RETURN 'TEXT LENGTH';
	CASE 1009: RETURN 'TEXT LENGTH';
	CASE 1010: RETURN 'WORD WRAP';
	CASE 1011: RETURN 'TEXT LENGTH';
	CASE 1012: RETURN 'TEXT LENGTH';
	CASE 1013: RETURN 'TEXT LENGTH';
	CASE 1014: RETURN 'TEXT LENGTH';
	CASE 1015: RETURN 'OPACITY VALUE';
	CASE 1016: RETURN 'TEXT LENGTH';
	CASE 1310: RETURN 'TOTAL PARTS'; // XML STATS
	CASE 1311: RETURN 'TOTAL PARTS'; // XML LOGS
	DEFAULT:   RETURN '';
 }
}
DEFINE_FUNCTION CHAR[32] __CUSTOM_EVENT_VALUE3_TO_STRING__ (LONG lCeType)
// FUNCTION __CUSTOM_EVENT_VALUE3_TO_STRING__ (LONG lCeType)
// PARAMETER:
// lCeType - CUSTOM.TYPE from CUSTOM.EVENT
// RETURNS:
// VALUE3 name string
{
 SWITCH (lCeType)
 {
	CASE 600:                       // Gesture precise velocity
	{
																	// double tap
	  IF (CUSTOM.VALUE1 = 7) { RETURN 'ms'; }
	  ELSE                   { RETURN 'pixels/second'; }
	}
	CASE 700: RETURN 'BYTES of DATA'; // NFC Tag
	CASE 710: RETURN 'TOTAL LENGTH';  // Smart Card
	CASE 768: RETURN 'BUTTON PORT';   // Streaming video
	CASE 1001: RETURN 'TEXT INDEX';   // ?TXT
	CASE 1016: RETURN 'TEXT INDEX';   // ?STX
	CASE 1310: RETURN 'TOTAL LENGTH'; // XML STATS
	CASE 1311: RETURN 'TOTAL LENGTH'; // XML LOGS

	DEFAULT:   RETURN '';
 }
}
DEFINE_FUNCTION LONG __formatAsciiHex__(char STR[], long lenSTR, char sTemp[], integer lenPacket, long i)
// FUNCTION: __formatAsciiHex__()
// Purpose:
//  - Create one ASCII packet of length lenPacket from STR, without destroying STR contents
//  - Can be called recursively if STR is too long to be formatted into one packet
//  - PASSES ASCII CHARACTERS UNFORMATTED, ASCII STRINGS INSIDE SINGLE-QUOTES
//  - FORMATS CHARS >= $7F (del) AS 2 DIGIT HEX, SEPERATED BY COMMAS
//  - FORMATS CONTROL CHARACTERS AS 2 DIGIT HEX, SEPERATED BY COMMAS
//  - ASCII STRINGS SEPERATED FROM HEX BY COMMAS
// PARAMETERS:
//  - STR:       data to be formatted
//  - lenSTR:    length of data to be formatted
//  - sTemp:     temp storage for formatted data
//  - lenPacket: maximum desired length of formatted data
//  - i:         current index into str[]
// RETURNS:
//  - i
{
 IF (i = 0) { i = 1 }               // NetLinx indexing starts at 1, not 0
 FOR (; i <= lenSTR; i++)
 {
                                    // IF IT'S A CONTROL CHARACTER
                                    // OR DEL OR EXTENDED ASCII
	IF (STR[i] < $20 || STR[i] >= $7F)
	{
	 IF (i = lenSTR)                  // IF IT'S THE LAST BYTE
	 {
                                    // JUST PRINT $XX
		sTemp = "sTemp,FORMAT('$%02X',STR[i])";
	 }
                                    // IF THE NEXT BYTE IS A CONTROL CHARACTER
                                    // OR DEL OR EXTENDED ASCII
	 ELSE IF (STR[i+1] < $20 || STR[i+1] >= $7F)
	 {
									 // END WITH COMMA
		sTemp = "sTemp,FORMAT('$%02X,',STR[i])";
	 }
	 ELSE
	 {
                                    // END WITH COMMA/SINGLE QUOTE
		 sTemp = "sTemp,FORMAT('$%02X,''',STR[i])";
	 }
	}
	ELSE                               // IF IT'S PRINTABLE
	{
	 SELECT
	 {
		ACTIVE (i = 1):                 // IF IT'S THE FIRST BYTE
		{
		 IF (i = lenSTR)                // IF IT'S THE ONLY BYTE
		 {
			 sTemp = "'''',STR[i],''''";  // PUT IT IN SINGLE QUOTES
		 }
		 ELSE IF                        // ELSE IF THE NEXT BYTE IS
			((STR[i+1] < $20) ||          // A CONTROL CHARACTER OR
			 (STR[i+1] >= $7F))           // THE NEXT BYTE IS DEL OR EXTENDED ASCII
		 {
			sTemp = "'''',STR[i],''','";  // TRAILING SINGLE QUOTE/COMMA
		 }
		 ELSE
		 {
			sTemp = "'''',STR[i]";        // LEADING SINGLE QUOTE ONLY
		 }
		}
		ACTIVE (i = lenSTR):            // IF IT'S THE LAST BYTE
		{
		 sTemp = "sTemp,STR[i],''''";   // TRAILING SINGLE QUOTE ONLY
		}
		ACTIVE (STR[i+1] < $20):        // IF THE NEXT BYTE IS A CONTROL CHARACTER
		{
		 sTemp = "sTemp,STR[i],''','";  // TRAILING SINGLE QUOTE/COMMA
		}
		ACTIVE (STR[i+1] >= $7F):       // IF THE NEXT BYTE IS DEL OR EXTENDED ASCII
		{
		 sTemp = "sTemp,STR[i],''','";  // TRAILING SINGLE QUOTE/COMMA
		}
		ACTIVE (1):
		{
		 sTemp = "sTemp,STR[i]";        // JUST PRINT THE CHARACTER
		}
	 }
	}

                                    // if ASCII packet is long enough, break the loop
	IF (LENGTH_ARRAY(sTemp) >= lenPacket)
	{
	  i++;                            // return next index into str[]
	  BREAK;
	}
 }
 RETURN i;                          // i = LENGTH_ARRAY(str)+1 or next index into str[]
}
DEFINE_FUNCTION LONG __formatAsciiExt__(char STR[], long lenSTR, char sTemp[], integer lenPacket, long i)
// FUNCTION: __formatAsciiExt__()
// Purpose:
//  - Create one ASCII packet of length lenPacket from STR, without destroying STR contents
//  - Can be called recursively if STR is too long to be formatted into one packet
//  - PASSES ASCII and EXTENDED ASCII CHARACTERS UNFORMATTED, ASCII STRINGS INSIDE SINGLE-QUOTES
//  - FORMATS CONTROL CHARACTERS AS 2 DIGIT HEX, SEPERATED BY COMMAS
//  - ASCII STRINGS SEPERATED FROM HEX BY COMMAS
// PARAMETERS:
//  - STR:       data to be formatted
//  - lenSTR:    length of data to be formatted
//  - sTemp:     temp storage for formatted data
//  - lenPacket: maximum desired length of formatted data
//  - i:         current index into str[]
// RETURNS:
//  - i
{
  IF (i = 0) { i = 1 }              // NetLinx indexing starts at 1, not 0
  FOR (; i <= lenSTR; i++)
  {
									// IF IT'S A CONTROL CHARACTER
    IF (STR[i] < $20 || STR[i] = $7F)
    {
      IF (i = lenSTR)               // IF IT'S THE LAST BYTE
      {
                                    // JUST PRINT $XX
        sTemp = "sTemp,FORMAT('$%02X',STR[i])";
      }
									// IF THE NEXT BYTE IS A CONTROL CHARACTER
      ELSE IF (STR[i+1] < $20 || STR[i+1] = $7F)
      {
                                    // END WITH COMMA
        sTemp = "sTemp,FORMAT('$%02X,',STR[i])";
      }
      ELSE                          // IF THE NEXT BYTE IS ASCII
      {
                                    // END WITH COMMA/SINGLE QUOTE
        sTemp = "sTemp,FORMAT('$%02X,''',STR[i])";
      }
    }
    ELSE                            // IF IT'S PRINTABLE
    {
      SELECT
      {
        ACTIVE (i = 1):             // IF IT'S THE FIRST BYTE
        {
          IF (i = lenSTR)           // IF IT'S THE ONLY BYTE
          {
                                    // PUT IT IN SINGLE QUOTES
            sTemp = "'''',STR[i],''''";
          }
		                            // ELSE IF THE NEXT BYTE IS A CONTROL CHARACTER
		  ELSE IF ((STR[i+1] < $20) || (STR[i+1] = $7F))
		  {
								    // PUT IT IN SINGLE QUOTES FOLLOWED BY COMMA
            sTemp = "'''',STR[i],''','";
		  }
          ELSE                      // NEXT BYTE IS ASCII
          {
            sTemp = "'''',STR[i]";  // LEADING SINGLE QUOTE ONLY
          }
        }
        ACTIVE (i = lenSTR):        // IF IT'S THE LAST BYTE
        {
                                    // TRAILING SINGLE QUOTE ONLY
          sTemp = "sTemp,STR[i],''''";
        }
									// IF THE NEXT BYTE IS A CONTROL CHARACTER
        ACTIVE (STR[i+1] < $20 || STR[i+1] = $7F):
        {
                                    // TRAILING SINGLE QUOTE/COMMA
          sTemp = "sTemp,STR[i],''','";
        }
        ACTIVE (1):
        {
          sTemp = "sTemp,STR[i]";   // JUST PRINT THE CHARACTER
        }
      }
    }
									// if ASCII packet is long enough, break the loop
	IF (LENGTH_ARRAY(sTemp) >= lenPacket)
	{
	  i++;                          // return next index into str[]
	  BREAK;
	}
  }
  RETURN i;                         // i = LENGTH_ARRAY(str)+1 or next index into str[]
}
DEFINE_FUNCTION LONG __formatDecimal__(char STR[], long lenSTR, char sTemp[], integer lenPacket, long i)
// FUNCTION: __formatDecimal__()
// Purpose:
//  - Create one ASCII packet of length lenPacket from STR, without destroying STR contents
//  - Can be called recursively if STR is too long to be formatted into one packet
//  - FORMATS ALL CHARACTERS AS 3 DIGIT DECIMAL, SEPERATED BY COMMAS
// PARAMETERS:
//  - STR:       data to be formatted
//  - lenSTR:    length of data to be formatted
//  - sTemp:     temp storage for formatted data
//  - lenPacket: desired length of formatted data
//  - i:         current index into str[]
// RETURNS:
//  - i
{
  IF (i = 0) { i = 1 }              // NetLinx indexing starts at 1, not 0
  FOR (; i <= lenSTR; i++)
  {
	IF (lenSTR == 4)                // if it could be IP address info 
	{
	  sTemp = "sTemp,FORMAT('%d.',STR[i])";
	}
	ELSE
	{
	  sTemp = "sTemp,FORMAT('%03d,',STR[i])";
	}

									// if ASCII packet is long enough, break the loop
	IF (LENGTH_ARRAY(sTemp) >= lenPacket)
	{
	  i++;                          // return next index into str[]
	  BREAK;
	}
  }
  SET_LENGTH_ARRAY(sTemp,LENGTH_ARRAY(sTemp)-1);
  RETURN i;                         // i = LENGTH_ARRAY(str)+1 or next index into str[]
}
DEFINE_FUNCTION LONG __formatHex__(char STR[], long lenSTR, char sTemp[], integer lenPacket, long i)
// FUNCTION: __formatHex__()
// Purpose:
//  - Create one ASCII packet of length lenPacket from STR, without destroying STR contents
//  - Can be called recursively if STR is too long to be formatted into one packet
//  - FORMATS ALL CHARACTERS AS 2 DIGIT HEX, SEPERATED BY COMMAS
// ASCII STRINGS SEPERATED FROM HEX BY COMMAS
// PARAMETERS:
//  - STR:       data to be formatted
//  - lenSTR:    length of data to be formatted
//  - sTemp:     temp storage for formatted data
//  - lenPacket: desired length of formatted data
//  - i:         current index into str[]
// RETURNS:
//  - i
{
 IF (i = 0) { i = 1 }              // NetLinx indexing starts at 1, not 0
 FOR (; i <= lenSTR; i++)
 {
	sTemp = "sTemp,FORMAT('$%02X,',STR[i])";

									// if ASCII packet is long enough, break the loop
	IF (LENGTH_ARRAY(sTemp) >= lenPacket)
	{
	 i++;                             // return next index into str[]
	 BREAK;
	}
 }
 IF (i >= lenSTR)                   // if it's the last byte, remove the last trailing comma added in the FOR loop
 {
	SET_LENGTH_ARRAY(sTemp,LENGTH_ARRAY(sTemp)-1);
 }
 RETURN i;                         	// i = LENGTH_ARRAY(str)+1 or next index into str[]
}
DEFINE_FUNCTION LONG __formatMIDI__(char STR[], long lenSTR, char sTemp[], integer lenPacket, integer nMode, integer s, long i)
// FUNCTION: __formatMIDI__()
// Purpose:
//  - Create one ASCII packet of length lenPacket from STR, without destroying STR contents
//  - Can be called recursively if STR is too long to be formatted into one packet
//  - Formats MIDI channel voice messages with:
//              MIDI status bytes as 2 digit hex, seperated by commas.
//              MIDI data bytes as 3 digit decimal, seperated by commas.
//  - Formats MIDI system messages as 2 digit hex, seperated by commas.
// PARAMETERS:
//  - STR:       data to be formatted
//  - lenSTR:    length of data to be formatted
//  - sTemp:     temp storage for formatted data
//  - lenPacket: desired length of formatted data
//  - s:         current MIDI status byte:
//                $EF and lower = default = format status bytes as hex, data bytes as decimal.
//                $F0 and higher = format status and data as hex.
//  - i:        current index into str[]
//  - nMode:    bitmapped flags
//               $1 = print MIDI SYSEX as ASCII data, i.e. unformatted
//               $2 = reserved (used by printMidiDebug) 
// RETURNS:
//  - i
{
 STACK_VAR INTEGER lenSysEx_Header
 lenSysEx_Header = 4                // "$F0" + 3 bytes for typical Midi Mfg. ID
/*  Note: Japanese MMID may be 1 byte, but the other 2 bytes are used for MIDI channel &
 *        a device ID (at least with Roland and Yamaha).
 *        e.g.
 *  char SYSX_ID_FC[]   = { $41, $00, $72 }    // FC-200 = { ROLAND, CH1, FC-200 }
 *                            
 *  char SYSX_ID_G50[]  = { $43, $10, $56 }    // G50    = { YAMAHA, CH1 + $10, G50 }
 *                            
 *  char SYSX_ID_MU80[] = { $43, $10, $49 }    // MU80   = { YAMAHA, CH1 + $10, MU80 }
 *                            
 *  char SYSX_ID_MU90[] = { $43, $10, $59 }    // MU90   = { YAMAHA, CH1 + $10, MU90 }
 *                            
 *  char SYSX_ID_XG[]   = { $43, $10, $4C }    // XG     = { YAMAHA, CH1 + $10, XG }
 */
 FOR (; i <= lenSTR; i++)
 {
	IF (s & $F0 == $F0)               // if it's a MIDI SYSTEM message
	{
	 IF (s = $F0)                     // if it's a SysEx message
	 {
		IF (nMode & BMP_MODE_MIDI_SYSEX_ASCII)
		{
		 IF (i <= lenSysEx_Header)      // Print $F0 and MMID as hex
		 {
			IF (i == lenSysEx_Header)
			{
                                    // IF THE NEXT BYTE IS A CONTROL CHARACTER
                                    // OR DEL OR EXTENDED ASCII
			 IF (STR[i+1] < $20 || STR[i+1] >= $7F)
			 {
                                    // END WITH COMMA
				 sTemp = "sTemp,FORMAT('$%02X,',STR[i])";
			 }
			 ELSE
			 {
                                    // END WITH COMMA/SINGLE QUOTE
				 sTemp = "sTemp,FORMAT('$%02X,''',STR[i])";
			 }
			}
			ELSE IF (i == lenSTR)         // IF IT'S THE LAST BYTE
			{
			 sTemp = "sTemp,FORMAT('$%02X',STR[i])";
			}
			ELSE
			{
			 sTemp = "sTemp,FORMAT('$%02X,',STR[i])";
			}
		 }
		 ELSE                           // format SysEx data a la __formatAsciiHex__()
		 {
									// IF IT'S A CONTROL CHARACTER
									// OR DEL OR EXTENDED ASCII
			IF (STR[i] < $20 || STR[i] >= $7F)
			{
			 IF (i = lenSTR)              // IF IT'S THE LAST BYTE
			 {
											 // JUST PRINT $XX
				 sTemp = "sTemp,FORMAT('$%02X',STR[i])";
			 }
                                    // IF THE NEXT BYTE IS A CONTROL CHARACTER
                                    // OR DEL OR EXTENDED ASCII
			 ELSE IF (STR[i+1] < $20 || STR[i+1] >= $7F)
			 {
                                    // END WITH COMMA
				 sTemp = "sTemp,FORMAT('$%02X,',STR[i])";
			 }
			 ELSE
			 {
                                    // END WITH COMMA/SINGLE QUOTE
				 sTemp = "sTemp,FORMAT('$%02X,''',STR[i])";
			 }
			}
			ELSE                          // IF IT'S PRINTABLE
			{
			 SELECT
			 {
				 ACTIVE (i = 1):            // IF IT'S THE FIRST BYTE
				 {
				 IF (i = lenSTR)            // IF IT'S THE ONLY BYTE
				 {
											 // PUT IT IN SINGLE QUOTES
					 sTemp = "'''',STR[i],''''";
				 }
				 ELSE IF                    // ELSE IF THE NEXT BYTE IS
						((STR[i+1] < $20) ||    // A CONTROL CHARACTER OR
						 (STR[i+1] >= $7F))     // THE NEXT BYTE IS DEL OR EXTENDED ASCII
				 {
                                    // PUT IT IN SINGLE QUOTES FOLLOWED BY COMMA
					 sTemp = "'''',STR[i],''','";
				 }
				 ELSE
				 {
					 sTemp = "'''',STR[i]";   // LEADING SINGLE QUOTE ONLY
				 }
				 }
				 ACTIVE (i = lenSTR):       // IF IT'S THE LAST BYTE
				 {
											 // TRAILING SINGLE QUOTE ONLY
				 sTemp = "sTemp,STR[i],''''";
				 }
				 ACTIVE (STR[i+1] < $20):   // IF THE NEXT BYTE IS A CONTROL CHARACTER
				 {
											 // TRAILING SINGLE QUOTE/COMMA
				 sTemp = "sTemp,STR[i],''','";
				 }
				 ACTIVE (STR[i+1] >= $7F):  // IF THE NEXT BYTE IS DEL OR EXTENDED ASCII
				 {
											 // TRAILING SINGLE QUOTE/COMMA
				 sTemp = "sTemp,STR[i],''','";
				 }
				 ACTIVE (1):
				 {
				 sTemp = "sTemp,STR[i]";    // JUST PRINT THE CHARACTER
				 }
			 }
			}
		 }
		}
		ELSE                            // print SYSEX as hex
		{
		 IF (i = lenSTR)                // IF IT'S THE LAST BYTE
		 {
			sTemp = "sTemp,FORMAT('$%02X',STR[i])";
		 }
		 ELSE
		 {
			sTemp = "sTemp,FORMAT('$%02X,',STR[i])";
		 }
		}
	 }
	 ELSE                             // print all other MIDI SYSTEM messages as hex
	 {
		IF (i = lenSTR)                 // IF IT'S THE LAST BYTE
		{
		 sTemp = "sTemp,FORMAT('$%02X',STR[i])";
		}
		ELSE
		{
		 sTemp = "sTemp,FORMAT('$%02X,',STR[i])";
		}
	 }
	}
	ELSE                              // Print status voice messages
	{
	 IF (i = lenSTR)                  // IF IT'S THE LAST BYTE
	 {
		IF (STR[i] & $80) sTemp = "sTemp,FORMAT('$%02X',STR[i])";
		ELSE              sTemp = "sTemp,FORMAT('%03d',STR[i])";
	 }
	 ELSE
	 {
		IF (STR[i] & $80) sTemp = "sTemp,FORMAT('$%02X,',STR[i])";
		ELSE              sTemp = "sTemp,FORMAT('%03d,',STR[i])";
	 }
	}

                                    // if ASCII packet is long enough, break the loop
	IF (LENGTH_ARRAY(sTemp) >= lenPacket)
	{
	 i++;                             // return next index into str[]
	 BREAK;
	}
 }
 RETURN i;                          // i = LENGTH_ARRAY(str)+1 or next index into str[]
}
DEFINE_FUNCTION CHAR[64] __sURLFlags__ (INTEGER Flg)
{
  STACK_VAR CHAR sReturn[64]
  IF (flg & URL_Flg_Stat_PrgNetLinx) { sReturn = 'Src-NetLinx' }
  ELSE                               { sReturn = 'Src-IDE' }

  IF (flg & URL_Flg_TCP) { sReturn = "sReturn,' TCP'" }
  ELSE                   { sReturn = "sReturn,' UDP'" }

  IF (flg & URL_Flg_Temp) { sReturn = "sReturn,' Temp'" }
  ELSE                    { sReturn = "sReturn,' Perm'" }

  SWITCH (flg & URL_Flg_Stat_Mask)
  {
	CASE URL_Flg_Stat_Lookup:     sReturn = "sReturn,' Stat-Lookup'"
	CASE URL_Flg_Stat_Connecting: sReturn = "sReturn,' Stat-Connecting'"
	CASE URL_Flg_Stat_Waiting:    sReturn = "sReturn,' Stat-Waiting'"
	CASE URL_Flg_Stat_Connected:  sReturn = "sReturn,' Stat-Connected'"
  }

  IF (Flg & URL_Flg_AcctInfoPresent) { sReturn = "sReturn,' AcctInfoPresent'" }
  RETURN sReturn
}

DEFINE_PROGRAM                      // older debug calls (deprecated v3.xx)
DEFINE_CALL 'SEND 131 BYTE PACKETS TO MASTER' (CHAR STR1[15999])
/* v3.00 - these old calls are now just wrappers for newer print<> functions */
// DEFINE_CALL 'SEND 131 BYTE PACKETS TO MASTER'
// BREAKS STRINGS UP INTO THE MAXIMUM SIZE
// THAT CAN BE DISPLAYED IN A MASTER TERMINAL,
// SENDS THEM TO THE MASTER.
// PARAMETER:
// STR1  - STRING TO BE SENT TO MASTER
{
  printToDebug(0:1:0,'',STR1,131,0,PRINT_UNFORMATTED)
}
DEFINE_CALL 'SEND 7BIT ASCII TO DEBUG' (DEV dvDEBUG,CHAR STR1[],CHAR STR2[15999],INTEGER nLINE,INTEGER nMODE)
/* v3.00 - these old calls are now just wrappers for newer print<> functions */
{
  printToDebug(dvDEBUG,STR1,STR2,nLine,nMode,PRINT_ASCII_HEX)
}
DEFINE_CALL 'SEND ASCII/HEX TO DEBUG' (DEV dvDEBUG,CHAR STR1[],CHAR STR2[15999],INTEGER nLINE,INTEGER nMODE)
/* v3.00 - these old calls are now just wrappers for newer print<> functions */
{
  printToDebug(dvDEBUG,STR1,STR2,nLine,nMode,PRINT_ASCII_HEX)
}
DEFINE_CALL 'SEND ASCII TO DEBUG' (DEV dvDEBUG,CHAR STR1[],CHAR STR2[15999],INTEGER nLINE,INTEGER nMODE)
/* v3.00 - these old calls are now just wrappers for newer print<> functions */
{
  printToDebug(dvDEBUG,STR1,STR2,nLine,nMode,PRINT_ASCII_EXT)
}
DEFINE_CALL 'SEND ASCII TO MASTER' (CHAR STR1[],CHAR STR2[15999])
/* v3.00 - these old calls are now just wrappers for newer print<> functions */
{
  printToDebug(0:1:0,STR1,STR2,131,0,PRINT_ASCII_EXT)
}
DEFINE_CALL 'SEND DECIMAL TO DEBUG' (DEV dvDEBUG,CHAR STR1[],CHAR STR2[15999],INTEGER nLine,INTEGER nMode)
/* v3.00 - these old calls are now just wrappers for newer print<> functions */
{
  printToDebug(dvDEBUG,STR1,STR2,nLine,nMode,PRINT_DECIMAL)
}
DEFINE_CALL 'SEND INTEGER TO DEBUG' (DEV dvDEBUG,CHAR STR1[],CHAR STR2[15999],INTEGER nLine,INTEGER nMode)
/* v3.00 - these old calls are now just wrappers for newer print<> functions */
{
  printToDebug(dvDEBUG,STR1,STR2,nLine,nMode,PRINT_UNFORMATTED)
}
DEFINE_CALL 'SEND ITOHEX TO DEBUG' (DEV dvDEBUG,CHAR STR1[],CHAR STR2[15999],INTEGER nLine,INTEGER nMode)
/* v3.00 - these old calls are now just wrappers for newer print<> functions */
{
  printToDebug(dvDEBUG,STR1,STR2,nLine,nMode,PRINT_HEX)
}
DEFINE_CALL 'SEND ITOHEX TO MASTER' (CHAR STR1[],CHAR STR2[15999])
/* v3.00 - these old calls are now just wrappers for newer print<> functions */
{
  printToDebug (0:1:0,STR1,STR2,131,0,PRINT_HEX)
}
DEFINE_CALL 'SEND MIDI TO DEBUG' (DEV dvDEBUG,CHAR STR1[],CHAR STR2[15999],INTEGER nLINE,INTEGER nMODE, INTEGER nSTATUS)
/* v3.00 - these old calls are now just wrappers for newer print<> functions */
{
  printMidiDebug(dvDEBUG,STR1,STR2,nLINE,nMODE,nSTATUS)
}
DEFINE_CALL 'SEND CUSTOM_EVENT DATA TO DEBUG' (DEV dvDebug)
/* v3.00 - these old calls are now just wrappers for newer print<> functions */
{
  printCUSTOM_EVENT (dvDebug,'',0,0)
}
DEFINE_CALL 'SEND DEVICE_INFO TO DEBUG' (DEV dvDEBUG, DEV dvDEV)
/* v3.00 - these old calls are now just wrappers for newer print<> functions */
{
  printDEVICE_INFO (dvDEBUG,'',dvDEV,0)
}

(***********************************************************)
(*                 STARTUP CODE GOES BELOW                 *)
(***********************************************************)
DEFINE_START

IF (!__V4_and_NX_check__)
{
  // Declare
  STACK_VAR DEV_INFO_STRUCT uDevInfo
  STACK_VAR CHAR    cVer[50]
  STACK_VAR CHAR    cTemp[50]
  STACK_VAR INTEGER nMajor
  STACK_VAR INTEGER nMinor
  STACK_VAR INTEGER nBuild

  // Check master version
  DEVICE_INFO(0:0:0,uDevInfo)
  cVer = uDevInfo.VERSION
  cTemp = REMOVE_STRING(cVer,'.',1)
  nMajor = ATOI(cTemp)
  cTemp = REMOVE_STRING(cVer,'.',1)
  nMinor = ATOI(cTemp)
  nBuild = ATOI(cVer)

  IF (uDevInfo.DEVICE_ID >= $18C)   // NX-x200 master or newer
	{
	 __V4_or_NX_master__ = $500+nMajor// 1280 + nMajor
	}
	ELSE IF (nMajor >= 4)             // Older master v4.xx.xx
  {
	 __V4_or_NX_master__ = nMajor
  }
  ELSE                              // Older master v3.xx.xx, v2.xx.xx
  {
	 __V4_or_NX_master__ = FALSE
  }
	__V4_and_NX_check__ = TRUE
}

(***********************************************************)
(*                THE EVENTS GOES BELOW                    *)
(***********************************************************)
DEFINE_EVENT

DATA_EVENT[0:1:0]
{
 ONLINE:
 {
	IF (__display_lib_version__ == TRUE)
	{
	 wait 50                          // WAIT FOR DEVICE HOLDOFF
	 {
		// WHAT VERSION?
		logString(0:1:0,AMX_ERROR,"'Using lib ',__DEBUG_LIB_NAME__,
																			', v',__DEBUG_LIB_VERSION__")
	 }
	}
 }
}

(***********************************************************)
(*            THE ACTUAL PROGRAM GOES BELOW                *)
(***********************************************************)
DEFINE_PROGRAM                      // #END_IF compiler directive

#END_IF
(***********************************************************)
(*                     END OF PROGRAM                      *)
(*        DO NOT PUT ANY CODE BELOW THIS COMMENT           *)
(***********************************************************)
