PROGRAM_NAME='file'
(***********************************************************)
(*  FILE CREATED ON: 08/08/2008  AT: 15:12:15              *)
(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 03/03/2023  AT: 09:38:06        *)
(***********************************************************)
(*  FILE REVISION: 4.02                                    *)
(*  REVISION DATE: 03/03/2023  AT: 09:38:06                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Removed 15999 character limit from all function      *)
(*    parameters.                                          *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 4.01                                    *)
(*  REVISION DATE: 12/27/2022  AT: 18:06:42                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Use AMX_LOG() when master supports it.               *)
(*  - file.axi has its own copy of logString(), as exists  *)
(*    in debug.axi.                                        *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 1.06/1.05                               *)
(*  REVISION DATE: 07/17/2016  AT: 10:03:00                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - __FILE_LIB_NAME__ now lines up with other libs.      *)
(*  - Removed <CR><LF> from debug strings not to a master. *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: Rev 1.04                                *)
(*  REVISION DATE: 03/15/2013  AT: 16:00:26                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added MKDIR() function.                              *)
(*  - Removed the clunky trailing slash from the directory *)
(*    printout in LS() and DIR().                          *)
(*  - Modified LS() and DIR() so they can use the current  *)
(*    working directory.                                   *)
(*  - modified ECHO() so it uses the full path to create   *)
(*    the file, not a seperate directory and filename.     *)
(*  - added RM() function to verbosely remove a file or    *)
(*    directory.                                           *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 1.03.01                                 *)
(*  REVISION DATE: 10/10/2011  AT: 17:44:37                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Comments cleanup only:                               *)
(*    Added successful RETURN value = 0 for CD(), ECHO(),  *)
(*    and PWD().                                           *)
(*  - Added comments indicating LS() features in DIR().    *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 1.03                                    *)
(*  REVISION DATE: 03/24/2011  AT: 16:12:18                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added <CR><LF> to debug strings when they are not to *)
(*    a master debug port.                                 *)
(*  - dir() now behaves like ls() if no filename is        *)
(*    specified.                                           *)
(*  - dir() returns file# instead of TRUE if file is       *)
(*    found.                                               *)
(*  - ls() now prints file# and directory with filenames.  *)
(*  - dir() and ls() print a ':' between the directory and *)
(*    filename: <file#>:<dir>:<filename>.                  *)
(*    Example:                                             *)
(*        1:/:/images                                      *)
(*        2:/:LOGFILE_Y11M3D24H8.TXT                       *)
(*        3:/:LOGFILE_Y11M3D24H9.TXT                       *)
(*        4:/:LOGFILE_Y11M3D24H12.TXT                      *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 1.02                                    *)
(*  REVISION DATE: 10/06/2008  AT: 08:01:46                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added __display_lib_version__ variable.              *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 1.01                                    *)
(*  REVISION DATE: 08/19/2008  AT: 19:41:07                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - printFileResult(), cd(), and pwd() now return SLONG  *)
(*    error values.                                        *)
(*  - Added dir() function.                                *)
(*  - Added echo() function.                               *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 1.00                                    *)
(*  REVISION DATE: 08/14/2008  AT: 16:11:37                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Version change to 1.00.                              *)
(*  - Increased printable filename size to 131 in LS().    *)
(*  - Added HEADER parameter to printFileResult().         *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 0.22, 0.21                              *)
(*  REVISION DATE: 08/11/2008  AT: 18:05:17                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Strings printed by cd(), ls(), and pwd() are of the  *)
(*    form <msg>:<data>.                                   *)
(*    For example:                                         *)
(*      good pwd() prints 'pwd:<directory name>'           *)
(*      fail pwd() prints 'pwd error:<error details>'      *)
(*    The exception: filenames printed by ls().            *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 0.20                                    *)
(*  REVISION DATE: 08/08/2008  AT: 15:20:31                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - new approach                                         *)
(***********************************************************)
(*  FILE REVISION: 0.10 GLM                                *)
(*  REVISION DATE: 06/30/2003                              *)
(*                                                         *)
(*  COMMENTS:                                              *)
(* FILE MANAGEMENT FUNCTIONS AND CALLS                     *)
/*  0   FUNCTION logString (DEV dvDebug, LONG lLogLevel, CHAR str[])         */
/*  1   FUNCTION CHAR[61] __FILE_RESULT__ (slong slResult)                   */
/*  2   FUNCTION printFileResult (dvD, hdr[], err, fNonError)                */
/*  3   FUNCTION CD (dvD, d1[], d2[])                                        */
/*  4   FUNCTION DIR (dvD, d[], f[])                                         */
/*  5   FUNCTION ECHO (dvD, d[], f[], data[])                                */
/*  6   FUNCTION LS (dvD, d[])                                               */
/*  7   FUNCTION MKDIR (dvD, d[])                                            */
/*  8   FUNCTION PWD (dvD, d[])                                              */
/*  9   FUNCTION RM (dvD, p[])                                               */
(***********************************************************)
#IF_NOT_DEFINED __FILE_LIB__
#DEFINE __FILE_LIB__
/*
 * tab-width: 2
 * indent-level: 1
 *
 */
(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

// VERSION FOR CODE

CHAR __FILE_LIB_NAME__[20]      = '            file.axi';
CHAR __FILE_LIB_VERSION__[]     = '4.02';

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

#IF_NOT_DEFINED __display_lib_version__
VOLATILE __display_lib_version__ = TRUE
#END_IF

#IF_NOT_DEFINED __V4_or_NX_master__
VOLATILE INTEGER __V4_or_NX_master__ = 0
#END_IF

#IF_NOT_DEFINED __V4_and_NX_check__
VOLATILE INTEGER __V4_and_NX_check__ = 0
#END_IF

(***********************************************************)
(*        SUBROUTINE/FUNCTION DEFINITIONS GO BELOW         *)
(***********************************************************)
(* EXAMPLE: DEFINE_FUNCTION <RETURN_TYPE> <NAME> (<PARAMETERS>) *)
(* EXAMPLE: DEFINE_CALL '<NAME>' (<PARAMETERS>) *)

#IF_NOT_DEFINED __logString__       // copy of logString() from debug.axi
#DEFINE __logString__
DEFINE_FUNCTION logString(DEV dvDebug, LONG lLogLevel, CHAR str[])
{
 IF (dvDebug.NUMBER == 0)           // if dvDebug is a master...
 {
  IF (__V4_or_NX_master__)          // and if it's a newer NetLinx master
  {
	 AMX_LOG(lLogLevel,str)
  }
  ELSE                              // for older masters, send_string 0
  {
	 SEND_STRING dvDebug,str
  }
 }
 ELSE
 {
  SEND_STRING dvDebug,str
 }
}
#END_IF

DEFINE_FUNCTION CHAR[61] __FILE_RESULT__  (SLONG slResult)
// function name: __FILE_RESULT__(slResult)
// purpose      : provide debugging info for FILE_???? functions
// parameter    : slResult - result from FILE_???? function
// returns      : result as a string, with error message if any
{
 STACK_VAR LONG ERR_ABS;
 STACK_VAR CHAR sRESULT[61];

 sRESULT = "FORMAT('%d',slResult)";

 IF (slResult < 0)                  // if result is an error
 {
	ERR_ABS = ABS_VALUE(slResult);
	SWITCH (ERR_ABS)
	{
		CASE 1:     sRESULT = "sRESULT,' invalid file handle'";
		CASE 2:     sRESULT = "sRESULT,' invalid file path or name'";
		CASE 3:     sRESULT = "sRESULT,' invalid IOFlag value'";
		CASE 4:     sRESULT = "sRESULT,' invalid directory path'";
		CASE 5:     sRESULT = "sRESULT,' disk I/O error'";
		CASE 6:     sRESULT = "sRESULT,' invalid parameter - too large or 0'";
		CASE 7:     sRESULT = "sRESULT,' file already closed'";
		CASE 8:     sRESULT = "sRESULT,' file name exists'";
		CASE 9:     sRESULT = "sRESULT,' End-Of-File reached'";
		CASE 10:    sRESULT = "sRESULT,' buffer too small'";
		CASE 11:    sRESULT = "sRESULT,' disk full'";
		CASE 12:    sRESULT = "sRESULT,' directory not loaded - empty or does not exist?'";
		CASE 13:    sRESULT = "sRESULT,' directory name exists'";
		DEFAULT:    sRESULT = "sRESULT,' unknown error'";
	}
 }

 RETURN sRESULT;
}

DEFINE_FUNCTION SLONG printFileResult (DEV dvD, CHAR hdr[], SLONG err, INTEGER fPNE)
// function: printFileResult (dvD, err, fPrintNonError)
// purpose: prints result from FILE_??? functions
// parameter: dvD   - destination for function output
// parameter: hdr   - header prepended to result, if desired
// parameter: err   - result from FILE_???? function
// parameter: flag  - Print Non Error Result?
// returns: err
{
 IF (err < 0)                       // if result is an error
 {
	logString(dvD,AMX_ERROR,"hdr,__FILE_RESULT__(err)")
 }
 ELSE IF (fPNE <> FALSE)            // print non-error result?
 {
	logString(dvD,AMX_ERROR,"hdr,__FILE_RESULT__(err)")
 }
 RETURN err
}

DEFINE_FUNCTION SLONG cd(DEV dvD, CHAR d1[], CHAR d2[])
// function: CD (dvD, d1[], d2[])
// purpose: change working directory, print error if any
// parameter: dvD	- destination for function output
// parameter: d1	- directory buffer
// parameter: d2	- new dir
// return: err      - If successful, err = 0
{
  STACK_VAR SLONG   err             // Error

  err = FILE_SETDIR(d2)

  IF (err < 0)
  {
	logString(dvD,AMX_ERROR,"'cd error:',__FILE_RESULT__(err)")
  }
  ELSE
  {
	FILE_GETDIR(d1)    // this will copy the successful cd() to d1, with '/' prepended if necessary
  }
  RETURN err           // success = 0
}

DEFINE_FUNCTION INTEGER dir(DEV dvD, CHAR d[], CHAR f[])
// function: dir (dvD, d[], f[])
// purpose: A) search for filename f in path d
//             prints file found and returns TRUE if found
//             returns FALSE if not found
//          B) if f = '', dir() functions like ls():
//                        dir() prints list of files in directory, or error if an error occurs
// parameter: f		- filename for dir
//                    If f = '', dir() functions like ls()
// parameter: d		- directory for dir
// parameter: dvD	- destination for function output
//					  If dvD is 0:1:0, function will print to terminal.
//                    If dvD is virtual, DATA_EVENT/STRING: handler for
//                    dvD can be written to parse the data.
// returns: FileNumber if filename found, FALSE if not found
{
 STACK_VAR CHAR    name[131]       // FileName
 STACK_VAR INTEGER nf              // Number of Files
 STACK_VAR SLONG   err             // Error
 STACK_VAR CHAR    dTmp[255]

 IF (LENGTH_ARRAY(d))
 {
	dTmp = d
 }
 ELSE                              // if d isn't specified, use the current working d
 {
	err = FILE_GETDIR(dTmp)
	printFileResult(dvD,
									"'ls(FILE_GETDIR( ',dTmp,' )) error:'",
									err,FALSE)
 }

 nf    = 1
 err   = 1

 logString(dvD,AMX_ERROR,"'dir:',dTmp,':',f")

 WHILE (err > 0)
 {
	err = FILE_DIR (dTmp, name, nf)    // nf must start at 1

	IF (err >= 0)
	{
	 IF (LENGTH_ARRAY(f))
	 {
		IF (UPPER_STRING(name) == UPPER_STRING(f))
		{
		 logString(dvD,AMX_ERROR,"ITOA(nf),':',dTmp,':',name")

		 RETURN nf                 // if file is found, exit function
		 BREAK
		}
		ELSE IF (err = 0)
		{
		 logString(dvD,AMX_ERROR,"'No such file or directory'")
		}
	 }
	 ELSE                          // if not dir-ing a particular file, list directory contents
	 {
		logString(dvD,AMX_ERROR,"ITOA(nf),':',dTmp,':',name")
	 }
	 nf++
	}
	ELSE
	{
	 logString(dvD,AMX_ERROR,"'dir ',dTmp,':',f,' error:',__FILE_RESULT__(err)")
	}
 }
 nf--

 IF (!LENGTH_ARRAY(f))
 {
	logString(dvD,AMX_ERROR,"'dir ',dTmp,' items:',ITOA(nf)")
	RETURN nf
 }
 ELSE
 {
	RETURN FALSE
 }
}

DEFINE_FUNCTION SLONG echo (DEV dvD, CHAR p[], char data[])
// function: echo (dvD, p[], data[])
// purpose: open/creates file at path p, echoes data into file
//          returns error, if necessary
// parameter: data  - data to write to file
// parameter: p		- path to file
// parameter: dvD	- destination for function output
//					  If dvD is 0:1:0, function will print to terminal.
//                    If dvD is virtual, DATA_EVENT/STRING: handler for
//                    dvD can be written to parse the data.
// returns: err     - If successful, err = 0 (file closed)
{
 STACK_VAR SLONG   h                // file Handle
 STACK_VAR SLONG   err              // Error

 err = printFileResult(dvD,
											 "'echo(FILE_WRITE( ',p,' )) error:'",
											 FILE_OPEN(p,FILE_RW_NEW),FALSE)
 IF (err > 0)
 {
	h = err
	printFileResult(dvD,"'echo(FILE_WRITE( ',p,' )) error:'",
												FILE_WRITE(h,data,LENGTH_ARRAY(data)),FALSE)

	err = FILE_CLOSE(h)
 }

 RETURN err                         // success = 0
}

DEFINE_FUNCTION INTEGER ls(DEV dvD, CHAR d[])
// function: LS (dvD, d[])
// purpose: prints list of files in directory, or error if an error occurs
// parameter: d	    - directory to LS
// parameter: dvD   - destination for function output
//                    If dvD is 0:1:0, function will print to terminal.
//                    If dvD is virtual, DATA_EVENT/STRING: handler for
//                    dvD can be written to parse the data.
// returns: number of items in directory
{
 STACK_VAR CHAR    name[131]        // FileName
 STACK_VAR INTEGER nf               // Number of Files
 STACK_VAR SLONG   err              // Error
 STACK_VAR CHAR    dTmp[255]

 IF (LENGTH_ARRAY(d))
 {
	dTmp = d
 }
 ELSE                               // if d isn't specified, use the current working d
 {
	err = FILE_GETDIR(dTmp)
	printFileResult(dvD,
									"'ls(FILE_GETDIR( ',dTmp,' )) error:'",
									err,FALSE)
 }

 nf    = 1
 err   = 1

 logString(dvD,AMX_ERROR,"'ls:',dTmp")

 WHILE (err > 0)
 {
	err = FILE_DIR (dTmp, name, nf) // nf must start at 1

	IF (err >= 0)
	{
	 logString(dvD,AMX_ERROR,"ITOA(nf),':',dTmp,':',name")
	 nf++
	}
	ELSE
	{
	 logString(dvD,AMX_ERROR,"'ls ',dTmp,' error:',__FILE_RESULT__(err)")
	}
 }
 nf--

 logString(dvD,AMX_ERROR,"'ls ',dTmp,' items:',ITOA(nf)")

 RETURN nf
}

DEFINE_FUNCTION SLONG mkdir(DEV dvD, CHAR p[])
{
// function: MKDIR (dvD, p[])
// purpose: create directory
// parameter: p     - path name of the directory to create
// parameter: dvD   - destination for function output
//                    If dvD is 0:1:0, function will print to terminal.
//                    If dvD is virtual, DATA_EVENT/STRING: handler for
//                    dvD can be written to parse the data.
// return: err      - If successful, err = 0
 RETURN printFileResult(dvD,"'mkdir( ',p,' ) error:'",FILE_CREATEDIR(p),FALSE)
}

DEFINE_FUNCTION SLONG pwd(DEV dvD, CHAR d[])
// function: PWD (dvD, d[])
// purpose: print working directory, or error if any
// parameter: d     - storage for working dir
// parameter: dvD   - destination for function output
//                    If dvD is 0:1:0, function will print to terminal.
//                    If dvD is virtual, DATA_EVENT/STRING: handler for
//                    dvD can be written to parse the data.
// return: err      - If successful, err = 0
{
 STACK_VAR SLONG   err
 err = FILE_GETDIR(d)

 IF (err < 0)
 {
	logString(dvD,AMX_ERROR,"'pwd error:',__FILE_RESULT__(err)")
 }
 ELSE
 {
	logString(dvD,AMX_ERROR,"'pwd:',d")
 }

 RETURN err         // success = 0
}

DEFINE_FUNCTION SLONG rm(DEV dvD, CHAR p[])
{
// function: RM (dvD, p[])
// purpose: Remove file or directory
// parameter: p     - path name of the file or directory to delete
// parameter: dvD   - destination for function output
//                    If dvD is 0:1:0, function will print to terminal.
//                    If dvD is virtual, DATA_EVENT/STRING: handler for
//                    dvD can be written to parse the data.
// return: err      - If successful, err = 0
 RETURN printFileResult(dvD,"'rm( ',p,' ) error:'",FILE_DELETE(p),TRUE)
}

(***********************************************************)
(*                 STARTUP CODE GOES BELOW                 *)
(***********************************************************)
DEFINE_START

IF (!__V4_and_NX_check__)
{
  // Declare
  STACK_VAR DEV_INFO_STRUCT uDevInfo
  STACK_VAR CHAR    cVer[50]
  STACK_VAR CHAR    cTemp[50]
  STACK_VAR INTEGER nMajor
  STACK_VAR INTEGER nMinor
  STACK_VAR INTEGER nBuild

  // Check master version
  DEVICE_INFO(0:0:0,uDevInfo)
  cVer = uDevInfo.VERSION
  cTemp = REMOVE_STRING(cVer,'.',1)
  nMajor = ATOI(cTemp)
  cTemp = REMOVE_STRING(cVer,'.',1)
  nMinor = ATOI(cTemp)
  nBuild = ATOI(cVer)

  IF (uDevInfo.DEVICE_ID >= $18C)   // NI-x200 master or newer
	{
	 __V4_or_NX_master__ = $500+nMajor// 1280 + nMajor
	}
	ELSE IF (nMajor >= 4)             // Older master v4.xx.xx
  {
	 __V4_or_NX_master__ = nMajor
  }
  ELSE                              // Older master v3.xx.xx, v2.xx.xx
  {
	 __V4_or_NX_master__ = FALSE
  }
	__V4_and_NX_check__ = TRUE
}

(***********************************************************)
(*                THE EVENTS GOES BELOW                    *)
(***********************************************************)
DEFINE_EVENT

DATA_EVENT[0:1:0]
{
 ONLINE:
 {
	IF (__display_lib_version__ == TRUE)
	{
	 wait 50                          // WAIT FOR DEVICE HOLDOFF
	 {
		// WHAT VERSION?
		 logString(0:0:0,AMX_ERROR,"'Using lib ',__FILE_LIB_NAME__,
																			 ', v',__FILE_LIB_VERSION__")
	 }
	}
 }
}

#END_IF
(***********************************************************)
(*                     END OF PROGRAM                      *)
(*        DO NOT PUT ANY CODE BELOW THIS COMMENT           *)
(***********************************************************)
